@@ -9,14 +9,15 @@ use Getopt::Long;
 use File::Spec;
 use Cwd;
 use lib 'tools/lib';
-use NQP::Configure qw(sorry slurp cmp_rev gen_nqp read_config 
+use NQP::Configure qw(sorry slurp cmp_rev gen_nqp read_config
                       fill_template_text fill_template_file
                       system_or_die verify_install);
 
 my $lang = 'Rakudo';
 my $lclang = lc $lang;
 my $uclang = uc $lang;
-my $slash  = $^O eq 'MSWin32' ? '\\' : '/';
+my $win    = $^O eq 'MSWin32';
+my $slash  = $win ? '\\' : '/';
 
 
 MAIN: {
@@ -32,13 +33,13 @@ MAIN: {
 
     my %options;
     GetOptions(\%options, 'help!', 'prefix=s',
-                'backends=s', 'no-clean!',
+               'sysroot=s', 'sdkroot=s',
+               'backends=s', 'no-clean!',
                'gen-nqp:s',
                'gen-moar:s', 'moar-option=s@',
                'git-protocol=s',
-               'gen-parrot:s', 'parrot-option=s@',
-               'parrot-make-option=s@',
                'make-install!', 'makefile-timing!',
+               'git-depth=s', 'git-reference=s',
     ) or do {
         print_help();
         exit(1);
@@ -50,27 +51,38 @@ MAIN: {
         exit(0);
     }
 
-    $options{prefix} ||= 'install';
+    unless (defined $options{prefix}) {
+	my $default = defined($options{sysroot}) ? '/usr' : File::Spec->catdir(getcwd, 'install');
+        print "ATTENTION: no --prefix supplied, building and installing to $default\n";
+        $options{prefix} = $default;
+    }
     $options{prefix} = File::Spec->rel2abs($options{prefix});
     my $prefix         = $options{'prefix'};
-    my %known_backends = (parrot => 1, jvm => 1, moar => 1);
+    my @known_backends = qw/moar jvm/;
+    my %known_backends = map { $_, 1; } @known_backends;
     my %letter_to_backend;
     my $default_backend;
     for (keys %known_backends) {
         $letter_to_backend{ substr($_, 0, 1) } = $_;
     }
+    my @backends;
     my %backends;
     if (defined $options{backends}) {
-        $options{backends} = join ",", keys %known_backends
+        $options{backends} = join ",", @known_backends
             if uc($options{backends}) eq 'ALL';
         for my $b (split /,\s*/, $options{backends}) {
             $b = lc $b;
+            if ($b eq 'parrot') {
+                die "The Parrot backend has been suspended.\n"
+                    . "Please use Rakudo 2015.02 (which still supports parrot), or the MoarVM backend instead\n";
+            }
             unless ($known_backends{$b}) {
                 die "Unknown backend '$b'; Supported backends are: " .
                     join(", ", sort keys %known_backends) .
                     "\n";
             }
             $backends{$b} = 1;
+            push @backends, $b;
             $default_backend ||= $b;
         }
         unless (%backends) {
@@ -79,30 +91,25 @@ MAIN: {
     }
     else {
         for my $l (sort keys %letter_to_backend) {
-            # TODO: needs .exe/.bat magic on windows?
-            if (-x "$prefix/bin/nqp-$l") {
+            if (-x "$prefix/bin/nqp-$l" || -x "$prefix/bin/nqp-$l.exe" || -x "$prefix/bin/nqp-$l.bat") {
                 my $b = $letter_to_backend{$l};
                 print "Found $prefix/bin/nqp-$l (backend $b)\n";
                 $backends{$b} = 1;
+                push @backends, $b;
                 $default_backend ||= $b;
             }
         }
         if (exists $options{'gen-moar'}) {
+            push @backends, 'moar' unless $backends{moar};
             $backends{moar} = 1;
             $default_backend ||= 'moar';
         }
-        if (exists $options{'gen-parrot'}) {
-            $backends{parrot} = 1;
-            $default_backend ||= 'parrot';
-        }
         unless (%backends) {
             die "No suitable nqp executables found! Please specify some --backends, or a --prefix that contains nqp-{p,j,m} executables\n\n"
               . "Example to build for all backends (which will take a while):\n"
-              . "\tperl Configure.pl --backends=moar,parrot,jvm --gen-moar --gen-parrot\n\n"
+              . "\tperl Configure.pl --backends=moar,jvm --gen-moar\n\n"
               . "Example to build for MoarVM only:\n"
               . "\tperl Configure.pl --gen-moar\n\n"
-              . "Example to build for Parrot only:\n"
-              . "\tperl Configure.pl --gen-parrot\n\n"
               . "Example to build for JVM only:\n"
               . "\tperl Configure.pl --backends=jvm --gen-nqp\n\n";
         }
@@ -117,91 +124,80 @@ MAIN: {
     }
 
     $config{prefix} = $prefix;
+    $config{sdkroot} = $options{sdkroot} || '';
+    $config{sysroot} = $options{sysroot} || '';
     $config{slash}  = $slash;
     $config{'makefile-timing'} = $options{'makefile-timing'};
     $config{'stagestats'} = '--stagestats' if $options{'makefile-timing'};
-    $config{'cpsep'} = $^O eq 'MSWin32' ? ';' : ':';
-    $config{'shell'} = $^O eq 'MSWin32' ? 'cmd' : 'sh';
-    $config{'runner_suffix'} = $^O eq 'MSWin32' ? '.bat' : '';
+    $config{'cpsep'} = $win ? ';' : ':';
+    $config{'shell'} = $win ? 'cmd' : 'sh';
+    $config{'runner_suffix'} = $win ? '.bat' : '';
 
-    my $make = $config{'make'} = $^O eq 'MSWin32' ? 'nmake' : 'make';
+    my $make = 'make';
+    if ($win) {
+        my $has_nmake = 0 == system('nmake /? >NUL 2>&1');
+        my $has_cl    = `cl 2>&1` =~ /Microsoft Corporation/;
+        my $has_gmake = 0 == system('gmake --version >NUL 2>&1');
+        my $has_gcc   = 0 == system('gcc --version >NUL 2>&1');
+        if (-x "$prefix/bin/nqp-m.bat"
+        && ($_ = `$prefix/bin/nqp-m.bat -e "print(nqp::backendconfig()<make>)"`)) {
+            $make = $_;
+        }
+        elsif ($has_nmake && $has_cl) {
+            $make = 'nmake';
+        }
+        elsif ($has_gmake && $has_gcc) {
+            $make = 'gmake';
+        }
+    }
+
+    for my $target (qw/common_bootstrap_sources moar_core_sources/) {
+        open my $FILELIST, '<', "tools/build/$target"
+            or die "Cannot read 'tools/build/$target': $!";
+        my @lines;
+        while (<$FILELIST>) {
+            chomp;
+            push @lines, "  $_\\\n";
+        }
+        close $FILELIST;
+        $config{$target} = join '', @lines;
+    }
 
     open my $MAKEFILE, '>', 'Makefile'
         or die "Cannot open 'Makefile' for writing: $!";
 
-    my @prefixes = sort map substr($_, 0, 1), keys %backends;
-
     print $MAKEFILE "\n# Makefile code generated by Configure.pl:\n";
 
+    fill_template_file('tools/build/Makefile-common-macros.in', $MAKEFILE, %config);
+
+    my @prefixes = map substr($_, 0, 1), @backends;
+
     my $launcher = substr($default_backend, 0, 1) . '-runner-default';
     print $MAKEFILE "all: ", join(' ', map("$_-all", @prefixes), $launcher), "\n";
     print $MAKEFILE "install: ", join(' ', map("$_-install", @prefixes), $launcher . '-install'), "\n";
 
     print $MAKEFILE "clean: ", join(' ', map "$_-clean", @prefixes), "\n";
-    print $MAKEFILE "\t\$(RM_F) perl6\$(EXE) perl6\$(BAT)\n\n";
+    print $MAKEFILE "\t\$(RM_F) perl6", $config{'runner_suffix'},"\n\n";
 
     for my $t (qw/test spectest coretest localtest stresstest/) {
         print $MAKEFILE "$t: ", join(' ', map "$_-$t", @prefixes), "\n";
     }
 
-    fill_template_file('tools/build/Makefile-common.in', $MAKEFILE, %config);
+    fill_template_file('tools/build/Makefile-common-rules.in', $MAKEFILE, %config);
 
     # determine the version of NQP we want
     my ($nqp_want) = split(' ', slurp('tools/build/NQP_REVISION'));
 
-    $options{'gen-nqp'} ||= '' if exists $options{'gen-parrot'} || exists $options{'gen-moar'};
+    $options{'gen-nqp'} ||= '' if exists $options{'gen-moar'};
 
     my %binaries;
     my %impls = gen_nqp($nqp_want, prefix => $prefix, backends => join(',', sort keys %backends), %options);
 
     my @errors;
     my %errors;
-    if ($backends{parrot}) {
-        my %nqp_config;
-        if ($impls{parrot}{ok}) {
-            %nqp_config = %{ $impls{parrot}{config} };
-        }
-        elsif ($impls{parrot}{config}) {
-            push @errors, "The nqp-p binary is too old";
-        }
-        else {
-            push @errors, "Unable to read configuration from NQP on Parrot";
-        }
-
-        my $nqp_have = $nqp_config{'nqp::version'} || '';
-        if ($nqp_have && cmp_rev($nqp_have, $nqp_want) < 0) {
-            push @errors, "NQP revision $nqp_want required (currently $nqp_have).";
-        }
-
-        if (!@errors) {
-            push @errors, verify_install([ @NQP::Configure::required_parrot_files,
-                                        @NQP::Configure::required_nqp_files ],
-                                        %config, %nqp_config);
-            push @errors,
-            "(Perhaps you need to 'make install', 'make install-dev',",
-            "or install the 'devel' package for NQP or Parrot?)"
-            if @errors;
-        }
-
-        $errors{parrot}{'no gen-nqp'} = @errors && !defined $options{'gen-nqp'};
-
-        unless (@errors) {
-            print "Using $impls{parrot}{bin} (version $nqp_config{'nqp::version'} / Parrot $nqp_config{'parrot::VERSION'}).\n";
-
-            if ($^O eq 'MSWin32' or $^O eq 'cygwin') {
-                $config{'dll'} = '$(PARROT_BIN_DIR)/$(PARROT_LIB_SHARED)';
-                $config{'dllcopy'} = '$(PARROT_LIB_SHARED)';
-                $config{'make_dllcopy'} =
-                    '$(PARROT_DLL_COPY): $(PARROT_DLL)'."\n\t".'$(CP) $(PARROT_DLL) .';
-            }
-
-            my $make = fill_template_text('@make@', %config, %nqp_config);
-            fill_template_file('tools/build/Makefile-Parrot.in', $MAKEFILE, %config, %nqp_config);
-        }
-    }
     if ($backends{jvm}) {
         $config{j_nqp} = $impls{jvm}{bin};
-        $config{j_nqp} =~ s{/}{\\}g if $^O eq 'MSWin32';
+        $config{j_nqp} =~ s{/}{\\}g if $win;
         my %nqp_config;
         if ( $impls{jvm}{ok} ) {
             %nqp_config = %{ $impls{jvm}{config} };
@@ -228,10 +224,11 @@ MAIN: {
 
             print "Using $bin (version $nqp_config{'nqp::version'} / $java_version).\n";
 
-            $config{'nqp_prefix'}    = $nqp_config{'jvm::runtime.prefix'};
+            $config{'nqp_prefix'}    = $nqp_config{'jvm::prefix'};
             $config{'nqp_jars'}      = $nqp_config{'jvm::runtime.jars'};
+            $config{'bld_nqp_jars'}  = join( $config{'cpsep'}, map { $config{'sysroot'} . $_ } split( $config{'cpsep'}, $nqp_config{'jvm::runtime.jars'} ) );
             $config{'nqp_classpath'} = $nqp_config{'jvm::runtime.classpath'};
-            $config{'j_runner'}      = $^O eq 'MSWin32' ? 'perl6-j.bat' : 'perl6-j';
+            $config{'j_runner'}      = $win ? 'perl6-j.bat' : 'perl6-j';
 
 
             fill_template_file('tools/build/Makefile-JVM.in', $MAKEFILE, %config);
@@ -239,43 +236,48 @@ MAIN: {
     }
     if ($backends{moar}) {
         $config{m_nqp} = $impls{moar}{bin};
-        $config{m_nqp} =~ s{/}{\\}g if $^O eq 'MSWin32';
+        $config{m_nqp} =~ s{/}{\\}g if $win;
         my %nqp_config;
-        if ( $impls{moar}{config} ) {
+        if ( $impls{moar}{ok} ) {
             %nqp_config = %{ $impls{moar}{config} };
         }
+        elsif ( $impls{moar}{config} ) {
+            push @errors, "The nqp-m binary is too old";
+        }
         else {
             push @errors, "Unable to read configuration from NQP on MoarVM";
         }
 
         $errors{moar}{'no gen-nqp'} = @errors && !defined $options{'gen-nqp'};
 
+        unless ($win) {
+            $config{'m_cleanups'} = "  \$(M_GDB_RUNNER) \\\n  \$(M_VALGRIND_RUNNER)";
+            $config{'m_all'}      = '$(M_GDB_RUNNER) $(M_VALGRIND_RUNNER)';
+            $config{'m_install'}  = '	$(PERL) tools/build/create-moar-runner.pl "$(MOAR)" perl6.moarvm $(DESTDIR)$(PREFIX)/bin/perl6-gdb-m "$(PERL6_LANG_DIR)/runtime" "gdb" "$(M_LIBPATH)" "$(PERL6_LANG_DIR)/lib" "$(PERL6_LANG_DIR)/runtime"' . "\n"
+                                  . '	$(PERL) tools/build/create-moar-runner.pl "$(MOAR)" perl6.moarvm $(DESTDIR)$(PREFIX)/bin/perl6-valgrind-m "$(PERL6_LANG_DIR)/runtime" "valgrind" "$(M_LIBPATH)" "$(PERL6_LANG_DIR)/lib" "$(PERL6_LANG_DIR)/runtime"';
+        }
+
         unless (@errors) {
             print "Using $config{m_nqp} (version $nqp_config{'nqp::version'} / MoarVM $nqp_config{'moar::version'}).\n";
 
             $config{'perl6_ops_dll'} = sprintf($nqp_config{'moar::dll'}, 'perl6_ops_moar');
-            
+
             # Add moar library to link command
             # TODO: Get this from Moar somehow
-            $config{'moarimplib'} = $^O eq 'MSWin32' ? "$prefix/bin/moar.dll.lib"
-                                  : $^O eq 'darwin'  ? "$prefix/lib/libmoar.dylib"
+            $config{'moarimplib'} = $win || $^O eq 'darwin'
+                                  ? $nqp_config{'moar::libdir'} . '/' . $nqp_config{'moar::sharedlib'}
                                   : '';
 
             fill_template_file('tools/build/Makefile-Moar.in', $MAKEFILE, %config, %nqp_config);
         }
     }
 
-    if ($errors{parrot}{'no gen-nqp'} || $errors{jvm}{'no gen-nqp'} || $errors{moar}{'no gen-nqp'}) {
+    if ($errors{jvm}{'no gen-nqp'} || $errors{moar}{'no gen-nqp'}) {
         my @options_to_pass;
-        push @options_to_pass, "--gen-parrot" if $backends{parrot};
         push @options_to_pass, "--gen-moar"   if $backends{moar};
         push @options_to_pass, "--gen-nqp"    unless @options_to_pass;
         my $options_to_pass  = join ' ', @options_to_pass;
-        my $want_executables = $backends{parrot} && $backends{moar}
-                             ? ', Parrot and MoarVM'
-                             : $backends{parrot}
-                             ? ' and Parrot'
-                             : $backends{moar}
+        my $want_executables =$backends{moar}
                              ? ' and MoarVM'
                              : '';
         my $s1               = @options_to_pass > 1 ? 's' : '';
@@ -325,23 +327,25 @@ Configure.pl - $lang Configure
 General Options:
     --help             Show this text
     --prefix=dir       Install files in dir; also look for executables there
-    --backends=parrot,jvm,moar
+    --sdkroot=dir      When given, use for searching build tools here, e.g.
+                       nqp, java etc.
+    --sysroot=dir      When given, use for searching runtime components here
+    --backends=jvm,moar
                        Which backend(s) to use (or ALL for all of them)
     --gen-nqp[=branch]
                        Download and build a copy of NQP
-        --gen-moar[=branch]
+    --gen-moar[=branch]
                        Download and build a copy of MoarVM to use
-        --moar-option='--option=value'
+    --moar-option='--option=value'
                        Options to pass to MoarVM's Configure.pl
-        --gen-parrot[=branch]
-                       Download and build a copy of Parrot
-        --parrot-option='--option'
-                       Options to pass to Parrot's Configure.pl
-        --parrot-make-option='--option'
-                       Options to pass to Parrot's make, for example:
-                       --parrot-make-option='--jobs=4'
-        --git-protocol={ssh,https,git}
+    --git-protocol={ssh,https,git}
                        Protocol used for cloning git repos
+    --git-depth=<number>
+                       Use the --git-depth option for git clone with parameter number
+    --git-reference=<path>
+                       Use --git-reference option to identify local path where git repositories are stored
+                       For example: --git-reference=/home/user/repo/for_perl6
+                       Folders 'nqp' and 'MoarVM' with corresponding git repos should be in for_perl6 folder
     --makefile-timing  Enable timing of individual makefile commands
 
 Configure.pl also reads options from 'config.default' in the current directory.
