@@ -3,6 +3,7 @@
 my class AST {
     has $!past;
     has $!quasi_context;
+    has $!Str;
 
     submethod BUILD(:$past) {
         $!past := $past;
@@ -12,7 +13,7 @@ my class AST {
         my $incarnation = self.clone();
         nqp::bindattr(nqp::decont($incarnation), AST, '$!past', $incarnation.evaluate_unquotes(@unquote_asts));
         nqp::bindattr(nqp::decont($incarnation), AST, '$!quasi_context', $quasi_context);
-        return $incarnation;
+        $incarnation;
     }
 
     method evaluate_unquotes(@unquote_asts) {
@@ -23,7 +24,7 @@ my class AST {
                 got      => $_,
                 expected => AST,
                 action   => 'unquote evaluation',
-            ).throw unless $_ ~~ AST;
+            ).throw unless nqp::istype($_,AST);
             nqp::push($pasts, nqp::getattr(nqp::decont($_), AST, '$!past'))
         }
         $!past.evaluate_unquotes($pasts);
@@ -32,6 +33,10 @@ my class AST {
     method is_quasi_ast {
         so $!quasi_context;
     }
+
+    method Str {
+        $!Str;
+    }
 }
 
 # vim: ft=perl6 expandtab sw=4
