@@ -1,17 +1,28 @@
+my class X::Numeric::Real { ... };
 my class Complex is Cool does Numeric {
     has num $.re;
     has num $.im;
 
     proto method new(|) { * }
     multi method new(Real \re, Real \im) {
-        my $new = nqp::create(self);
-        $new.BUILD(re.Num, im.Num);
-        $new;
+        nqp::create(self).BUILD(re, im);
     }
-    method BUILD(Num \re, Num \im) {
+    submethod BUILD(Num() \re, Num() \im) {
         $!re = re;
         $!im = im;
+        self;
     }
+
+    multi method WHICH(Complex:D:) {
+        nqp::box_s(
+            nqp::concat(
+                nqp::concat(nqp::unbox_s(self.^name), '|'),
+                nqp::concat($!re, nqp::concat('|', $!im))
+            ),
+            ObjAt
+        );
+    }
+
     method reals(Complex:D:) {
         (self.re, self.im);
     }
@@ -20,9 +31,8 @@ my class Complex is Cool does Numeric {
         self.re.isNaN || self.im.isNaN;
     }
 
-    my class X::Numeric::Real { ... };
     method coerce-to-real(Complex:D: $exception-target) {
-        unless $!im == 0 { fail X::Numeric::Real.new(target => $exception-target, reason => "imaginary part not zero", source => self);}
+        unless $!im == 0e0 { fail X::Numeric::Real.new(target => $exception-target, reason => "imaginary part not zero", source => self);}
         $!re;
     }
     multi method Real(Complex:D:) { self.coerce-to-real(Real); }
@@ -45,7 +55,7 @@ my class Complex is Cool does Numeric {
     }
 
     multi method perl(Complex:D:) {
-        "Complex.new($.re, $.im)";
+        '<' ~ self.Str ~ '>';
     }
     method conj(Complex:D:) {
         Complex.new($.re, -$.im);
@@ -78,8 +88,7 @@ my class Complex is Cool does Numeric {
         Complex.new($mag * $!im.cos, $mag * $!im.sin);
     }
 
-    method roots(Complex:D: $an) {
-        my Int $n = $an.Int;
+    method roots(Complex:D: Int() $n) {
         return NaN if $n < 1;
         return self if $n == 1;
         for $!re, $!im {
@@ -90,45 +99,45 @@ my class Complex is Cool does Numeric {
         $mag **= 1e0 / $n;
         (^$n).map: { $mag.unpolar( ($angle + $_ * 2e0 * pi) / $n) };
     }
-    
+
     method sin(Complex:D:) {
         $!re.sin * $!im.cosh + ($!re.cos * $!im.sinh)i;
     }
-    
+
     method asin(Complex:D:) {
-        (Complex.new(0, -1) * log((self)i + sqrt(1 - self * self)));
+        (Complex.new(0e0, -1e0) * log((self)i + sqrt(1e0 - self * self)));
     }
 
     method cos(Complex:D:) {
         $!re.cos * $!im.cosh - ($!re.sin * $!im.sinh)i;
     }
-    
+
     method acos(Complex:D:) {
-        (pi / 2) - self.asin;
+        (pi / 2e0) - self.asin;
     }
-    
+
     method tan(Complex:D:) {
         self.sin / self.cos;
     }
-    
+
     method atan(Complex:D:) {
-        ((log(1 - (self)i) - log(1 + (self)i))i / 2);
+        ((log(1e0 - (self)i) - log(1e0 + (self)i))i / 2e0);
     }
 
     method sec(Complex:D:) {
-        1 / self.cos;
+        1e0 / self.cos;
     }
 
     method asec(Complex:D:) {
-        (1 / self).acos;
+        (1e0 / self).acos;
     }
 
     method cosec(Complex:D:) {
-        1 / self.sin;
+        1e0 / self.sin;
     }
 
     method acosec(Complex:D:) {
-        (1 / self).asin;
+        (1e0 / self).asin;
     }
 
     method cotan(Complex:D:) {
@@ -136,55 +145,55 @@ my class Complex is Cool does Numeric {
     }
 
     method acotan(Complex:D:) {
-        (1 / self).atan;
+        (1e0 / self).atan;
     }
 
     method sinh(Complex:D:) {
-        -((Complex.new(0, 1) * self).sin)i;
+        -((Complex.new(0e0, 1e0) * self).sin)i;
     }
-    
+
     method asinh(Complex:D:) {
-        (self + sqrt(1 + self * self)).log;
+        (self + sqrt(1e0 + self * self)).log;
     }
-    
+
     method cosh(Complex:D:) {
-        (Complex.new(0, 1) * self).cos;
+        (Complex.new(0e0, 1e0) * self).cos;
     }
-    
+
     method acosh(Complex:D:) {
-        (self + sqrt(self * self - 1)).log;
+        (self + sqrt(self * self - 1e0)).log;
     }
-    
+
     method tanh(Complex:D:) {
-        -((Complex.new(0, 1) * self).tan)i;
+        -((Complex.new(0e0, 1e0) * self).tan)i;
     }
-    
+
     method atanh(Complex:D:) {
-        (((1 + self) / (1 - self)).log / 2);
+        (((1e0 + self) / (1e0 - self)).log / 2e0);
     }
-    
+
     method sech(Complex:D:) {
-        1 / self.cosh;
+        1e0 / self.cosh;
     }
-    
+
     method asech(Complex:D:) {
-        (1 / self).acosh;
+        (1e0 / self).acosh;
     }
-    
+
     method cosech(Complex:D:) {
-        1 / self.sinh;
+        1e0 / self.sinh;
     }
-    
+
     method acosech(Complex:D:) {
-        (1 / self).asinh;
+        (1e0 / self).asinh;
     }
-    
+
     method cotanh(Complex:D:) {
-        1 / self.tanh;
+        1e0 / self.tanh;
     }
-    
+
     method acotanh(Complex:D:) {
-        (1 / self).atanh;
+        (1e0 / self).atanh;
     }
 
     method floor(Complex:D:) {
@@ -196,7 +205,10 @@ my class Complex is Cool does Numeric {
     }
 
     proto method round(|) {*}
-    multi method round(Complex:D: $scale as Real = 1) {
+    multi method round(Complex:D:) {
+        Complex.new( self.re.round, self.im.round );
+    }
+    multi method round(Complex:D: Real(Real) $scale) {
         Complex.new( self.re.round($scale), self.im.round($scale) );
     }
 
@@ -249,12 +261,12 @@ multi sub infix:<+>(Complex:D \a, Complex:D \b) returns Complex:D {
     $new;
 }
 
-multi sub infix:<+>(Complex:D \a, Real \b) returns Complex:D {
+multi sub infix:<+>(Complex:D \a, Num(Real) \b) returns Complex:D {
     my $new := nqp::create(Complex);
     nqp::bindattr_n( $new, Complex, '$!re',
         nqp::add_n(
             nqp::getattr_n(nqp::decont(a), Complex, '$!re'),
-            nqp::unbox_n(b.Num)
+            nqp::unbox_n(b)
         )
     );
     nqp::bindattr_n($new, Complex, '$!im',
@@ -263,11 +275,11 @@ multi sub infix:<+>(Complex:D \a, Real \b) returns Complex:D {
     $new
 }
 
-multi sub infix:<+>(Real \a, Complex:D \b) returns Complex:D {
+multi sub infix:<+>(Num(Real) \a, Complex:D \b) returns Complex:D {
     my $new := nqp::create(Complex);
     nqp::bindattr_n($new, Complex, '$!re',
         nqp::add_n(
-            nqp::unbox_n(a.Num),
+            nqp::unbox_n(a),
             nqp::getattr_n(nqp::decont(b), Complex, '$!re'),
         )
     );
@@ -294,12 +306,12 @@ multi sub infix:<->(Complex:D \a, Complex:D \b) returns Complex:D {
     $new
 }
 
-multi sub infix:<->(Complex:D \a, Real \b) returns Complex:D {
+multi sub infix:<->(Complex:D \a, Num(Real) \b) returns Complex:D {
     my $new := nqp::create(Complex);
     nqp::bindattr_n( $new, Complex, '$!re',
         nqp::sub_n(
             nqp::getattr_n(nqp::decont(a), Complex, '$!re'),
-            b.Num,
+            b,
         )
     );
     nqp::bindattr_n($new, Complex, '$!im',
@@ -308,11 +320,11 @@ multi sub infix:<->(Complex:D \a, Real \b) returns Complex:D {
     $new
 }
 
-multi sub infix:<->(Real \a, Complex:D \b) returns Complex:D {
+multi sub infix:<->(Num(Real) \a, Complex:D \b) returns Complex:D {
     my $new := nqp::create(Complex);
     nqp::bindattr_n( $new, Complex, '$!re',
         nqp::sub_n(
-            a.Num,
+            a,
             nqp::getattr_n(nqp::decont(b), Complex, '$!re'),
         )
     );
@@ -339,9 +351,9 @@ multi sub infix:<*>(Complex:D \a, Complex:D \b) returns Complex:D {
     $new;
 }
 
-multi sub infix:<*>(Complex:D \a, Real \b) returns Complex:D {
+multi sub infix:<*>(Complex:D \a, Num(Real) \b) returns Complex:D {
     my $new := nqp::create(Complex);
-    my num $b_num = b.Num;
+    my num $b_num = b;
     nqp::bindattr_n($new, Complex, '$!re',
         nqp::mul_n(
             nqp::getattr_n(nqp::decont(a), Complex, '$!re'),
@@ -357,9 +369,9 @@ multi sub infix:<*>(Complex:D \a, Real \b) returns Complex:D {
     $new
 }
 
-multi sub infix:<*>(Real \a, Complex:D \b) returns Complex:D {
+multi sub infix:<*>(Num(Real) \a, Complex:D \b) returns Complex:D {
     my $new := nqp::create(Complex);
-    my num $a_num = a.Num;
+    my num $a_num = a;
     nqp::bindattr_n($new, Complex, '$!re',
         nqp::mul_n(
             $a_num,
@@ -402,28 +414,31 @@ multi sub infix:</>(Complex:D \a, Real \b) returns Complex:D {
 }
 
 multi sub infix:</>(Real \a, Complex:D \b) returns Complex:D {
-    Complex.new(a, 0) / b;
+    Complex.new(a, 0e0) / b;
 }
 
 multi sub infix:<**>(Complex:D \a, Complex:D \b) returns Complex:D {
     (a.re == 0e0 && a.im == 0e0) ?? Complex.new(0e0, 0e0) !! (b * a.log).exp
 }
-multi sub infix:<**>(Real      \a, Complex:D \b) returns Complex:D {
-    a == 0 ?? Complex.new(0e0, 0e0) !! (b * a.log).exp 
+multi sub infix:<**>(Num(Real) \a, Complex:D \b) returns Complex:D {
+    a == 0e0 ?? Complex.new(0e0, 0e0) !! (b * a.log).exp
 }
-multi sub infix:<**>(Complex:D \a, Real      \b) returns Complex:D {
+multi sub infix:<**>(Complex:D \a, Num(Real) \b) returns Complex:D {
     (b * a.log).exp
 }
 
 multi sub infix:<==>(Complex:D \a, Complex:D \b) returns Bool:D { a.re == b.re && a.im == b.im }
-multi sub infix:<==>(Complex:D \a, Real      \b) returns Bool:D { a.re == b    && a.im == 0e0   }
-multi sub infix:<==>(Real      \a, Complex:D \b) returns Bool:D { a    == b.re && 0e0   == b.im }
+multi sub infix:<==>(Complex:D \a, Num(Real) \b) returns Bool:D { a.re == b    && a.im == 0e0  }
+multi sub infix:<==>(Num(Real) \a, Complex:D \b) returns Bool:D { a    == b.re && 0e0  == b.im }
+multi sub infix:<===>(Complex:D \a, Complex:D \b) returns Bool:D {
+    a.WHAT =:= b.WHAT && a == b
+}
 
-proto postfix:<i>(|) returns Complex:D is pure { * }
-multi postfix:<i>(Real      \a) returns Complex:D { Complex.new(0e0, a);     }
-multi postfix:<i>(Complex:D \a) returns Complex:D { Complex.new(-a.im, a.re) }
-multi postfix:<i>(Numeric   \a) returns Complex:D { a * Complex.new(0e0, 1e0) }
-multi postfix:<i>(Cool      \a) returns Complex:D { a.Numeric * Complex.new(0e0, 1e0) }
+proto sub postfix:<i>(\a) returns Complex:D is pure { * }
+multi sub postfix:<i>(Real      \a) returns Complex:D { Complex.new(0e0, a);     }
+multi sub postfix:<i>(Complex:D \a) returns Complex:D { Complex.new(-a.im, a.re) }
+multi sub postfix:<i>(Numeric   \a) returns Complex:D { a * Complex.new(0e0, 1e0) }
+multi sub postfix:<i>(Cool      \a) returns Complex:D { a.Numeric * Complex.new(0e0, 1e0) }
 
 constant i = Complex.new(0e0, 1e0);
 
