@@ -1,9 +1,9 @@
-my role  IO { ... }
-my class IO::Handle { ... }
+my role  IO         { ... }
+my class IO::Path   { ... }
 
 my class SprintfHandler {
     method mine($x) { nqp::reprname($x) eq "P6opaque"; }
-    
+
     method int($x) { $x.Int }
 }
 
@@ -17,7 +17,7 @@ my class Cool { # declared in BOOTSTRAP
     method abs()  { self.Numeric.abs }
     method conj()  { self.Numeric.conj }
     method sqrt()  { self.Numeric.sqrt }
-    method sign()  { self.Numeric.sign }
+    method sign()  { self.Real.sign }
     method rand() { self.Num.rand }
     method sin()  { self.Numeric.sin }
     method asin() { self.Numeric.asin }
@@ -45,7 +45,7 @@ my class Cool { # declared in BOOTSTRAP
     method cotanh()  { self.Numeric.cotanh }
     method acotanh() { self.Numeric.acotanh }
     method cis()     { self.Numeric.cis }
-    
+
     proto method log(|) {*}
     multi method log(Cool:D: )      { self.Numeric.log          }
     multi method log(Cool:D: $base) { self.Numeric.log($base.Numeric) }
@@ -54,23 +54,25 @@ my class Cool { # declared in BOOTSTRAP
     multi method exp(Cool:D: )      { self.Numeric.exp          }
     multi method exp(Cool:D: $base) { self.Numeric.exp($base.Numeric) }
 
+    proto method round(|) { * }
+    multi method round()      { self.Numeric.round()      }
+    multi method round($base) { self.Numeric.round($base) }
 
     method roots(Cool $n)   { self.Numeric.roots($n)    }
     method log10()          { self.Numeric.log10        }
     method unpolar($n)      { self.Numeric.unpolar($n.Numeric) }
 
-    method round($base = 1) { self.Numeric.round($base) }
     method floor()          { self.Numeric.floor        }
     method ceiling()        { self.Numeric.ceiling      }
     method truncate()       { self.Numeric.truncate     }
 
     ## string methods
 
-    method chars() {
-        nqp::p6box_i(nqp::chars(nqp::unbox_s(self.Str)));
+    method chars() returns Int:D {
+        self.Str.chars
     }
     method codes() {
-        nqp::p6box_i(nqp::chars(nqp::unbox_s(self.Str)));
+        self.Str.codes
     }
 
     method fmt($format = '%s') {
@@ -83,47 +85,51 @@ my class Cool { # declared in BOOTSTRAP
         )
     }
 
-    method substr($start, $length?) { 
-        self.Stringy.substr($start, $length);
-    }
+    method substr($from, $length?)           { substr(   self,$from,$length) }
+    method substr-rw(\SELF: $from, $length?) { substr-rw(SELF,$from,$length) }
 
     method uc() {
-        nqp::p6box_s(nqp::uc(nqp::unbox_s(self.Str)))
+        self.Str.uc
     }
 
     method lc() {
-        nqp::p6box_s(nqp::lc(nqp::unbox_s(self.Str)))
+        self.Str.lc
     }
 
     method tc() {
-        my $u := nqp::unbox_s(self.Str);
-        nqp::p6box_s(nqp::uc(nqp::substr($u,0,1)) ~ nqp::substr($u,1));
+        self.Str.tc
+    }
+
+    method fc() {
+        self.Str.fc
     }
 
     method tclc() {
-        nqp::p6box_s(nqp::tclc(nqp::unbox_s(self.Str)))
-    }
-
-    method ucfirst() { # is DEPRECATED doesn't work in settings
-        DEPRECATED("'tc'");
-        self.tc;
+        self.Str.tclc
     }
 
     method wordcase()   { self.Str.wordcase }
 
+    method uniname()        { uniname(self) }
+    method uninames()       { uninames(self) }
+    method unival()         { unival(self) }
+    method univals()        { univals(self) }
+    method uniprop(|c)      { uniprop(self, |c) }
+    method uniprop-int(|c)  { uniprop-int(self, |c) }
+    method uniprop-bool(|c) { uniprop-bool(self, |c) }
+    method uniprop-str(|c)  { uniprop-str(self, |c) }
+    method unimatch(|c)     { unimatch(self, |c) }
+
     method chomp() {
         self.Str.chomp;
     }
 
-    method chop() {
-        self.Str.chop
+    method chop(Int() $n = 1) {
+        self.Str.chop($n)
     }
 
     method ord(--> Int) {
-        my $s := self.Str;
-        $s.chars
-          ?? nqp::p6box_i(nqp::ord(nqp::unbox_s($s)))
-          !! Int;
+        self.Str.ord
     }
     method chr() {
         self.Int.chr;
@@ -131,56 +137,113 @@ my class Cool { # declared in BOOTSTRAP
     method chrs(Cool:D:) {
         self>>.chr.join;
     }
+    method ords(Cool:D:) { self.Str.ords }
+
 
     method flip() {
-        nqp::p6box_s(nqp::flip(nqp::unbox_s(self.Str)))
+        self.Str.flip
     }
     method trans(*@a) { self.Str.trans(@a) }
 
-    proto method index(|) {*}
-    multi method index(Cool $needle, Cool $pos = 0) {
-        if $needle eq '' {
-            my $chars = self.chars;
-            return $pos < $chars ?? $pos !! $chars;
+    proto method starts-with(|) {*}
+    multi method starts-with(Cool:D: Str(Cool) $needle) {
+        nqp::p6bool(
+          nqp::eqat(nqp::unbox_s(self.Str),nqp::unbox_s($needle),0)
+        );
+    }
+
+    proto method ends-with(Str(Cool) $suffix) { * }
+    multi method ends-with(Cool:D: Str(Cool) $suffix) {
+        my str $str    = nqp::unbox_s(self.Str);
+        my str $needle = nqp::unbox_s($suffix);
+        nqp::p6bool(
+          nqp::eqat($str,$needle,nqp::chars($str) - nqp::chars($needle))
+        );
+    }
+
+    proto method substr-eq(|) {*}
+    multi method substr-eq(Cool:D: Str(Cool) $needle, Cool $start?) {
+        my str $str = nqp::unbox_s(self.Str);
+        my int $pos =
+          nqp::defined($start) ?? nqp::chars($str) min $start.Int !! 0;
+        $pos >= 0 && nqp::eqat($str, nqp::unbox_s($needle), $pos);
+    }
+
+    proto method contains(|) {*}
+    multi method contains(Cool:D: Str(Cool) $needle, Cool $start?) {
+        my str $str = nqp::unbox_s(self.Str);
+        my int $pos =
+          nqp::defined($start) ?? nqp::chars($str) min $start.Int !! 0;
+        nqp::index($str, nqp::unbox_s($needle), $pos) != -1;
+    }
+
+    proto method indices(|) {*}
+    multi method indices(Cool:D: Str(Cool) $needle, Cool $start?, :$overlap) {
+        my str $str  = nqp::unbox_s(self.Str);
+        my int $pos  =
+          nqp::defined($start) ?? nqp::chars($str) min $start.Int !! 0;
+        my str $need = nqp::unbox_s($needle);
+        my int $add  = $overlap ?? 1 !! nqp::chars($need) || 1;
+
+        my $rpa := nqp::list();
+        my int $i;
+        while ($i = nqp::index($str, $need, $pos)) >= 0 {
+            nqp::push($rpa,nqp::p6box_i($i));
+            $pos = $i + $add;
         }
-        my $result := nqp::p6box_i(nqp::index(
-                nqp::unbox_s(self.Str),
-                nqp::unbox_s($needle.Str),
-                nqp::unbox_i($pos.Int)
-        ));
-        # TODO: fail() instead of returning Int
-        $result < 0 ?? Int !! $result;
+        nqp::p6bindattrinvres(nqp::create(List), List, '$!reified', $rpa)
+    }
+
+    proto method index(|) {*}
+    multi method index(Cool:D: Str(Cool) $needle) {
+        my int $i = nqp::index(nqp::unbox_s(self.Str), nqp::unbox_s($needle));
+        $i < 0 ?? Nil !! nqp::box_i($i,Int);
+    }
+    multi method index(Cool:D: Str(Cool) $needle, Int(Cool) $pos) {
+        my int $i;
+        try {
+            $i = nqp::unbox_i($pos);
+            CATCH {
+                default {
+                    return Nil
+                }
+            }
+        }
+        $i = nqp::index(
+          nqp::unbox_s(self.Str),
+          nqp::unbox_s($needle),
+          $i
+        );
+        $i < 0 ?? Nil !! nqp::box_i($i,Int);
     }
 
     proto method rindex(|) {*}
-    multi method rindex(Cool $needle, Cool $pos?) {
-        if $needle eq '' {
-            return $pos.defined && $pos < self.chars
-                    ?? $pos
-                    !! self.chars;
+    multi method rindex(Cool:D: Str(Cool) $needle) {
+        my int $i = nqp::rindex(nqp::unbox_s(self.Str), nqp::unbox_s($needle));
+        $i < 0 ?? Nil !! nqp::box_i($i,Int);
+    }
+    multi method rindex(Cool:D: Str(Cool) $needle, Int(Cool) $pos) {
+        my int $i;
+        try {
+            $i = nqp::unbox_i($pos);
+            CATCH {
+                default {
+                    return Nil
+                }
+            }
         }
-        my $result = $pos.defined
-            ?? nqp::p6box_i(
-                nqp::rindex(
-                    nqp::unbox_s(self.Str),
-                    nqp::unbox_s($needle.Str),
-                    nqp::unbox_i($pos.Int)
-                ))
-            !! nqp::p6box_i(
-                nqp::rindex(
-                    nqp::unbox_s(self.Str),
-                    nqp::unbox_s($needle.Str),
-                ));
-        fail "substring not found" if $result < 0;
-        $result;
+        $i = nqp::rindex(
+          nqp::unbox_s(self.Str),
+          nqp::unbox_s($needle),
+          $i
+        );
+        $i < 0 ?? Nil !! nqp::box_i($i,Int);
     }
 
-    method ords(Cool:D:) { self.Str.ords }
-    proto method split(|) {*}
-    multi method split(Regex $pat, $limit = Inf, :$all) {
+    multi method split(Cool: Regex:D $pat, $limit = Inf;; :$all) {
         self.Stringy.split($pat, $limit, :$all);
     }
-    multi method split(Cool $pat, $limit = Inf, :$all) {
+    multi method split(Cool: Cool:D $pat, $limit = Inf;; :$all) {
         self.Stringy.split($pat.Stringy, $limit, :$all);
     }
     proto method match(|) {*}
@@ -188,12 +251,16 @@ my class Cool { # declared in BOOTSTRAP
         self.Stringy.match($target, |%adverbs)
     }
 
-    proto method comb(|) {*}
+    proto method comb(|) { * }
     multi method comb() { self.Str.comb() }
     multi method comb(Regex $matcher, $limit = Inf) { self.Str.comb($matcher, $limit) }
+    multi method comb(Str $matcher, $limit = Inf) { self.Str.comb($matcher, $limit) }
 
     proto method lines(|) {*}
-    multi method lines(Cool:D:) { self.Str.lines() }
+    multi method lines(Cool:D: |c) { self.Str.lines(|c) }
+
+    proto method words(|) {*}
+    multi method words(Cool:D: |c) { self.Str.words(|c) }
 
     proto method subst(|) {
         $/ := nqp::getlexdyn('$/');
@@ -204,12 +271,26 @@ my class Cool { # declared in BOOTSTRAP
         self.Stringy.subst($matcher, $replacement, |%adverbs);
     }
 
+    proto method subst-mutate(|c) {
+        $/ := nqp::getlexdyn('$/');
+        {*}
+    }
+    multi method subst-mutate($self is rw: |c) {
+        $/ := nqp::getlexdyn('$/');
+        my $str = Str($self);
+        my $match = $str.subst-mutate(|c);
+        $self = $str;
+        $match;
+    }
+
+    proto method IO(|) { * }
+    multi method IO(|c) { IO::Path.new(self) }
+
     method sprintf(*@args) { sprintf(self, @args) };
     method printf (*@args) {  printf(self, @args) };
     method samecase(Cool:D: Cool $pattern) { self.Stringy.samecase($pattern) }
 
-    method IO() { IO::Handle.new(:path(self.Stringy)) }
-    method path() { self.Stringy.path }
+    method path() { self.Stringy.IO }
     method trim         () { self.Stringy.trim          };
     method trim-leading () { self.Stringy.trim-leading  };
     method trim-trailing() { self.Stringy.trim-trailing };
@@ -219,28 +300,41 @@ my class Cool { # declared in BOOTSTRAP
     }
 
     multi method Real() { self.Numeric.Real }
-    method Int()  { self.Numeric.Int }
+
+    proto method Int(|) { * }
+    multi method Int()  { self.Numeric.Int }
+
+    proto method UInt(|) { * }
+    multi method UInt()  {
+        my $got := self.Int;
+        fail X::OutOfRange.new(
+          :what('Coercion to UInt'),
+          :$got,
+          :range("0..Inf")
+        ) if $got < 0;
+        $got;
+    }
+
     method Num()  { self.Numeric.Num }
     method Rat()  { self.Numeric.Rat }
 }
 Metamodel::ClassHOW.exclude_parent(Cool);
 
-sub ucfirst(Cool $s) { # is DEPRECATED doesn't work in settings
-    DEPRECATED("'tc'");
-    $s.tc;
-}
-
-sub chop(Cool $s)                  { $s.chop }
-sub chomp(Cool $s)                 { $s.chomp }
-sub flip(Cool $s)                  { $s.flip }
+sub chop(Cool $s, Int() $n = 1) returns Str { $s.chop($n) }
+sub chomp(Cool $s) returns Str     { $s.chomp }
+sub flip(Cool $s) returns Str      { $s.flip }
 sub index(Cool $s,$needle,$pos=0)  { $s.index($needle,$pos) }
 sub lc(Cool $s)                    { $s.lc }
 sub ord(Cool $s)                   { $s.ord }
-sub substr(Cool $s,$pos,$chars?)   { $s.substr($pos,$chars) }
 sub uc(Cool $s)                    { $s.uc }
 sub tc(Cool $s)                    { $s.tc }
+sub fc(Cool $s)                    { $s.fc }
 sub tclc(Cool $s)                  { $s.tclc }
 
+sub indices(Cool $s,$needle,$pos=0,:$overlap) {
+    $s.indices($needle,$pos,:$overlap);
+}
+
 proto sub rindex($, $, $?) is pure { * };
 multi sub rindex(Cool $s, Cool $needle, Cool $pos) { $s.rindex($needle, $pos) };
 multi sub rindex(Cool $s, Cool $needle)            { $s.rindex($needle) };
@@ -250,6 +344,8 @@ multi sub ords(Cool $s)       { ords($s.Stringy) }
 
 proto sub comb($, $, $?)            { * }
 multi sub comb(Regex $matcher, Cool $input, $limit = *) { $input.comb($matcher, $limit) }
+multi sub comb(Str $matcher, Cool $input, $limit = *) { $input.comb($matcher, $limit) }
+multi sub comb(Int:D $matcher, Cool $input, $limit = *) { $input.comb($matcher, $limit) }
 
 proto sub wordcase($) is pure { * }
 multi sub wordcase(Str:D $x) {$x.wordcase }
@@ -261,19 +357,17 @@ sub sprintf(Cool $format, *@args) {
         $sprintfHandlerInitialized = True;
     }
 
-    @args.gimme(*);
+    @args.elems;
     nqp::p6box_s(
         nqp::sprintf(nqp::unbox_s($format.Stringy),
-            nqp::clone(nqp::getattr(@args, List, '$!items'))
+            nqp::clone(nqp::getattr(@args, List, '$!reified'))
         )
     );
 }
 
-sub printf(Cool $format, *@args) { print sprintf $format, @args };
+sub printf(Cool $format, *@args)          { print sprintf $format, @args }
 sub samecase(Cool $string, Cool $pattern) { $string.samecase($pattern) }
-sub split($pat, Cool $target, $limit = Inf, :$all) {
-    $target.split($pat, $limit, :$all);
-}
+sub split($pat, Cool $target, |c)         { $target.split($pat, |c) }
 
 proto sub chars($) is pure {*}
 multi sub chars(Cool $x)  { $x.Str.chars }
