@@ -1,6 +1,7 @@
 # Scheduler that always does things immediately, on the current thread.
 
 my class CurrentThreadScheduler does Scheduler {
+
     method handle_uncaught($exception) {
         $exception.throw
     }
@@ -10,7 +11,7 @@ my class CurrentThreadScheduler does Scheduler {
           if $at.defined and $in.defined;
         die "Cannot specify :every and :times at the same time"
           if $every.defined and $times > 1;
-        die "Cannot specify :every in {self.HOW.name(self)}"
+        die "Cannot specify :every in {self.^name}"
           if $every;
 
         my $delay = $at ?? $at - now !! $in;
@@ -20,6 +21,7 @@ my class CurrentThreadScheduler does Scheduler {
 
         code() for 1 .. $times;
         CATCH { default { catch($_) } };
+        class { method cancel() {} }
     }
 
     method loads() { 0 }
