@@ -10,19 +10,12 @@
         %ENV{$key} = nqp::p6box_s(nqp::iterval($envelem));
     }
     PROCESS::<%ENV> := %ENV;
+}
 
-    ## duplicate src/core/IO.pm::cwd
-    my $CWD = IO::Path.new(nqp::p6box_s(
-#?if parrot
-        pir::trans_encoding__Ssi(
-            nqp::cwd(),
-            pir::find_encoding__Is('utf8'))
-#?endif
-#?if !parrot
-            nqp::cwd(),
-#?endif
-    ));
-    PROCESS::<$CWD> = $CWD;
+multi sub INITIALIZE_DYNAMIC('$*CWD') {
+#    PROCESS::<$CWD> = nqp::p6box_s(nqp::cwd());
+    my $CWD := nqp::p6box_s(nqp::cwd());
+    PROCESS::<$CWD> = IO::Path.new($CWD, :$CWD); # need :CWD to prevent looping
 }
 
 # vim: ft=perl6 expandtab sw=4
