@@ -6,9 +6,13 @@ my class IO::Notification {
     class Change {
         has $.path;
         has $.event;
+        multi method gist(Change:D:) {
+            "$.path: $.event";
+        }
+        method IO { $!path.IO }
     }
 
-    method watch_path($path as Str, :$scheduler = $*SCHEDULER) {
+    method watch-path(Str() $path, :$scheduler = $*SCHEDULER) {
         my $s = Supply.new;
         nqp::watchfile(
             $scheduler.queue,
@@ -18,7 +22,7 @@ my class IO::Notification {
                 }
                 else {
                     my $event = rename ?? FileRenamed !! FileChanged;
-                    $s.more(Change.new(:path(path // $path), :$event));
+                    $s.emit(Change.new(:path($*SPEC.catdir($path, path)), :$event));
                 }
             },
             $path, FileWatchCancellation);
