@@ -1,32 +1,25 @@
 my class Junction { # declared in BOOTSTRAP
     # class Junction is Mu {
-    #     has Mu $!storage;             # elements of Junction
+    #     has Mu $!storage;              # elements of Junction
     #     has str $!type;                # type of Junction
 
-    proto method new(|) { * }
-    multi method new(@values, Str :$type!) {
+    multi method new(\values, Str :$type!) {
         my $junc := nqp::create(Junction);
-        nqp::bindattr($junc, Junction, '$!storage', @values.eager);
-        nqp::bindattr($junc, Junction, '$!type', $type);
-        $junc
-    }
-    multi method new(*@values, Str :$type!) {
-        my $junc := nqp::create(Junction);
-        nqp::bindattr($junc, Junction, '$!storage', @values.eager);
+        nqp::bindattr($junc, Junction, '$!storage', values.eager.list);
         nqp::bindattr($junc, Junction, '$!type', $type);
         $junc
     }
 
     multi method Bool(Junction:D:) {
-        ($!storage.map({return True if $_}).gimme(*); return False)
+        SEQ($!storage.map({return True if $_}).sink; return False)
             if nqp::iseq_s($!type, 'any');
-        ($!storage.map({return False unless $_}).gimme(*); return True) 
+        SEQ($!storage.map({return False unless $_}).sink; return True)
             if nqp::iseq_s($!type, 'all');
-        ($!storage.map({return False if $_}).gimme(*); return True)
+        SEQ($!storage.map({return False if $_}).sink; return True)
             if nqp::iseq_s($!type, 'none');
         # 'one' junction
         my $count = 0;
-        $!storage.map({ $count++ if $_; return False if $count > 1 }).gimme(*);
+        $!storage.map({ $count++ if $_; return False if $count > 1 }).sink;
         $count == 1;
     }
 
@@ -34,38 +27,43 @@ my class Junction { # declared in BOOTSTRAP
         self.perl
     }
 
+    multi method ACCEPTS(Junction:U: Mu:D \topic) {
+        nqp::p6bool(nqp::istype(topic, Junction));
+    }
+    multi method ACCEPTS(Junction:U: Any \topic) {
+        nqp::p6bool(nqp::istype(topic, Junction));
+    }
     multi method ACCEPTS(Junction:D: Mu \topic) {
-        ($!storage.map({return True if $_.ACCEPTS(topic)}).gimme(*); return False)
+        SEQ($!storage.map({return True if $_.ACCEPTS(topic)}).sink; return False)
             if nqp::iseq_s($!type, 'any');
-        ($!storage.map({return False unless $_.ACCEPTS(topic)}).gimme(*); return True) 
+        SEQ($!storage.map({return False unless $_.ACCEPTS(topic)}).sink; return True)
             if nqp::iseq_s($!type, 'all');
-        ($!storage.map({return False if $_.ACCEPTS(topic)}).gimme(*); return True)
+        SEQ($!storage.map({return False if $_.ACCEPTS(topic)}).sink; return True)
             if nqp::iseq_s($!type, 'none');
         # 'one' junction
         my $count = 0;
-        $!storage.map({ $count++ if $_.ACCEPTS(topic); return False if $count > 1 }).gimme(*);
+        $!storage.map({ $count++ if $_.ACCEPTS(topic); return False if $count > 1 }).sink;
         $count == 1;
     }
 
     multi method gist(Junction:D:) {
         $!type ~ '(' ~ $!storage.map({$_.gist}).join(', ') ~ ')'
     }
-    
-    multi method perl(Junction:D:) { 
+
+    multi method perl(Junction:D:) {
         $!type ~ '(' ~ $!storage.map({$_.perl}).join(', ') ~ ')'
     }
-    
-    method postcircumfix:<( )>($c) {
+
+    method CALL-ME(|c) {
         self.AUTOTHREAD(
             -> $obj, |c { $obj(|c) },
-            self, |$c);
+            self, |c);
     }
-    
-    method sink(Junction:D:) {
+
+    method sink(Junction:D: --> Nil) {
         .?sink for $!storage.list;
-        Nil;
     }
-    
+
     method AUTOTHREAD(&call, |args) {
         my Mu $pos_rpa := nqp::getattr(nqp::decont(args), Capture, '$!list');
         sub thread_junction(int $i) {
@@ -74,15 +72,15 @@ my class Junction { # declared in BOOTSTRAP
             my @states := nqp::getattr(nqp::decont($arg), Junction, '$!storage');
 
             my Mu $res := nqp::list();
-            for @states -> $s {
+            @states.map: -> \st --> Nil {
                 # Next line is Officially Naughty, since captures are meant to be
                 # immutable. But hey, it's our capture to be naughty with...
-                nqp::bindpos($pos_rpa, $i, $s);
+                nqp::bindpos($pos_rpa, $i, st);
                 nqp::push($res, call(|args));
-                Nil;
             }
             my $res_junc := nqp::clone(nqp::decont($arg));
-            nqp::bindattr($res_junc, Junction, '$!storage', nqp::p6parcel($res, Nil));
+            nqp::bindattr($res_junc, Junction, '$!storage',
+                nqp::p6bindattrinvres(nqp::create(List), List, '$!reified', $res));
             return $res_junc;
         }
 
@@ -118,30 +116,30 @@ my class Junction { # declared in BOOTSTRAP
                 my @states := nqp::getattr(nqp::decont($v), Junction, '$!storage');
                 my $type   := nqp::getattr(nqp::decont($v), Junction, '$!type');
                 my Mu $res := nqp::list();
-                for @states -> $s {
-                    nqp::bindkey($nam_hash, $k, $s);
+                @states.map: -> \st --> Nil {
+                    nqp::bindkey($nam_hash, $k, st);
                     nqp::push($res, call(|args));
-                    Nil;
                 }
                 my $res_junc := nqp::clone(nqp::decont($v));
-                nqp::bindattr($res_junc, Junction, '$!storage', nqp::p6parcel($res, Nil));
+                nqp::bindattr($res_junc, Junction, '$!storage',
+                    nqp::p6bindattrinvres(nqp::create(List), List, '$!reified', $res));
                 return $res_junc;
             }
         }
-        
+
         # If we get here, wasn't actually anything to autothread.
         call(|args);
     }
 }
 
-sub any(*@values) { Junction.new(@values, :type<any>); }
-sub all(*@values) { Junction.new(@values, :type<all>); }
-sub one(*@values) { Junction.new(@values, :type<one>); }
-sub none(*@values) { Junction.new(@values, :type<none>); }
+sub any (+values) is pure { values.any }
+sub all (+values) is pure { values.all }
+sub one (+values) is pure { values.one }
+sub none(+values) is pure { values.none }
 
-sub infix:<|>(**@values) { Junction.new(@values, :type<any>); }
-sub infix:<&>(**@values) { Junction.new(@values, :type<all>); }
-sub infix:<^>(**@values) { Junction.new(@values, :type<one>); }
+sub infix:<|>(+values) is pure { values.any }
+sub infix:<&>(+values) is pure { values.all }
+sub infix:<^>(+values) is pure { values.one }
 
 sub AUTOTHREAD(|c) {
     Junction.AUTOTHREAD(|c)
