@@ -1,11 +1,10 @@
-#?if !parrot
 # A reentrant lock mechanism with condition variable support.
 my class X::Lock::ConditionVariable::New is Exception {
     method message() {
         "Cannot directly create a ConditionVariable; use the 'condition' method on a lock"
     }
 }
-my class Lock is repr('ReentrantMutex') {
+my class Lock {
     class ConditionVariable is repr('ConditionVariable') {
         method new() {
             X::Lock::ConditionVariable::New.new.throw
@@ -15,11 +14,11 @@ my class Lock is repr('ReentrantMutex') {
         method signal_all() { nqp::condsignalall(self) }
     }
 
-    method lock() { nqp::lock(self) }
+    method lock(Lock:D:) { nqp::lock(self) }
 
-    method unlock() { nqp::unlock(self) }
+    method unlock(Lock:D:) { nqp::unlock(self) }
 
-    method protect(&code) {
+    method protect(Lock:D: &code) {
         nqp::lock(self);
         my \res := code();
         nqp::unlock(self);
@@ -27,16 +26,9 @@ my class Lock is repr('ReentrantMutex') {
         res
     }
 
-    method condition() {
+    method condition(Lock:D:) {
         nqp::getlockcondvar(self, ConditionVariable)
     }
 }
 
-#?endif
-#?if parrot
-my class Lock {
-    method protect (&code) { code() }
-}
-#?endif
-
 # vim: ft=perl6 expandtab sw=4
