@@ -1,11 +1,4 @@
 # TODO:
-# * Align number parsing to STD
-#   * Rakudo's .Numeric
-#     * complex numbers
-#   * Rakudo's grammar
-#   * val()
-# * Strengthen val()
-#   * Make val() available globally
 # * $?USAGE
 #   * Create $?USAGE at compile time
 #   * Make $?USAGE available globally
@@ -15,40 +8,24 @@
 #   * Allow exact Perl 6 forms, quoted away from shell
 # * Fix remaining XXXX
 
-my sub MAIN_HELPER($retval = 0) is hidden_from_backtrace {
+my sub MAIN_HELPER($retval = 0) {
     # Do we have a MAIN at all?
     my $m = callframe(1).my<&MAIN>;
     return $retval unless $m;
 
-    # Temporary stand-in for magic val() routine
-    my sub hack-val ($v) {
-        # Convert to native type if appropriate
-
-        my $val;
-        if    $v ~~ /^ 'Bool::'?'False' $/ { $val := Bool::False }
-        elsif $v ~~ /^ 'Bool::'?'True'  $/ { $val := Bool::True  }
-        elsif $v.Numeric.defined           { $val := +$v }
-        else                               { return $v   }
-
-        # Mix in original stringifications
-        my role orig-string[$orig] {
-                  method Str  ()      { $orig.Str  }
-            multi method gist (Mu:D:) { $orig.gist }
-        };
-        return $val but orig-string[$v];
-    }
-
     # Convert raw command line args into positional and named args for MAIN
-    my sub process-cmd-args (@args is copy) {
+    my sub process-cmd-args(@args is copy) {
         my (@positional-arguments, %named-arguments);
-        while (@args) {
+        my $stopped = False;
+        while +@args {
             my $passed-value = @args.shift;
-            if $passed-value ~~ /^ ( '--' | '-' | ':' ) ('/'?) (<-[0..9\.]> .*) $/ {
+            $stopped = $passed-value eq '--';
+            if !$stopped && $passed-value ~~ /^ ( '--' | '-' | ':' ) ('/'?) (<-[0..9\.]> .*) $/ {
                 my ($switch, $negate, $arg) = (~$0, ?((~$1).chars), ~$2);
 
-                if $arg.index('=').defined  {
+                with $arg.index('=') {
                     my ($name, $value) = $arg.split('=', 2);
-                    $value = hack-val($value);
+                    $value = val($value);
                     $value = $value but False if $negate;
                     %named-arguments.push: $name => $value;
                 } else {
@@ -56,34 +33,38 @@ my sub MAIN_HELPER($retval = 0) is hidden_from_backtrace {
                 }
             } else {
                 @args.unshift($passed-value) unless $passed-value eq '--';
-                @positional-arguments.push: @args.map: &hack-val;
+                @positional-arguments.append: @args.map: &val;
                 last;
             }
         }
         $PROCESS::ARGFILES = IO::ArgFiles.new(:args(@args));
-        return @positional-arguments, %named-arguments;
+        @positional-arguments, %named-arguments;
     }
 
     # Generate $?USAGE string (default usage info for MAIN)
-    my sub gen-usage () {
+    my sub gen-usage() {
         my @help-msgs;
 
         my sub strip_path_prefix($name) {
-            my ($vol, $dir, $base) = IO::Spec.splitpath($name);
-            $dir = IO::Spec.canonpath($dir);
-            for IO::Spec.path() -> $elem {
-                if IO::Spec.catpath($vol, $elem, $base).IO.x {
-                    return $base if IO::Spec.canonpath($elem) eq $dir;
+            my $SPEC := $*SPEC;
+            my ($vol, $dir, $base) = $SPEC.splitpath($name);
+            $dir = $SPEC.canonpath($dir);
+            for $SPEC.path() -> $elem {
+                if $SPEC.catpath($vol, $elem, $base).IO.x {
+                    return $base if $SPEC.canonpath($elem) eq $dir;
                     # Shadowed command found in earlier PATH element
                     return $name;
                 }
             }
             # Not in PATH
-            return $name;
+            $name;
         }
 
-        my $prog-name = $*PROGRAM_NAME eq '-e' ?? "-e '...'" !! strip_path_prefix($*PROGRAM_NAME);
+        my $prog-name = $*PROGRAM-NAME eq '-e'
+          ?? "-e '...'"
+          !! strip_path_prefix($*PROGRAM-NAME);
         for $m.candidates -> $sub {
+            next if $sub.?is-hidden-from-USAGE;
             my (@required-named, @optional-named, @positional, $docs);
             for $sub.signature.params -> $param {
                 my $argument;
@@ -101,7 +82,7 @@ my sub MAIN_HELPER($retval = 0) is hidden_from_backtrace {
                 else {
                     my $constraints  = $param.constraint_list.map(*.gist).join(' ');
                     my $simple-const = $constraints && $constraints !~~ /^_block/;
-                    $argument = $param.name   ?? '<' ~ $param.name.substr(1) ~ '>' !!
+                    $argument = $param.name   ?? '<' ~ substr($param.name,1) ~ '>' !!
                                 $simple-const ??       $constraints                !!
                                                  '<' ~ $param.type.^name     ~ '>' ;
 
@@ -111,32 +92,36 @@ my sub MAIN_HELPER($retval = 0) is hidden_from_backtrace {
                 }
             }
             if $sub.WHY {
-                $docs = '-- ' ~ $sub.WHY.content
+                $docs = '-- ' ~ $sub.WHY.contents
             }
             my $msg = join(' ', $prog-name, @required-named, @optional-named, @positional, $docs // '');
             @help-msgs.push($msg);
         }
         my $usage = "Usage:\n" ~ @help-msgs.map('  ' ~ *).join("\n");
-        return $usage;
+        $usage;
     }
 
     sub has-unexpected-named-arguments($signature, %named-arguments) {
         my @named-params = $signature.params.grep: *.named;
         return False if @named-params.grep: *.slurpy;
 
-        my %accepts-argument = @named-params.map({ .named_names }) Z=> 1 xx *;
+        my %accepts-argument = @named-params.map({ .named_names.Slip }) Z=> 1 xx *;
         for %named-arguments.keys -> $name {
             return True if !%accepts-argument{$name}
         }
 
-        return False;
+        False;
     }
 
     # Process command line arguments
-    my ($p, $n) = process-cmd-args(@*ARGS).lol;
+    my ($p, $n) := process-cmd-args(@*ARGS);
 
     # Generate default $?USAGE message
-    my $?USAGE = gen-usage();
+    my $usage;
+    my $?USAGE := Proxy.new(
+        FETCH => -> | { $usage || ($usage = gen-usage()) },
+        STORE => -> | { }
+    );
 
     # Get a list of candidates that match according to the dispatcher
     my @matching_candidates = $m.cando(Capture.new(list => $p, hash => $n));
@@ -144,13 +129,17 @@ my sub MAIN_HELPER($retval = 0) is hidden_from_backtrace {
     @matching_candidates .=grep: {!has-unexpected-named-arguments($_.signature, $n)};
     # If there are still some candidates left, try to dispatch to MAIN
     if +@matching_candidates {
-        return $m(|@($p), |%($n));
+        $m(|@($p), |%($n));
+        return;
     }
 
     # We could not find the correct MAIN to dispatch to!
     # Let's try to run a user defined USAGE sub
     my $h = callframe(1).my<&USAGE>;
-    return $h() if $h;
+    if $h {
+        $h();
+        return;
+    }
 
     # We could not find a user defined USAGE sub!
     # Let's display the default USAGE message
