@@ -5,11 +5,22 @@ my class Match is Capture is Cool {
     has $.CURSOR;
     has $.made;
 
+    # new/BUILD here only for performance reasons
+    method new(:$orig,:$from,:$to,:$CURSOR,:$made) {
+        nqp::create(self).BUILD(:$orig,:$from,:$to,:$CURSOR,:$made);
+    }
+    method BUILD(:$!orig,:$from,:$to,:$!CURSOR,:$!made) {
+        $!from   = $from // 0;  # cannot assign to int in sig
+        $!to     = $to   // 0;  # cannot assign to int in sig
+        self;
+    }
+
     method ast(Match:D:) { $!made }
 
     multi method Str(Match:D:) {
-        $!to > $!from ?? $!orig.substr($!from, $!to-$!from) !! ''
+        $!to > $!from ?? substr($!orig,$!from,$!to-$!from) !! ''
     }
+
     multi method Numeric(Match:D:) {
         self.Str.Numeric
     }
@@ -19,22 +30,22 @@ my class Match is Capture is Cool {
     multi method ACCEPTS(Match:D: Any $) { self }
 
     method prematch(Match:D:) {
-        $!orig.substr(0, $!from);
+        substr($!orig,0,$!from);
     }
     method postmatch(Match:D:) {
-        $!orig.substr($!to)
+        substr($!orig,$!to)
     }
 
     method caps(Match:D:) {
         my @caps;
         for self.pairs -> $p {
-            if $p.value ~~ Array {
+            if nqp::istype($p.value,Array) {
                 @caps.push: $p.key => $_ for $p.value.list
-            } else {
-                @caps.push: $p;
+            } elsif $p.value.DEFINITE {
+                @caps.push: $p
             }
         }
-        @caps.sort: -> $p { $p.value.from }
+        @caps.sort: -> $a { $a.value.from +< 32 + $a.value.to }
     }
 
     method chunks(Match:D:) {
@@ -42,12 +53,12 @@ my class Match is Capture is Cool {
         gather {
             for self.caps {
                 if .value.from > $prev {
-                    take '~' => $!orig.substr($prev, .value.from - $prev)
+                    take '~' => substr($!orig,$prev, .value.from - $prev)
                 }
                 take $_;
                 $prev = .value.to;
             }
-            take '~' => $!orig.substr($prev, $!to - $prev) if $prev < $!to;
+            take '~' => substr($!orig,$prev, $!to - $prev) if $prev < $!to;
         }
     }
 
@@ -68,11 +79,11 @@ my class Match is Capture is Cool {
         for @.caps {
             $r ~= $s ~ (.key // '?') ~ ' ' ~ .value.gist($d + 1)
         }
-        $r;
+        $d == 0 ?? $r.chomp !! $r;
     }
 
     method make(Match:D: Mu \made) {
-        $!made = made;
+        $!made := made;
         nqp::bindattr(
             nqp::decont(self.CURSOR),
             Cursor,
@@ -83,18 +94,9 @@ my class Match is Capture is Cool {
 }
 
 sub make(Mu \made) {
-    nqp::bindattr(
-        nqp::decont(nqp::getlexcaller('$/')),
-        Match,
-        '$!made',
-        made
-    );
-    nqp::bindattr(
-        nqp::decont(nqp::getlexcaller('$/').CURSOR),
-        Cursor,
-        '$!made',
-        made
-    );
+    my $slash := nqp::getlexcaller('$/');
+    nqp::bindattr( nqp::decont($slash),        Match,  '$!made', made );
+    nqp::bindattr( nqp::decont($slash.CURSOR), Cursor, '$!made', made );
 }
 
 
