@@ -1,11 +1,28 @@
-my class X::Constructor::Positional { ... }
-my class X::Method::NotFound        { ... }
+my class X::Constructor::Positional  { ... }
+my class X::Method::NotFound         { ... }
 my class X::Method::InvalidQualifier { ... }
-my class Obsolete { ... }
+my class X::Attribute::Required      { ... }
 
 my class Mu { # declared in BOOTSTRAP
+
+    method self { self }
+
+    # XXX while the GLR is fanning out to the ecosystem
+    proto method gimme(|) {
+        die "
+The 'gimme' method was an internal method used by rakudo before the Great
+List Refactor.  The fact that you are seeing this message, means that you
+have code that was using that unsupported rakudo internal API.
+
+Please refactor this code using the new Iterator / Seq interface.
+"
+    }
+
     proto method ACCEPTS(|) { * }
-    multi method ACCEPTS(Mu:U: Mu \topic) {
+    multi method ACCEPTS(Mu:U: Any \topic) {
+        nqp::p6bool(nqp::istype(topic, self))
+    }
+    multi method ACCEPTS(Mu:U: Mu:U \topic) {
         nqp::p6bool(nqp::istype(topic, self))
     }
 
@@ -15,26 +32,52 @@ my class Mu { # declared in BOOTSTRAP
 
     proto method WHICH(|) {*}
     multi method WHICH(Mu:U:) {
-        nqp::box_s(nqp::unbox_s(self.HOW.name(self)), ObjAt);
+        nqp::box_s(nqp::unbox_s(self.^name), ObjAt);
     }
     multi method WHICH(Mu:D:) {
         nqp::box_s(
             nqp::concat(
-                nqp::concat(nqp::unbox_s(self.HOW.name(self)), '|'),
-                nqp::where(self)
+                nqp::concat(nqp::unbox_s(self.^name), '|'),
+                nqp::objectid(self)
             ),
             ObjAt
         )
     }
 
+    proto method split(|) { * }
+
     method take {
         take self;
     }
-
-    method WHY() {
-        self.HOW.docs // Any
+    method return-rw(|) {  # same code as control.pm's return-rw
+        my $list := RETURN-LIST(nqp::p6argvmarray());
+        nqp::p6routinereturn($list);
+        $list;
     }
-    
+    method return(|) {  # same code as control.pm's return
+        my $list := RETURN-LIST(nqp::p6argvmarray());
+        nqp::p6routinereturn(nqp::p6recont_ro($list));
+        $list;
+    }
+
+    proto method WHY(|) { * }
+    multi method WHY(Mu:) {
+        my Mu $why;
+
+        if nqp::can(self.HOW, 'WHY') {
+            $why := self.HOW.WHY;
+        }
+
+        if $why.defined && !$.defined #`(ie. we're a type object) {
+            $why.set_docee(self);
+        }
+        $why // Any
+    }
+
+    method set_why($why) {
+        self.HOW.set_why($why);
+    }
+
     proto method Bool(|) {*}
     multi method Bool() {
         self.defined
@@ -42,11 +85,11 @@ my class Mu { # declared in BOOTSTRAP
 
     method so()  { self.Bool }
     method not() { self ?? False !! True }
-    
+
     method defined() {
         nqp::p6bool(nqp::isconcrete(self))
     }
-    
+
     proto method new(|) { * }
     multi method new(*%) {
         nqp::invokewithcapture(nqp::findmethod(self, 'bless'), nqp::usecapture())
@@ -54,103 +97,25 @@ my class Mu { # declared in BOOTSTRAP
     multi method new($, *@) {
         X::Constructor::Positional.new(:type( self )).throw();
     }
-    
+
+    proto method is-lazy (|) { * }
+    multi method is-lazy(Mu:) { False }
+
     method CREATE() {
         nqp::create(self)
     }
-    
+
     method bless(*@autovivs, *%attrinit) {
-        if @autovivs && nqp::istype(@autovivs.at_pos(0), Whatever) {
-            DEPRECATED( "a call to bless without initial * parameter" );
-            @autovivs.shift;
-        }
         nqp::create(self).BUILDALL(@autovivs, %attrinit);
     }
-    
+
     method BUILDALL(@autovivs, %attrinit) {
         # Get the build plan. Note that we do this "low level" to
         # avoid the NQP type getting mapped to a Rakudo one, which
-        # would get expensive. Need to do it a bit differently on
-        # Parrot; it's not so 6model-y as other backends.
+        # would get expensive.
         my $build_plan := nqp::findmethod(self.HOW, 'BUILDALLPLAN')(self.HOW, self);
         my int $count   = nqp::elems($build_plan);
         my int $i       = 0;
-#?if parrot
-        while nqp::islt_i($i, $count) {
-            my $task := nqp::atpos($build_plan, $i);
-            my int $code = nqp::atpos_i($task, 0);
-            $i = nqp::add_i($i, 1);
-            if nqp::iseq_i($code, 0) {
-                # Custom BUILD call.
-                nqp::atpos($task, 1)(self, |%attrinit);
-            }
-            elsif nqp::iseq_i($code, 1) {
-                # See if we have a value to initialize this attr
-                # with.
-                my $key_name := nqp::p6box_s(nqp::atpos_s($task, 2));
-                if %attrinit.exists_key($key_name) {
-                    # XXX Should not really need the decontainerize, but seems
-                    # that slurpy hashes sometimes lead to double containers
-                    # somehow...
-                    nqp::getattr(self, nqp::atpos($task, 1),
-                        nqp::atpos($task, 3)) = nqp::decont(%attrinit{$key_name});
-                }
-            }
-            elsif nqp::iseq_i($code, 2) {
-                my $key_name := nqp::p6box_s(nqp::atpos_s($task, 2));
-                if %attrinit.exists_key($key_name) {
-                    nqp::getattr(self, nqp::atpos($task, 1),
-                        nqp::atpos_s($task, 3)) = nqp::decont(%attrinit{$key_name});
-                }
-                else {
-                    nqp::bindattr(self, nqp::atpos($task, 1),
-                        nqp::atpos_s($task, 3), nqp::list())
-                }
-            }
-            elsif nqp::iseq_i($code, 3) {
-                my $key_name := nqp::p6box_s(nqp::atpos_s($task, 2));
-                if %attrinit.exists_key($key_name) {
-                    nqp::getattr(self, nqp::atpos($task, 1),
-                        nqp::atpos_s($task, 3)) = nqp::decont(%attrinit{$key_name});
-                }
-                else {
-                    nqp::bindattr(self, nqp::atpos($task, 1),
-                        nqp::atpos_s($task, 3), nqp::hash())
-                }
-            }
-            elsif nqp::iseq_i($code, 4) {
-                unless nqp::attrinited(self, nqp::atpos($task, 1), nqp::atpos_s($task, 2)) {
-                    my \attr := nqp::getattr(self, nqp::atpos($task, 1), nqp::atpos_s($task, 2));
-                    attr = nqp::atpos($task, 3)(self, attr);
-                }
-            }
-            elsif nqp::iseq_i($code, 5) {
-                my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
-                    nqp::bindattr_i(self, nqp::atpos($task, 1), nqp::atpos($task, 3),
-                        nqp::decont(%attrinit{$key_name}));
-                }
-            }
-            elsif nqp::iseq_i($code, 6) {
-                my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
-                    nqp::bindattr_n(self, nqp::atpos($task, 1), nqp::atpos($task, 3),
-                        nqp::decont(%attrinit{$key_name}));
-                }
-            }
-            elsif nqp::iseq_i($code, 7) {
-                my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
-                    nqp::bindattr_s(self, nqp::atpos($task, 1), nqp::atpos($task, 3),
-                        nqp::decont(%attrinit{$key_name}));
-                }
-            }
-            else {
-                die "Invalid BUILDALLPLAN";
-            }
-        }
-#?endif
-#?if !parrot
         while nqp::islt_i($i, $count) {
             my $task := nqp::atpos($build_plan, $i);
             my int $code = nqp::atpos($task, 0);
@@ -163,7 +128,7 @@ my class Mu { # declared in BOOTSTRAP
                 # See if we have a value to initialize this attr
                 # with.
                 my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
+                if %attrinit.EXISTS-KEY($key_name) {
                     # XXX Should not really need the decontainerize, but seems
                     # that slurpy hashes sometimes lead to double containers
                     # somehow...
@@ -173,7 +138,7 @@ my class Mu { # declared in BOOTSTRAP
             }
             elsif nqp::iseq_i($code, 2) {
                 my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
+                if %attrinit.EXISTS-KEY($key_name) {
                     nqp::getattr(self, nqp::atpos($task, 1),
                         nqp::atpos($task, 3)) = nqp::decont(%attrinit{$key_name});
                 }
@@ -184,7 +149,7 @@ my class Mu { # declared in BOOTSTRAP
             }
             elsif nqp::iseq_i($code, 3) {
                 my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
+                if %attrinit.EXISTS-KEY($key_name) {
                     nqp::getattr(self, nqp::atpos($task, 1),
                         nqp::atpos($task, 3)) = nqp::decont(%attrinit{$key_name});
                 }
@@ -201,119 +166,68 @@ my class Mu { # declared in BOOTSTRAP
             }
             elsif nqp::iseq_i($code, 5) {
                 my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
+                if %attrinit.EXISTS-KEY($key_name) {
                     nqp::bindattr_i(self, nqp::atpos($task, 1), nqp::atpos($task, 3),
                         nqp::decont(%attrinit{$key_name}));
                 }
             }
             elsif nqp::iseq_i($code, 6) {
                 my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
+                if %attrinit.EXISTS-KEY($key_name) {
                     nqp::bindattr_n(self, nqp::atpos($task, 1), nqp::atpos($task, 3),
                         nqp::decont(%attrinit{$key_name}));
                 }
             }
             elsif nqp::iseq_i($code, 7) {
                 my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
+                if %attrinit.EXISTS-KEY($key_name) {
                     nqp::bindattr_s(self, nqp::atpos($task, 1), nqp::atpos($task, 3),
                         nqp::decont(%attrinit{$key_name}));
                 }
             }
+            elsif nqp::iseq_i($code, 8) {
+                my int $cur_value = nqp::getattr_i(self, nqp::atpos($task, 1), nqp::atpos($task, 2));
+                if nqp::iseq_i($cur_value, 0) {
+                    nqp::bindattr_i(self, nqp::atpos($task, 1), nqp::atpos($task, 2),
+                        nqp::atpos($task, 3)(self, $cur_value));
+                }
+            }
+            elsif nqp::iseq_i($code, 9) {
+                my num $cur_value = nqp::getattr_n(self, nqp::atpos($task, 1), nqp::atpos($task, 2));
+                if nqp::iseq_n($cur_value, 0e0) {
+                    nqp::bindattr_n(self, nqp::atpos($task, 1), nqp::atpos($task, 2),
+                        nqp::atpos($task, 3)(self, $cur_value));
+                }
+            }
+            elsif nqp::iseq_i($code, 10) {
+                my str $cur_value = nqp::getattr_s(self, nqp::atpos($task, 1), nqp::atpos($task, 2));
+                if nqp::isnull_s($cur_value) {
+                    nqp::bindattr_s(self, nqp::atpos($task, 1), nqp::atpos($task, 2),
+                        nqp::atpos($task, 3)(self, $cur_value));
+                }
+            }
+            elsif nqp::iseq_i($code, 11) {
+                my $attr_name = nqp::p6box_s(nqp::atpos($task, 2));
+                unless nqp::attrinited(self, nqp::atpos($task, 1), $attr_name)  {
+                    X::Attribute::Required.new(name => $attr_name).throw;
+                }
+            }
+            elsif nqp::iseq_i($code, 12) {
+                nqp::bindattr(self, nqp::atpos($task, 1), nqp::atpos($task, 2),
+                    nqp::atpos($task, 3)());
+            }
             else {
                 die "Invalid BUILDALLPLAN";
             }
         }
-#?endif
         self
     }
-    
+
     method BUILD_LEAST_DERIVED(%attrinit) {
-        # Get the build plan for just this class. Need to do it a bit
-        # differently on Parrot; it's not so 6model-y as other backends.
+        # Get the build plan for just this class.
         my $build_plan := nqp::findmethod(self.HOW, 'BUILDPLAN')(self.HOW, self);
         my int $count   = nqp::elems($build_plan);
         my int $i       = 0;
-#?if parrot
-        while nqp::islt_i($i, $count) {
-            my $task := nqp::atpos($build_plan, $i);
-            my int $code = nqp::atpos_i($task, 0);
-            $i = nqp::add_i($i, 1);
-            if nqp::iseq_i($code, 0) {
-                # Custom BUILD call.
-                nqp::atpos($task, 1)(self, |%attrinit);
-            }
-            elsif nqp::iseq_i($code, 1) {
-                # See if we have a value to initialize this attr
-                # with.
-                my $key_name := nqp::p6box_s(nqp::atpos_s($task, 2));
-                if %attrinit.exists_key($key_name) {
-                    nqp::getattr(self, nqp::atpos($task, 1),
-                        nqp::atpos_s($task, 3)) = nqp::decont(%attrinit{$key_name});
-                }
-            }
-            elsif nqp::iseq_i($code, 2) {
-                my $key_name := nqp::p6box_s(nqp::atpos_s($task, 2));
-                if %attrinit.exists_key($key_name) {
-                    nqp::getattr(self, nqp::atpos($task, 1),
-                        nqp::atpos_s($task, 3)) = nqp::decont(%attrinit{$key_name});
-                }
-                else {
-                    nqp::bindattr(self, nqp::atpos($task, 1),
-                        nqp::atpos_s($task, 3), nqp::list())
-                }
-            }
-            elsif nqp::iseq_i($code, 3) {
-                my $key_name := nqp::p6box_s(nqp::atpos_s($task, 2));
-                if %attrinit.exists_key($key_name) {
-                    nqp::getattr(self, nqp::atpos($task, 1),
-                        nqp::atpos_s($task, 3)) = nqp::decont(%attrinit{$key_name});
-                }
-                else {
-                    nqp::bindattr(self, nqp::atpos($task, 1),
-                        nqp::atpos_s($task, 3), nqp::hash())
-                }
-            }
-            elsif nqp::iseq_i($code, 4) {
-                unless nqp::attrinited(self, nqp::atpos($task, 1), nqp::atpos_s($task, 2)) {
-                    my \attr := nqp::getattr(self, nqp::atpos($task, 1), nqp::atpos_s($task, 2));
-                    attr = nqp::atpos($task, 3)(self, attr);
-                }
-            }
-            elsif nqp::iseq_i($code, 5) || nqp::iseq_i($code, 6) || nqp::iseq_i($code, 7) {
-                my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
-                    nqp::bindattr(self, nqp::atpos($task, 1), nqp::atpos($task, 3),
-                        nqp::decont(%attrinit{$key_name}));
-                }
-            }
-            elsif nqp::iseq_i($code, 5) {
-                my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
-                    nqp::bindattr_i(self, nqp::atpos($task, 1), nqp::atpos($task, 3),
-                        nqp::decont(%attrinit{$key_name}));
-                }
-            }
-            elsif nqp::iseq_i($code, 6) {
-                my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
-                    nqp::bindattr_n(self, nqp::atpos($task, 1), nqp::atpos($task, 3),
-                        nqp::decont(%attrinit{$key_name}));
-                }
-            }
-            elsif nqp::iseq_i($code, 7) {
-                my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
-                    nqp::bindattr_s(self, nqp::atpos($task, 1), nqp::atpos($task, 3),
-                        nqp::decont(%attrinit{$key_name}));
-                }
-            }
-            else {
-                die "Invalid BUILDALLPLAN";
-            }
-        }
-#?endif
-#?if !parrot
         while nqp::islt_i($i, $count) {
             my $task := nqp::atpos($build_plan, $i);
             my int $code = nqp::atpos($task, 0);
@@ -326,14 +240,14 @@ my class Mu { # declared in BOOTSTRAP
                 # See if we have a value to initialize this attr
                 # with.
                 my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
+                if %attrinit.EXISTS-KEY($key_name) {
                     nqp::getattr(self, nqp::atpos($task, 1),
                         nqp::atpos($task, 3)) = nqp::decont(%attrinit{$key_name});
                 }
             }
             elsif nqp::iseq_i($code, 2) {
                 my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
+                if %attrinit.EXISTS-KEY($key_name) {
                     nqp::getattr(self, nqp::atpos($task, 1),
                         nqp::atpos($task, 3)) = nqp::decont(%attrinit{$key_name});
                 }
@@ -344,7 +258,7 @@ my class Mu { # declared in BOOTSTRAP
             }
             elsif nqp::iseq_i($code, 3) {
                 my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
+                if %attrinit.EXISTS-KEY($key_name) {
                     nqp::getattr(self, nqp::atpos($task, 1),
                         nqp::atpos($task, 3)) = nqp::decont(%attrinit{$key_name});
                 }
@@ -361,21 +275,21 @@ my class Mu { # declared in BOOTSTRAP
             }
             elsif nqp::iseq_i($code, 5) {
                 my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
+                if %attrinit.EXISTS-KEY($key_name) {
                     nqp::bindattr_i(self, nqp::atpos($task, 1), nqp::atpos($task, 3),
                         nqp::decont(%attrinit{$key_name}));
                 }
             }
             elsif nqp::iseq_i($code, 6) {
                 my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
+                if %attrinit.EXISTS-KEY($key_name) {
                     nqp::bindattr_n(self, nqp::atpos($task, 1), nqp::atpos($task, 3),
                         nqp::decont(%attrinit{$key_name}));
                 }
             }
             elsif nqp::iseq_i($code, 7) {
                 my $key_name := nqp::p6box_s(nqp::atpos($task, 2));
-                if %attrinit.exists_key($key_name) {
+                if %attrinit.EXISTS-KEY($key_name) {
                     nqp::bindattr_s(self, nqp::atpos($task, 1), nqp::atpos($task, 3),
                         nqp::decont(%attrinit{$key_name}));
                 }
@@ -384,55 +298,67 @@ my class Mu { # declared in BOOTSTRAP
                 die "Invalid BUILDALLPLAN";
             }
         }
-#?endif
         self
     }
-    
+
     proto method Numeric(|) { * }
     multi method Numeric(Mu:U \v:) {
-        warn "use of uninitialized value of type {self.HOW.name(self)} in numeric context";
+        warn "Use of uninitialized value of type {self.^name} in numeric context";
         0
     }
     proto method Real(|) { * }
     multi method Real(Mu:U \v:) {
-        warn "use of uninitialized value of type {self.HOW.name(self)} in numeric context";
+        warn "Use of uninitialized value of type {self.^name} in numeric context";
         0
     }
-    
+
     proto method Str(|) { * }
     multi method Str(Mu:U \v:) {
-        warn "use of uninitialized value of type {self.HOW.name(self)} in string context";
+        my $name = (defined($*VAR_NAME) ?? $*VAR_NAME !! v.VAR.?name) // '';
+        $name   ~= ' ' if $name ne '';
+        warn "Use of uninitialized value {$name}of type {self.^name} in string context\n" ~
+             "Any of .^name, .perl, .gist, or .say can stringify undefined things, if needed.";
         ''
     }
     multi method Str(Mu:D:) {
-        self.HOW.name(self) ~ '<' ~ nqp::tostr_I(self.WHERE) ~ '>'
+        self.^name ~ '<' ~ nqp::tostr_I(self.WHERE) ~ '>'
     }
 
     proto method Stringy(|) { * }
-    multi method Stringy() { self.Str }
-    
-    method item(Mu \item:) is rw { item }
-    
+    multi method Stringy(Mu:U \v:) {
+        my $*VAR_NAME = v.VAR.?name;
+        self.Str
+    }
+    multi method Stringy(Mu:D $:) { self.Str }
+
+    method item(Mu \item:) is raw { item }
+
     proto method say(|) { * }
-    multi method say(Obsolete:D:) { say(self.gist) }
     multi method say() { say(self) }
     method print() { print(self) }
+    method put() { put(self) }
+    method note() { note(self) }
 
     proto method gist(|) { * }
-    multi method gist(Mu:U:) { '(' ~ self.HOW.name(self) ~ ')' }
+    multi method gist(Mu:U:) { '(' ~ self.^shortname ~ ')' }
     multi method gist(Mu:D:) { self.perl }
 
     proto method perl(|) { * }
-    multi method perl(Mu:U:) { self.HOW.name(self) }
+    multi method perl(Mu:U:) { self.^name }
     multi method perl(Mu:D:) {
+        if not %*perlseen<TOP> { my %*perlseen = :TOP ; return self.perl }
+        if %*perlseen{self.WHICH} { %*perlseen{self.WHICH} = 2; return "{self.^name}_{self.WHERE}" }
+        %*perlseen{self.WHICH} = 1;
         my @attrs;
-        for self.^attributes().grep: { .has_accessor } -> $attr {
-            my $name := $attr.Str.substr(2);
+        for self.^attributes().flat.grep: { .has_accessor } -> $attr {
+            my $name := substr($attr.Str,2);
             @attrs.push: $name
                         ~ ' => '
-                        ~ self."$name"().perl
+                        ~ $attr.get_value(self).perl
         }
-        self.HOW.name(self) ~ '.new(' ~  @attrs.join(', ') ~ ')';
+        my $result = self.^name ~ '.new' ~ ('(' ~ @attrs.join(', ') ~ ')' if @attrs);
+        $result = "(my \\{self.^name}_{self.WHERE} = $result)" if %*perlseen{self.WHICH}:delete == 2;
+        $result;
     }
 
     proto method DUMP(|) { * }
@@ -441,7 +367,7 @@ my class Mu { # declared in BOOTSTRAP
         return DUMP(self, :$indent-step) unless %ctx;
 
         my Mu $attrs := nqp::list();
-        for self.HOW.attributes(self) -> $attr {
+        for self.^attributes.flat -> $attr {
             my str $name       = $attr.name;
             my str $acc_name   = nqp::substr($name, 2, nqp::chars($name) - 2);
             my str $build_name = $attr.has_accessor ?? $acc_name !! $name;
@@ -481,7 +407,7 @@ my class Mu { # declared in BOOTSTRAP
     method DUMP-OBJECT-ATTRS(|args (*@args, :$indent-step, :%ctx, :$flags?)) {
         my Mu  $attrs := nqp::clone(nqp::captureposarg(nqp::usecapture(), 1));
         my str $where  = nqp::base_I(nqp::where(self), 16);
-        my str $before = ($flags if defined $flags) ~ self.HOW.name(self) ~ '<' ~ %ctx{$where} ~ '>(';
+        my str $before = ($flags if defined $flags) ~ self.^name ~ '<' ~ %ctx{$where} ~ '>(';
 
         my @pieces;
         while $attrs {
@@ -494,67 +420,88 @@ my class Mu { # declared in BOOTSTRAP
 
     proto method isa(|) { * }
     multi method isa(Mu \SELF: Mu $type) {
-        nqp::p6bool(SELF.HOW.isa(SELF, $type.WHAT))
+        nqp::p6bool(SELF.^isa($type.WHAT))
     }
     multi method isa(Mu \SELF: Str:D $name) {
-        my @mro = SELF.HOW.mro(SELF);
+        my @mro = SELF.^mro;
         my int $mro_count = +@mro;
         my int $i = 0;
         while $i < $mro_count {
             my $obj = @mro[$i];
-            if $obj.HOW.name($obj) eq $name {
+            if $obj.^name eq $name {
                 return Bool::True;
             }
             $i = $i + 1;
         }
         Bool::False
     }
-    
+
     method does(Mu \SELF: Mu $type) {
         nqp::p6bool(nqp::istype(SELF, $type.WHAT))
     }
-    
+
     method can(Mu \SELF: $name) {
-        SELF.HOW.can(SELF, $name)
+        SELF.^can($name)
     }
-    
+
     method clone(*%twiddles) {
         my $cloned := nqp::clone(nqp::decont(self));
-        for self.^attributes() -> $attr {
-            my $name := $attr.name;
-            my $package := $attr.package;
-            unless nqp::objprimspec($attr.type) {
-                my $attr_val := nqp::getattr($cloned, $package, $name);
-                nqp::bindattr($cloned, $package, $name, nqp::clone($attr_val.VAR))
-                    if nqp::iscont($attr_val);
+        if %twiddles.elems {
+            for self.^attributes.flat -> $attr {
+                my $name    := $attr.name;
+                my $package := $attr.package;
+#?if jvm
+                unless nqp::objprimspec($attr.type) {
+                    my $attr_val = nqp::getattr($cloned, $package, $name);
+                    nqp::bindattr($cloned, $package, $name, nqp::clone($attr_val.VAR));
+                }
+#?endif
+#?if !jvm
+                nqp::bindattr($cloned, $package, $name,
+                  nqp::clone(nqp::getattr($cloned, $package, $name).VAR)
+                ) unless nqp::objprimspec($attr.type);
+#?endif
+
+                my $acc_name := substr($name,2);
+                nqp::getattr($cloned, $package, $name) =
+                  nqp::decont(%twiddles{$acc_name})
+                  if $attr.has-accessor && %twiddles.EXISTS-KEY($acc_name);
             }
-            my $acc_name := $name.substr(2);
-            if $attr.has-accessor && %twiddles.exists_key($acc_name) {
-                nqp::getattr($cloned, $package, $name) = %twiddles{$acc_name};
+        }
+        else {
+            for self.^attributes.flat -> $attr {
+                unless nqp::objprimspec($attr.type) {
+                    my $name     := $attr.name;
+                    my $package  := $attr.package;
+                    my $attr_val := nqp::getattr($cloned, $package, $name);
+                    nqp::bindattr($cloned,
+                      $package, $name, nqp::clone($attr_val.VAR))
+                        if nqp::iscont($attr_val);
+                }
             }
         }
         $cloned
     }
-    
+
     method Capture() {
         my %attrs;
-        for self.^attributes -> $attr {
+        for self.^attributes.flat -> $attr {
             if $attr.has-accessor {
-                my $name = $attr.name.substr(2);
-                unless %attrs.exists_key($name) {
+                my $name = substr($attr.name,2);
+                unless %attrs.EXISTS-KEY($name) {
                     %attrs{$name} = self."$name"();
                 }
             }
         }
         %attrs.Capture
     }
-    
+
     # XXX TODO: Handle positional case.
-    method dispatch:<var>(Mu \SELF: $var, |c) is rw is hidden_from_backtrace {
+    method dispatch:<var>(Mu \SELF: $var, |c) is raw {
         $var(SELF, |c)
     }
-    
-    method dispatch:<::>(Mu \SELF: $name, Mu $type, |c) is rw {
+
+    method dispatch:<::>(Mu \SELF: $name, Mu $type, |c) is raw {
         unless nqp::istype(SELF, $type) {
             X::Method::InvalidQualifier.new(
                     method          => $name,
@@ -563,73 +510,83 @@ my class Mu { # declared in BOOTSTRAP
 
             ).throw;
         }
-        self.HOW.find_method_qualified(self, $type, $name)(SELF, |c)
+        self.^find_method_qualified($type, $name)(SELF, |c)
     }
-    
-    method dispatch:<!>(Mu \SELF: $name, Mu $type, |c) is rw is hidden_from_backtrace {
-        my $meth := $type.HOW.find_private_method($type, $name);
+
+    method dispatch:<!>(Mu \SELF: $name, Mu $type, |c) is raw {
+        my $meth := $type.^find_private_method($name);
         $meth ??
             $meth(SELF, |c) !!
             X::Method::NotFound.new(
               invocant => SELF,
               method   => '!' ~ $name,
-              typename => $type.HOW.name($type),
+              typename => $type.^name,
               :private,
             ).throw;
     }
-    
-    method dispatch:<.^>(Mu \SELF: $name, |c) is rw is hidden_from_backtrace {
+
+    method dispatch:<.^>(Mu \SELF: $name, |c) is raw {
         self.HOW."$name"(SELF, |c)
     }
-    
-    method dispatch:<.=>(\mutate: $name, |c) is rw {
+
+    method dispatch:<.=>(\mutate: $name, |c) is raw {
         $/ := nqp::getlexcaller('$/');
         mutate = mutate."$name"(|c)
     }
-    
-    method dispatch:<.?>(Mu \SELF: $name, |c) is rw is hidden_from_backtrace {
+
+    method dispatch:<.?>(Mu \SELF: $name, |c) is raw {
         nqp::can(SELF, $name) ??
             SELF."$name"(|c) !!
             Nil
     }
-    
+
     method dispatch:<.+>(Mu \SELF: $name, |c) {
         my @result := SELF.dispatch:<.*>($name, |c);
         if @result.elems == 0 {
             X::Method::NotFound.new(
               invocant => SELF,
               method   => $name,
-              typename => SELF.HOW.name(SELF),
+              typename => SELF.^name,
             ).throw;
         }
         @result
     }
-    
+
     method dispatch:<.*>(Mu \SELF: $name, |c) {
-        my @mro = SELF.HOW.mro(SELF);
+        my @mro = SELF.^mro;
         my int $mro_count = +@mro;
         my @results;
         my int $i = 0;
         while $i < $mro_count {
             my $obj = @mro[$i];
-            my $meth = ($obj.HOW.method_table($obj)){$name};
+            my $meth = ($obj.^method_table){$name};
             if !$meth && $i == 0 {
-                $meth = ($obj.HOW.submethod_table($obj)){$name};
+                $meth = ($obj.^submethod_table){$name};
             }
             if $meth {
                 @results.push($meth(SELF, |c));
             }
             $i = $i + 1;
         }
-        &infix:<,>(|@results)
+        my $list := nqp::create(List);
+        nqp::bindattr($list, List, '$!reified', nqp::getattr(@results, List, '$!reified'));
+        $list
     }
 
     method dispatch:<hyper>(Mu \SELF: $name, |c) {
-        c
-            ?? hyper( -> \obj { obj."$name"(|c) }, SELF )
-            !! hyper( -> \obj { obj."$name"() }, SELF )
+        my $listcan = List.can($name);
+        if $listcan and $listcan[0].?nodal {
+            c
+                ?? HYPER( sub (\obj) is nodal { obj."$name"(|c) }, SELF )
+                !! HYPER( sub (\obj) is nodal { obj."$name"() }, SELF )
+        }
+        else {
+            c
+                ?? HYPER( -> \obj { obj."$name"(|c) }, SELF )
+                !! HYPER( -> \obj { obj."$name"() }, SELF )
+        }
     }
-    
+
     method WALK(:$name!, :$canonical, :$ascendant, :$descendant, :$preorder, :$breadth,
                 :$super, :$omit, :$include) {
         # First, build list of classes in the order we'll need them.
@@ -640,10 +597,10 @@ my class Mu { # declared in BOOTSTRAP
         elsif $breadth {
             my @search_list = self.WHAT;
             while @search_list {
-                push @classes, @search_list;
+                append @classes, @search_list;
                 my @new_search_list;
                 for @search_list -> $current {
-                    for $current.^parents(:local) -> $next {
+                    for flat $current.^parents(:local) -> $next {
                         unless @new_search_list.grep({ $^c.WHAT =:= $next.WHAT }) {
                             push @new_search_list, $next;
                         }
@@ -655,7 +612,7 @@ my class Mu { # declared in BOOTSTRAP
             sub build_ascendent(Mu $class) {
                 unless @classes.grep({ $^c.WHAT =:= $class.WHAT }) {
                     push @classes, $class;
-                    for $class.^parents(:local) {
+                    for flat $class.^parents(:local) {
                         build_ascendent($^parent);
                     }
                 }
@@ -664,7 +621,7 @@ my class Mu { # declared in BOOTSTRAP
         } elsif $descendant {
             sub build_descendent(Mu $class) {
                 unless @classes.grep({ $^c.WHAT =:= $class.WHAT }) {
-                    for $class.^parents(:local) {
+                    for flat $class.^parents(:local) {
                         build_descendent($^parent);
                     }
                     push @classes, $class;
@@ -683,7 +640,7 @@ my class Mu { # declared in BOOTSTRAP
             if (!defined($include) || $include.ACCEPTS($class)) &&
               (!defined($omit) || !$omit.ACCEPTS($class)) {
                 try {
-                    for $class.^methods(:local) -> $method {
+                    for flat $class.^methods(:local) -> $method {
                         my $check_name = $method.?name;
                         if $check_name.defined && $check_name eq $name {
                             @methods.push($method);
@@ -694,7 +651,7 @@ my class Mu { # declared in BOOTSTRAP
             }
         }
 
-        return @methods;
+        @methods;
     }
 }
 
@@ -702,33 +659,38 @@ my class Mu { # declared in BOOTSTRAP
 proto sub defined(Mu) is pure { * }
 multi sub defined(Mu \x) { x.defined }
 
-proto sub infix:<~~>(|) { * }
+proto sub infix:<~~>(Mu \topic, Mu \matcher) { * }
 multi sub infix:<~~>(Mu \topic, Mu \matcher) {
     matcher.ACCEPTS(topic).Bool;
 }
 
-proto sub infix:<=:=>(Mu $?, Mu $?) { * }
+proto sub infix:<=:=>(Mu $?, Mu $?) is pure { * }
 multi sub infix:<=:=>($?)      { Bool::True }
-multi sub infix:<=:=>(Mu \a, Mu \b) { 
+multi sub infix:<=:=>(Mu \a, Mu \b) {
     nqp::p6bool(nqp::eqaddr(a, b));
 }
 
-proto sub infix:<eqv>(Any $?, Any $?) { * }
+proto sub infix:<eqv>(Any $?, Any $?) is pure { * }
 multi sub infix:<eqv>($?)            { Bool::True }
 multi sub infix:<eqv>(Any $a, Any $b) {
-    $a.WHICH eq $b.WHICH
+    # Last ditch snapshot semantics.  We shouldn't come here too often, so
+    # please do not change this to be faster but wronger.  (Instead, add
+    # specialized multis for datatypes that can be tested piecemeal.)
+    $a.WHAT === $b.WHAT and $a.perl eq $b.perl;
 }
 
 multi sub infix:<eqv>(@a, @b) {
-    unless @a.WHAT === @b.WHAT && (my int $n = @a.elems) == @b.elems {
-        return Bool::False
+    if @a.WHAT === @b.WHAT && (my int $n = @a.elems) == @b.elems {
+        my int $i;
+        while $i < $n {
+            return Bool::False unless @a.AT-POS($i) eqv @b.AT-POS($i);
+            $i = $i + 1;
+        }
+        Bool::True
     }
-    my int $i = 0;
-    while $i < $n {
-        return Bool::False unless @a.at_pos($i) eqv @b.at_pos($i);
-        $i = $i + 1;
+    else {
+        Bool::False;
     }
-    Bool::True
 }
 
 sub DUMP(|args (*@args, :$indent-step = 4, :%ctx?)) {
@@ -739,15 +701,7 @@ sub DUMP(|args (*@args, :$indent-step = 4, :%ctx?)) {
         if nqp::iscont($topic);
     return '(null)' if nqp::isnull($topic);
 
-    # On Parrot, not everything is a 6model object, so use the typeof op to
-    # get a real type name. On other platforms, .HOW.name(...) can be relied
-    # on to work.
-#?if parrot
-    my str $type  = pir::typeof__SP($topic);
-#?endif
-#?if !parrot
-    my str $type  = $topic.HOW.name($topic);
-#?endif
+    my str $type  = $topic.^name;
     my str $where = nqp::base_I(nqp::where($topic), 16);
 
     if %ctx{$where} -> $obj_num {
@@ -761,9 +715,6 @@ sub DUMP(|args (*@args, :$indent-step = 4, :%ctx?)) {
         %ctx{$where} = $obj_num;
 
         if    nqp::islist($topic) {
-#?if parrot
-            $type = 'RPA' if $type eq 'ResizablePMCArray';
-#?endif
             my str $id = $type ~ '<' ~ $obj_num ~ '>';
 
             my @pieces;
@@ -780,7 +731,7 @@ sub DUMP(|args (*@args, :$indent-step = 4, :%ctx?)) {
 
             my @pieces;
             {
-                for $topic {
+                for $topic.pairs {
                     @pieces.push: $_.key ~ ' => ' ~ DUMP($_.value, :$indent-step, :%ctx);
                 }
                 CATCH { default { @pieces.push: '...' } }
@@ -802,6 +753,12 @@ sub DUMP(|args (*@args, :$indent-step = 4, :%ctx?)) {
     }
 }
 
+# These must collapse Junctions
+proto sub so(Mu $) {*}
+multi sub so(Mu $x)  { ?$x }
+proto sub not(Mu $) {*}
+multi sub not(Mu $x) { !$x }
+
 Metamodel::ClassHOW.exclude_parent(Mu);
 
 # vim: ft=perl6 expandtab sw=4
