@@ -15,7 +15,8 @@ my class Num does Real { # declared in BOOTSTRAP
     }
     method Num() { self }
     method Bridge(Num:D:) { self }
-    
+    method Range(Num:U:) { Range.new(-Inf,Inf) }
+
     method Int(Num:D:) {
         nqp::isnanorinf(nqp::unbox_n(self)) ??
             fail("Cannot coerce Inf or NaN to an Int") !!
@@ -24,7 +25,7 @@ my class Num does Real { # declared in BOOTSTRAP
 
     multi method new() { nqp::box_n(0e0, self) }
     multi method new($n) { nqp::box_n($n.Num, self) }
-    
+
     multi method perl(Num:D:) {
         my $res = self.Str;
         if nqp::isnanorinf(nqp::unbox_n(self))
@@ -41,26 +42,33 @@ my class Num does Real { # declared in BOOTSTRAP
             return self;
         }
 
-        my sub modf($num) { my $q = $num.Int; $num - $q, $q; }
-
         (self == Inf || self == -Inf) && fail("Cannot coerce Inf to a Rat");
-        
+
         my Num $num = self;
         my Int $signum = $num < 0 ?? -1 !! 1;
         $num = -$num if $signum == -1;
 
         # Find convergents of the continued fraction.
 
-        my Num $r = $num - $num.Int;
-        my Int $q = $num.Int;
-        my ($a, $b) = 1, $q;
-        my ($c, $d) = 0, 1;
+        my Int $q = nqp::fromnum_I($num, Int);
+        my num $r = $num - floor($num);
+        my Int $a = 1;
+        my Int $b = $q;
+        my Int $c = 0;
+        my Int $d = 1;
 
-        while $r != 0 && abs($num - ($b/$d)) > $epsilon {
-            ($r, $q) = modf(1/$r);
+        while $r != 0e0 && abs($num - ($b / $d)) > $epsilon {
+            my num $modf_arg = 1e0 / $r;
+            $q = nqp::fromnum_I($modf_arg, Int);
+            $r = $modf_arg - floor($modf_arg);
 
-            ($a, $b) = ($b, $q*$b + $a);
-            ($c, $d) = ($d, $q*$d + $c);
+            my $orig_b = $b;
+            $b = $q * $b + $a;
+            $a = $orig_b;
+
+            my $orig_d = $d;
+            $d = $q * $d + $c;
+            $c = $orig_d;
         }
 
         # Note that this result has less error than any Rational with a
@@ -229,41 +237,59 @@ my class Num does Real { # declared in BOOTSTRAP
 my constant pi = 3.14159_26535_89793_238e0;
 my constant e  = 2.71828_18284_59045_235e0;
 
-multi prefix:<++>(Num:D \a is rw) {   # XXX
-    a = nqp::p6box_n(nqp::add_n(nqp::unbox_n(a), 1e0))
+my constant π := pi;
+
+multi sub prefix:<++>(Num:D $a is rw) {
+    $a = nqp::p6box_n(nqp::add_n(nqp::unbox_n($a), 1e0))
 }
-multi prefix:<++>(Num:U \a is rw) {   # XXX
-    a = 1e0;
+multi sub prefix:<++>(Num:U $a is rw) {
+    $a = 1e0;
 }
-multi prefix:<-->(Num:D \a is rw) {   # XXX
-    a = nqp::p6box_n(nqp::sub_n(nqp::unbox_n(a), 1e0))
+multi sub prefix:<++>(num $a is rw) {
+    $a = nqp::add_n($a, 1e0)
 }
-multi prefix:<-->(Num:U \a is rw) {   # XXX
-    a = -1e0;
+multi sub prefix:<-->(Num:D $a is rw) {
+    $a = nqp::p6box_n(nqp::sub_n(nqp::unbox_n($a), 1e0))
 }
-multi postfix:<++>(Num:D \a is rw) {  # XXX
-    my $b = a;
-    a = nqp::p6box_n(nqp::add_n(nqp::unbox_n(a), 1e0));
+multi sub prefix:<-->(Num:U $a is rw) {
+    $a = -1e0;
+}
+multi sub prefix:<-->(num $a is rw) {
+    $a = nqp::sub_n($a, 1e0)
+}
+multi sub postfix:<++>(Num:D $a is rw) {
+    my $b = $a;
+    $a = nqp::p6box_n(nqp::add_n(nqp::unbox_n($a), 1e0));
     $b
 }
-multi postfix:<++>(Num:U \a is rw) {   # XXX
-    a = 1e0;
+multi sub postfix:<++>(Num:U $a is rw) {
+    $a = 1e0;
     0
 }
-multi postfix:<-->(Num:D \a is rw) {  # XXX
-    my $b = a;
-    a = nqp::p6box_n(nqp::sub_n(nqp::unbox_n(a), 1e0));
+multi sub postfix:<++>(num $a is rw) {
+    my num $b = $a;
+    $a = nqp::add_n($a, 1e0);
     $b
 }
-multi postfix:<-->(Num:U \a is rw) {   # XXX
-    a = -1e0;
+multi sub postfix:<-->(Num:D $a is rw) {
+    my $b = $a;
+    $a = nqp::p6box_n(nqp::sub_n(nqp::unbox_n($a), 1e0));
+    $b
+}
+multi sub postfix:<-->(Num:U $a is rw) {
+    $a = -1e0;
     0e0
 }
+multi sub postfix:<-->(num $a is rw) {
+    my num $b = $a;
+    $a = nqp::sub_n($a, 1e0);
+    $b
+}
 
-multi prefix:<->(Num:D \a) {
+multi sub prefix:<->(Num:D \a) {
     nqp::p6box_n(nqp::neg_n(nqp::unbox_n(a)))
 }
-multi prefix:<->(num $a) {
+multi sub prefix:<->(num $a) {
     nqp::neg_n($a);
 }
 
@@ -274,110 +300,121 @@ multi sub abs(num $a) {
     nqp::abs_n($a)
 }
 
-multi infix:<+>(Num:D \a, Num:D \b) {
+multi sub infix:<+>(Num:D \a, Num:D \b) {
     nqp::p6box_n(nqp::add_n(nqp::unbox_n(a), nqp::unbox_n(b)))
 }
-multi infix:<+>(num $a, num $b) {
+multi sub infix:<+>(num $a, num $b) {
     nqp::add_n($a, $b)
 }
 
-multi infix:<->(Num:D \a, Num:D \b) {
+multi sub infix:<->(Num:D \a, Num:D \b) {
     nqp::p6box_n(nqp::sub_n(nqp::unbox_n(a), nqp::unbox_n(b)))
 }
-multi infix:<->(num $a, num $b) {
+multi sub infix:<->(num $a, num $b) {
     nqp::sub_n($a, $b)
 }
 
-multi infix:<*>(Num:D \a, Num:D \b) {
+multi sub infix:<*>(Num:D \a, Num:D \b) {
     nqp::p6box_n(nqp::mul_n(nqp::unbox_n(a), nqp::unbox_n(b)))
 }
-multi infix:<*>(num $a, num $b) {
+multi sub infix:<*>(num $a, num $b) {
     nqp::mul_n($a, $b)
 }
 
-multi infix:</>(Num:D \a, Num:D \b) {
-    fail X::Numeric::DivideByZero.new unless b;
+multi sub infix:</>(Num:D \a, Num:D \b) {
+    fail X::Numeric::DivideByZero.new(
+      using => '/', numerator => a
+    ) unless b;
     nqp::p6box_n(nqp::div_n(nqp::unbox_n(a), nqp::unbox_n(b)))
 }
-multi infix:</>(num $a, num $b) {
-    fail X::Numeric::DivideByZero.new unless $b;
+multi sub infix:</>(num $a, num $b) {
+    fail X::Numeric::DivideByZero.new(
+      using => '/', numerator => $a
+    ) unless $b;
     nqp::div_n($a, $b)
 }
 
-multi infix:<%>(Num:D \a, Num:D \b) {
-    fail X::Numeric::DivideByZero.new(:using<%>) unless b;
+multi sub infix:<%>(Num:D \a, Num:D \b) {
+    fail X::Numeric::DivideByZero.new(
+      using => '%', numerator => a
+    ) unless b;
     nqp::p6box_n(nqp::mod_n(nqp::unbox_n(a), nqp::unbox_n(b)))
 }
-multi infix:<%>(num $a, num $b) {
-    fail X::Numeric::DivideByZero.new(:using<%>) unless $b;
+multi sub infix:<%>(num $a, num $b) {
+    fail X::Numeric::DivideByZero.new(
+      using => '%', numerator => $a
+    ) unless $b;
     nqp::mod_n($a, $b)
 }
 
-multi infix:<**>(Num:D \a, Num:D \b) {
+multi sub infix:<**>(Num:D \a, Num:D \b) {
     nqp::p6box_n(nqp::pow_n(nqp::unbox_n(a), nqp::unbox_n(b)))
 }
-multi infix:<**>(num $a, num $b) {
+multi sub infix:<**>(num $a, num $b) {
     nqp::pow_n($a, $b)
 }
 
 
-multi infix:<cmp>(Num:D \a, Num:D \b) {
+multi sub infix:<cmp>(Num:D \a, Num:D \b) {
     ORDER(nqp::cmp_n(nqp::unbox_n(a), nqp::unbox_n(b)))
 }
-multi infix:<cmp>(num $a, num $b) {
+multi sub infix:<cmp>(num $a, num $b) {
     ORDER(nqp::cmp_n($a, $b))
 }
 
-multi infix:«<=>»(Num:D \a, Num:D \b) {
+multi sub infix:«<=>»(Num:D \a, Num:D \b) {
     ORDER(nqp::cmp_n(nqp::unbox_n(a), nqp::unbox_n(b)))
 }
-multi infix:«<=>»(num $a, num $b) {
+multi sub infix:«<=>»(num $a, num $b) {
     ORDER(nqp::cmp_n($a, $b))
 }
 
-multi infix:<===>(Num:D \a, Num:D \b) {
-    nqp::p6bool(nqp::iseq_n(nqp::unbox_n(a), nqp::unbox_n(b)))
+multi sub infix:<===>(Num:D \a, Num:D \b) {
+    a.WHAT =:= b.WHAT && nqp::p6bool(nqp::iseq_n(nqp::unbox_n(a), nqp::unbox_n(b)))
 }
-multi infix:<===>(num $a, num $b) returns Bool:D {
+multi sub infix:<===>(NaN, NaN) {
+    True;
+}
+multi sub infix:<===>(num $a, num $b) returns Bool:D {
     nqp::p6bool(nqp::iseq_n($a, $b))
 }
 
-multi infix:<==>(Num:D \a, Num:D \b) returns Bool:D  {
+multi sub infix:<==>(Num:D \a, Num:D \b) returns Bool:D  {
     nqp::p6bool(nqp::iseq_n(nqp::unbox_n(a), nqp::unbox_n(b)))
 }
-multi infix:<==>(num $a, num $b) returns Bool:D  {
+multi sub infix:<==>(num $a, num $b) returns Bool:D  {
     nqp::p6bool(nqp::iseq_n($a, $b))
 }
 
-multi infix:<!=>(num $a, num $b) returns Bool:D {
+multi sub infix:<!=>(num $a, num $b) returns Bool:D {
     nqp::p6bool(nqp::isne_n($a, $b))
 }
 
-multi infix:«<»(Num:D \a, Num:D \b) returns Bool:D {
+multi sub infix:«<»(Num:D \a, Num:D \b) returns Bool:D {
     nqp::p6bool(nqp::islt_n(nqp::unbox_n(a), nqp::unbox_n(b)))
 }
-multi infix:«<»(num $a, num $b) returns Bool:D {
+multi sub infix:«<»(num $a, num $b) returns Bool:D {
     nqp::p6bool(nqp::islt_n($a, $b))
 }
 
-multi infix:«<=»(Num:D \a, Num:D \b) returns Bool:D {
+multi sub infix:«<=»(Num:D \a, Num:D \b) returns Bool:D {
     nqp::p6bool(nqp::isle_n(nqp::unbox_n(a), nqp::unbox_n(b)))
 }
-multi infix:«<=»(num $a, num $b) returns Bool:D {
+multi sub infix:«<=»(num $a, num $b) returns Bool:D {
     nqp::p6bool(nqp::isle_n($a, $b))
 }
 
-multi infix:«>»(Num:D \a, Num:D \b) returns Bool:D {
+multi sub infix:«>»(Num:D \a, Num:D \b) returns Bool:D {
     nqp::p6bool(nqp::isgt_n(nqp::unbox_n(a), nqp::unbox_n(b)))
 }
-multi infix:«>»(num $a, num $b) returns Bool:D {
+multi sub infix:«>»(num $a, num $b) returns Bool:D {
     nqp::p6bool(nqp::isgt_n($a, $b))
 }
 
-multi infix:«>=»(Num:D \a, Num:D \b) returns Bool:D {
+multi sub infix:«>=»(Num:D \a, Num:D \b) returns Bool:D {
     nqp::p6bool(nqp::isge_n(nqp::unbox_n(a), nqp::unbox_n(b)))
 }
-multi infix:«>=»(num $a, num $b) returns Bool:D {
+multi sub infix:«>=»(num $a, num $b) returns Bool:D {
     nqp::p6bool(nqp::isge_n($a, $b))
 }
 
@@ -474,4 +511,14 @@ multi sub acotanh(num $x) {
     atanh(1e0 / $x)
 }
 
+multi sub floor(num $a) returns num {
+    nqp::floor_n($a)
+}
+multi sub ceiling(num $a) returns num {
+    nqp::ceil_n($a)
+}
+multi sub sqrt(num $a) returns num {
+    nqp::sqrt_n($a)
+}
+
 # vim: ft=perl6 expandtab sw=4
