@@ -4,8 +4,8 @@ my class X::Numeric::DivideByZero { ... }
 my role Numeric {
     multi method Numeric(Numeric:D:) { self }
 
-    multi method ACCEPTS(Numeric:D: $a) {
-        self.isNaN ?? $a.isNaN !! $a == self;
+    multi method ACCEPTS(Numeric:D: \a) {
+        self.isNaN ?? a.defined && a.isNaN !! a == self;
     }
 
     proto method log(|) {*}
@@ -27,8 +27,6 @@ my role Numeric {
 
     method succ() { self + 1 }
     method pred() { self - 1 }
-
-    method sleep() { sleep self }
 }
 
 multi sub infix:<eqv>(Numeric:D $a, Numeric:D $b) {
@@ -37,11 +35,11 @@ multi sub infix:<eqv>(Numeric:D $a, Numeric:D $b) {
 
 ## arithmetic operators
 
-proto prefix:<+>($?) is pure { * }
-multi prefix:<+>(\a) { a.Numeric }
+proto sub prefix:<+>($?) is pure { * }
+multi sub prefix:<+>(\a) { a.Numeric }
 
-proto prefix:<->($?) is pure { * }
-multi prefix:<->(\a) { -a.Numeric }
+proto sub prefix:<->($?) is pure { * }
+multi sub prefix:<->(\a) { -a.Numeric }
 
 proto sub abs($) is pure { * }
 multi sub abs(\a) { abs a.Numeric }
@@ -181,109 +179,111 @@ multi sub round($a)                 { $a.Numeric.round }
 multi sub round(Numeric $a)         { $a.round }
 multi sub round(Numeric $a, $scale) { $a.round($scale) }
 
-proto infix:<+>($?, $?) is pure   { * }
-multi infix:<+>($x = 0)      { $x.Numeric }
-multi infix:<+>(\a, \b)    { a.Numeric + b.Numeric }
+proto sub infix:<+>(Mu $?, Mu $?) is pure   { * }
+multi sub infix:<+>($x = 0)      { $x.Numeric }
+multi sub infix:<+>(\a, \b)    { a.Numeric + b.Numeric }
 
-proto infix:<->($?, $?) is pure   { * }
-multi infix:<->($x = 0)      { $x.Numeric }
-multi infix:<->(\a, \b)    { a.Numeric - b.Numeric }
+proto sub infix:<->(Mu $?, Mu $?) is pure   { * }
+multi sub infix:<->($x = 0)      { $x.Numeric }
+multi sub infix:<->(\a, \b)    { a.Numeric - b.Numeric }
 
-proto infix:<*>($?, $?) is pure   { * }
-multi infix:<*>($x = 1)      { $x.Numeric }
-multi infix:<*>(\a, \b)    { a.Numeric * b.Numeric }
+proto sub infix:<*>(Mu $?, Mu $?) is pure   { * }
+multi sub infix:<*>($x = 1)      { $x.Numeric }
+multi sub infix:<*>(\a, \b)    { a.Numeric * b.Numeric }
 
-proto infix:</>($?, $?) { * }
-multi infix:</>()            { fail "No zero-arg meaning for infix:</>" }
-multi infix:</>($x)          { $x.Numeric }
-multi infix:</>(\a, \b)    { a.Numeric / b.Numeric }
+proto sub infix:</>(Mu $?, Mu $?) { * }
+multi sub infix:</>()            { fail "No zero-arg meaning for infix:</>" }
+multi sub infix:</>($x)          { $x.Numeric }
+multi sub infix:</>(\a, \b)    { a.Numeric / b.Numeric }
 
-proto infix:<div>($?, $?) is pure  { * }
+proto sub infix:<div>(Mu $?, Mu $?) is pure  { * }
 # rest of infix:<div> is in Int.pm
 
-proto infix:<%>($?, $?) is pure   { * }
-multi infix:<%>()            { fail "No zero-arg meaning for infix:<%>" }
-multi infix:<%>($x)          { $x }
-multi infix:<%>(\a, \b)    { a.Real % b.Real }
+proto sub infix:<%>(Mu $?, Mu $?) is pure   { * }
+multi sub infix:<%>()            { fail "No zero-arg meaning for infix:<%>" }
+multi sub infix:<%>($x)          { $x }
+multi sub infix:<%>(\a, \b)    { a.Real % b.Real }
 
-proto infix:<%%>($?, $?) is pure  { * }
-multi infix:<%%>()           { fail "No zero-arg meaning for infix:<%%>" }
-multi infix:<%%>($)         { Bool::True }
-multi infix:<%%>(\a, \b)   {
-    fail X::Numeric::DivideByZero.new(using => 'infix:<%%>') unless b;
+proto sub infix:<%%>(Mu $?, Mu $?) is pure  { * }
+multi sub infix:<%%>()           { fail "No zero-arg meaning for infix:<%%>" }
+multi sub infix:<%%>($)         { Bool::True }
+multi sub infix:<%%>(\a, \b)   {
+    fail X::Numeric::DivideByZero.new(
+      using => 'infix:<%%>', numerator => a
+    ) unless b;
     a.Real % b.Real == 0;
 }
 
-proto infix:<lcm>($?, $?) is pure  { * }
-multi infix:<lcm>(Int $x = 1) { $x }
-multi infix:<lcm>(\a, \b)   { a.Int lcm b.Int }
+proto sub infix:<lcm>(Mu $?, Mu $?) is pure  { * }
+multi sub infix:<lcm>(Int $x = 1) { $x }
+multi sub infix:<lcm>(\a, \b)   { a.Int lcm b.Int }
 
-proto infix:<gcd>($?, $?) is pure { * }
-multi infix:<gcd>()          { fail 'No zero-arg meaning for infix:<gcd>' }
-multi infix:<gcd>(Int $x)    { $x }
-multi infix:<gcd>(\a, \b)  { a.Int gcd b.Int }
+proto sub infix:<gcd>(Mu $?, Mu $?) is pure { * }
+multi sub infix:<gcd>()          { fail 'No zero-arg meaning for infix:<gcd>' }
+multi sub infix:<gcd>(Int $x)    { $x }
+multi sub infix:<gcd>(\a, \b)  { a.Int gcd b.Int }
 
-proto infix:<**>($?, $?) is pure  { * }
-multi infix:<**>($x = 1)     { $x.Numeric }
-multi infix:<**>(\a, \b)   { a.Numeric ** b.Numeric }
+proto sub infix:<**>(Mu $?, Mu $?) is pure  { * }
+multi sub infix:<**>($x = 1)     { $x.Numeric }
+multi sub infix:<**>(\a, \b)   { a.Numeric ** b.Numeric }
 
 ## relational operators
 
-proto infix:«<=>»($, $?) is pure       { * }
-multi infix:«<=>»(\a, \b)  { a.Real <=> b.Real }
+proto sub infix:«<=>»(Mu $, Mu $?) is pure       { * }
+multi sub infix:«<=>»(\a, \b)  { a.Real <=> b.Real }
 
-proto infix:<==>($?, $?) is pure  { * }
-multi infix:<==>($?)        { Bool::True }
-multi infix:<==>(\a, \b)   { a.Numeric == b.Numeric }
+proto sub infix:<==>(Mu $?, Mu $?) is pure  { * }
+multi sub infix:<==>($?)        { Bool::True }
+multi sub infix:<==>(\a, \b)   { a.Numeric == b.Numeric }
 
-proto infix:<!=>(Mu $?, Mu $?) is pure  { * }
-multi infix:<!=>($?)        { Bool::True }
-multi infix:<!=>(Mu \a, Mu \b)   { not a == b }
+proto sub infix:<!=>(Mu $?, Mu $?) is pure  { * }
+multi sub infix:<!=>($?)        { Bool::True }
+multi sub infix:<!=>(Mu \a, Mu \b)   { not a == b }
 
-proto infix:«<»($?, $?) is pure   { * }
-multi infix:«<»($?)         { Bool::True }
-multi infix:«<»(\a, \b)    { a.Real < b.Real }
+proto sub infix:«<»(Mu $?, Mu $?) is pure   { * }
+multi sub infix:«<»($?)         { Bool::True }
+multi sub infix:«<»(\a, \b)    { a.Real < b.Real }
 
-proto infix:«<=»($?, $?) is pure  { * }
-multi infix:«<=»($?)        { Bool::True }
-multi infix:«<=»(\a, \b)   { a.Real <= b.Real }
+proto sub infix:«<=»(Mu $?, Mu $?) is pure  { * }
+multi sub infix:«<=»($?)        { Bool::True }
+multi sub infix:«<=»(\a, \b)   { a.Real <= b.Real }
 
-proto infix:«>»($?, $?) is pure   { * }
-multi infix:«>»($?)         { Bool::True }
-multi infix:«>»(\a, \b)    { a.Real > b.Real }
+proto sub infix:«>»(Mu $?, Mu $?) is pure   { * }
+multi sub infix:«>»($?)         { Bool::True }
+multi sub infix:«>»(\a, \b)    { a.Real > b.Real }
 
-proto infix:«>=»($?, $?) is pure  { * }
-multi infix:«>=»($?)        { Bool::True }
-multi infix:«>=»(\a, \b)   { a.Real >= b.Real }
+proto sub infix:«>=»(Mu $?, Mu $?) is pure  { * }
+multi sub infix:«>=»($?)        { Bool::True }
+multi sub infix:«>=»(\a, \b)   { a.Real >= b.Real }
 
 ## bitwise operators
 
-proto infix:<+&>($?, $?) is pure { * }
-multi infix:<+&>()           { +^0 }
-multi infix:<+&>($x)         { $x }
-multi infix:<+&>($x, $y)     { $x.Numeric.Int +& $y.Numeric.Int }
+proto sub infix:<+&>(Mu $?, Mu $?) is pure { * }
+multi sub infix:<+&>()           { +^0 }
+multi sub infix:<+&>($x)         { $x }
+multi sub infix:<+&>($x, $y)     { $x.Numeric.Int +& $y.Numeric.Int }
 
-proto infix:<+|>($?, $?) is pure { * }
-multi infix:<+|>()           { 0 }
-multi infix:<+|>($x)         { $x }
-multi infix:<+|>($x, $y)     { $x.Numeric.Int +| $y.Numeric.Int }
+proto sub infix:<+|>(Mu $?, Mu $?) is pure { * }
+multi sub infix:<+|>()           { 0 }
+multi sub infix:<+|>($x)         { $x }
+multi sub infix:<+|>($x, $y)     { $x.Numeric.Int +| $y.Numeric.Int }
 
-proto infix:<+^>($?, $?) is pure { * }
-multi infix:<+^>()           { 0 }
-multi infix:<+^>($x)         { $x }
-multi infix:<+^>($x, $y)     { $x.Numeric.Int +^ $y.Numeric.Int }
+proto sub infix:<+^>(Mu $?, Mu $?) is pure { * }
+multi sub infix:<+^>()           { 0 }
+multi sub infix:<+^>($x)         { $x }
+multi sub infix:<+^>($x, $y)     { $x.Numeric.Int +^ $y.Numeric.Int }
 
-proto infix:«+<»($?, $?) is pure { * }
-multi infix:«+<»()           { fail "No zero-arg meaning for infix:«+<»"; }
-multi infix:«+<»($x)         { $x }
-multi infix:«+<»($x,$y)      { $x.Numeric.Int +< $y.Numeric.Int }
+proto sub infix:«+<»(Mu $?, Mu $?) is pure { * }
+multi sub infix:«+<»()           { fail "No zero-arg meaning for infix:«+<»"; }
+multi sub infix:«+<»($x)         { $x }
+multi sub infix:«+<»($x,$y)      { $x.Numeric.Int +< $y.Numeric.Int }
 
-proto infix:«+>»($?, $?) is pure { * }
-multi infix:«+>»()           { fail "No zero-arg meaning for infix:«+>»"; }
-multi infix:«+>»($x)         { $x }
-multi infix:«+>»($x,$y)      { $x.Numeric.Int +> $y.Numeric.Int }
+proto sub infix:«+>»(Mu $?, Mu $?) is pure { * }
+multi sub infix:«+>»()           { fail "No zero-arg meaning for infix:«+>»"; }
+multi sub infix:«+>»($x)         { $x }
+multi sub infix:«+>»($x,$y)      { $x.Numeric.Int +> $y.Numeric.Int }
 
-proto prefix:<+^>($?, $?) is pure { * }
-multi prefix:<+^>($x)        { +^ $x.Numeric.Int }
+proto sub prefix:<+^>(Mu $) is pure { * }
+multi sub prefix:<+^>($x)        { +^ $x.Numeric.Int }
 
 # vim: ft=perl6 expandtab sw=4
