@@ -1,6 +1,8 @@
 my class Pod::Block {
     has %.config;
-    has @.content;
+    has @.contents;
+
+    submethod BUILD(:%!config, :@!contents) {}
 
     sub pod-gist(Pod::Block $pod, $level = 0) {
         my $leading = ' ' x $level;
@@ -9,20 +11,21 @@ my class Pod::Block {
         for <config name level caption type> {
             my $thing = $pod.?"$_"();
             if $thing {
-                %confs{$_} = $thing ~~ Iterable ?? $thing.perl
-                                                !! $thing.Str;
+                %confs{$_} = nqp::istype($thing,Iterable)
+                  ?? $thing.perl
+                  !! $thing.Str;
             }
         }
         @chunks = $leading, $pod.^name, (%confs.perl if %confs), "\n";
-        for $pod.content.list -> $c {
-            if $c ~~ Pod::Block {
+        for $pod.contents.flat -> $c {
+            if nqp::istype($c,Pod::Block) {
                 @chunks.push: pod-gist($c, $level + 2);
             }
-            elsif $c ~~ Positional {
-                @chunks.push: $c>>.Str.perl.indent($level + 2), "\n";
+            elsif nqp::istype($c,Positional) {
+                @chunks.append: $c>>.Str.perl.indent($level + 2), "\n";
             }
             else {
-                @chunks.push: $c.Str.indent($level + 2), "\n";
+                @chunks.append: $c.Str.indent($level + 2), "\n";
             }
         }
         @chunks.join;
@@ -48,14 +51,42 @@ my class Pod::Block::Code is Pod::Block {
 
 my class Pod::Block::Declarator is Pod::Block {
     has $.WHEREFORE;
+    has @!leading;
+    has @!trailing;
+
+    submethod BUILD(:@!leading, :@!trailing) {}
+
     method set_docee($d) {
         $!WHEREFORE = $d
     }
     method Str {
-        ~@.content
+        @.contents.join('')
     }
     multi method gist(Pod::Block::Declarator:D:) {
-        self.Str
+        @.contents.join('')
+    }
+
+    method contents {
+        if @!leading && @!trailing {
+            [ $.leading ~ "\n" ~ $.trailing ]
+        } elsif @!leading {
+            [ $.leading ]
+        } elsif @!trailing {
+            [ $.trailing ]
+        } else {
+            []
+        }
+    }
+
+    method leading  { @!leading  ?? @!leading.join(' ')  !! Any }
+    method trailing { @!trailing ?? @!trailing.join(' ') !! Any }
+
+    method _add_leading($addition) {
+        @!leading.push: ~$addition;
+    }
+
+    method _add_trailing($addition) {
+        @!trailing.push: ~$addition;
     }
 }
 
@@ -82,4 +113,10 @@ class Pod::Config {
     has %.config;
 }
 
+# for passing raw instructions to specific backends
+class Pod::Raw {
+    has $.target;
+    has @.contents;
+}
+
 # vim: ft=perl6 expandtab sw=4
