@@ -1,10 +1,10 @@
 my class X::Bind { ... }
 my class X::Caller::NotDynamic { ... }
 
-my class PseudoStash is EnumMap {
+my class PseudoStash is Map {
     has Mu $!ctx;
     has int $!mode;
-    
+
     # Lookup modes.
     my int constant PICK_CHAIN_BY_NAME = 0;
     my int constant STATIC_CHAIN       = 1;
@@ -16,10 +16,10 @@ my class PseudoStash is EnumMap {
         my $obj := nqp::create(self);
         my $ctx := nqp::ctxcaller(nqp::ctx());
         nqp::bindattr($obj, PseudoStash, '$!ctx', $ctx);
-        nqp::bindattr($obj, EnumMap, '$!storage', nqp::ctxlexpad($ctx));
+        nqp::bindattr($obj, Map, '$!storage', nqp::ctxlexpad($ctx));
         $obj
     }
-    
+
     my %pseudoers =
         'MY' => sub ($cur) {
             my $stash := nqp::clone($cur);
@@ -34,7 +34,7 @@ my class PseudoStash is EnumMap {
                 $ctx := nqp::ctxouterskipthunks($ctx);
             }
             my $stash := nqp::create(PseudoStash);
-            nqp::bindattr($stash, EnumMap, '$!storage', nqp::ctxlexpad($ctx));
+            nqp::bindattr($stash, Map, '$!storage', nqp::ctxlexpad($ctx));
             nqp::bindattr($stash, PseudoStash, '$!ctx', $ctx);
             nqp::bindattr_i($stash, PseudoStash, '$!mode', PRECISE_SCOPE);
             nqp::setwho(
@@ -45,7 +45,7 @@ my class PseudoStash is EnumMap {
             my Mu $ctx := nqp::ctxcallerskipthunks(
                 nqp::getattr(nqp::decont($cur), PseudoStash, '$!ctx'));
             my $stash := nqp::create(PseudoStash);
-            nqp::bindattr($stash, EnumMap, '$!storage', nqp::ctxlexpad($ctx));
+            nqp::bindattr($stash, Map, '$!storage', nqp::ctxlexpad($ctx));
             nqp::bindattr($stash, PseudoStash, '$!ctx', $ctx);
             nqp::bindattr_i($stash, PseudoStash, '$!mode', PRECISE_SCOPE +| REQUIRE_DYNAMIC);
             nqp::setwho(
@@ -56,13 +56,31 @@ my class PseudoStash is EnumMap {
             my Mu $ctx := nqp::ctxouterskipthunks(
                 nqp::getattr(nqp::decont($cur), PseudoStash, '$!ctx'));
             my $stash := nqp::create(PseudoStash);
-            nqp::bindattr($stash, EnumMap, '$!storage', nqp::ctxlexpad($ctx));
+            nqp::bindattr($stash, Map, '$!storage', nqp::ctxlexpad($ctx));
             nqp::bindattr($stash, PseudoStash, '$!ctx', $ctx);
             nqp::bindattr_i($stash, PseudoStash, '$!mode', PRECISE_SCOPE);
             nqp::setwho(
                 Metamodel::ModuleHOW.new_type(:name('OUTER')),
                 $stash);
         },
+        'LEXICAL' => sub ($cur) {
+            my $stash := nqp::clone($cur);
+            nqp::bindattr_i($stash, PseudoStash, '$!mode', STATIC_CHAIN);
+            nqp::setwho(
+                Metamodel::ModuleHOW.new_type(:name('LEXICAL')),
+                $stash);
+        },
+        'OUTERS' => sub ($cur) {
+            my Mu $ctx := nqp::ctxouterskipthunks(
+                nqp::getattr(nqp::decont($cur), PseudoStash, '$!ctx'));
+            my $stash := nqp::create(PseudoStash);
+            nqp::bindattr($stash, Map, '$!storage', nqp::ctxlexpad($ctx));
+            nqp::bindattr($stash, PseudoStash, '$!ctx', $ctx);
+            nqp::bindattr_i($stash, PseudoStash, '$!mode', STATIC_CHAIN);
+            nqp::setwho(
+                Metamodel::ModuleHOW.new_type(:name('OUTERS')),
+                $stash);
+        },
         'DYNAMIC' => sub ($cur) {
             my $stash := nqp::clone($cur);
             nqp::bindattr_i($stash, PseudoStash, '$!mode', DYNAMIC_CHAIN);
@@ -70,13 +88,24 @@ my class PseudoStash is EnumMap {
                 Metamodel::ModuleHOW.new_type(:name('DYNAMIC')),
                 $stash);
         },
+        'CALLERS' => sub ($cur) {
+            my Mu $ctx := nqp::ctxcallerskipthunks(
+                nqp::getattr(nqp::decont($cur), PseudoStash, '$!ctx'));
+            my $stash := nqp::create(PseudoStash);
+            nqp::bindattr($stash, Map, '$!storage', nqp::ctxlexpad($ctx));
+            nqp::bindattr($stash, PseudoStash, '$!ctx', $ctx);
+            nqp::bindattr_i($stash, PseudoStash, '$!mode', DYNAMIC_CHAIN +| REQUIRE_DYNAMIC);
+            nqp::setwho(
+                Metamodel::ModuleHOW.new_type(:name('CALLERS')),
+                $stash);
+        },
         'UNIT' => sub ($cur) {
             my Mu $ctx := nqp::getattr(nqp::decont($cur), PseudoStash, '$!ctx');
             until nqp::existskey(nqp::ctxlexpad($ctx), '!UNIT_MARKER') {
                 $ctx := nqp::ctxouterskipthunks($ctx);
             }
             my $stash := nqp::create(PseudoStash);
-            nqp::bindattr($stash, EnumMap, '$!storage',nqp::ctxlexpad($ctx));
+            nqp::bindattr($stash, Map, '$!storage',nqp::ctxlexpad($ctx));
             nqp::bindattr($stash, PseudoStash, '$!ctx', $ctx);
             nqp::bindattr_i($stash, PseudoStash, '$!mode', PRECISE_SCOPE);
             nqp::setwho(
@@ -92,11 +121,30 @@ my class PseudoStash is EnumMap {
             }
             $ctx := nqp::ctxouter(nqp::ctxouter($ctx));
             my $stash := nqp::create(PseudoStash);
-            nqp::bindattr($stash, EnumMap, '$!storage', nqp::ctxlexpad($ctx));
+            nqp::bindattr($stash, Map, '$!storage', nqp::ctxlexpad($ctx));
             nqp::bindattr($stash, PseudoStash, '$!ctx', $ctx);
-            nqp::bindattr_i($stash, PseudoStash, '$!mode', PRECISE_SCOPE);
+            nqp::bindattr_i($stash, PseudoStash, '$!mode', STATIC_CHAIN);
             nqp::setwho(
-                Metamodel::ModuleHOW.new_type(:name('UNIT')),
+                Metamodel::ModuleHOW.new_type(:name('SETTING')),
+                $stash);
+        },
+        'CLIENT' => sub ($cur) {
+            my $pkg := nqp::getlexrel(
+                nqp::getattr(nqp::decont($cur), PseudoStash, '$!ctx'),
+                '$?PACKAGE');
+            die "GLOBAL can have no client package" if $pkg.^name eq "GLOBAL";
+            my Mu $ctx := nqp::ctxcallerskipthunks(
+                nqp::getattr(nqp::decont($cur), PseudoStash, '$!ctx'));
+            while nqp::getlexrel($ctx, '$?PACKAGE') === $pkg {
+                $ctx := nqp::ctxcallerskipthunks($ctx);
+                die "No client package found" unless $ctx;
+            }
+            my $stash := nqp::create(PseudoStash);
+            nqp::bindattr($stash, Map, '$!storage', nqp::ctxlexpad($ctx));
+            nqp::bindattr($stash, PseudoStash, '$!ctx', $ctx);
+            nqp::bindattr_i($stash, PseudoStash, '$!mode', PRECISE_SCOPE +| REQUIRE_DYNAMIC);
+            nqp::setwho(
+                Metamodel::ModuleHOW.new_type(:name('CLIENT')),
                 $stash);
         },
         'OUR' => sub ($cur) {
@@ -104,20 +152,19 @@ my class PseudoStash is EnumMap {
                 nqp::getattr(nqp::decont($cur), PseudoStash, '$!ctx'),
                 '$?PACKAGE')
         };
-    
-    method at_key($key is copy) is rw {
-        $key = $key.Str;
+
+    multi method AT-KEY(PseudoStash:D: Str() $key) is raw {
         my Mu $nkey := nqp::unbox_s($key);
-        if %pseudoers.exists_key($key) {
+        if %pseudoers.EXISTS-KEY($key) {
             %pseudoers{$key}(self)
         }
         elsif nqp::bitand_i($!mode, PRECISE_SCOPE) {
-            my Mu $store := nqp::getattr(self, EnumMap, '$!storage');
+            my Mu $store := nqp::getattr(self, Map, '$!storage');
             my Mu $res := nqp::existskey($store, $nkey) ??
                             nqp::atkey($store, $nkey) !!
                             Any;
             if !($res =:= Any) && nqp::bitand_i($!mode, REQUIRE_DYNAMIC) {
-                if !$res.VAR.dynamic {
+                if try !$res.VAR.dynamic {
                     X::Caller::NotDynamic.new(
                         symbol => $key,
                     ).throw;
@@ -125,54 +172,53 @@ my class PseudoStash is EnumMap {
             }
             $res;
         }
-        elsif nqp::bitand_i($!mode, nqp::bitor_i(DYNAMIC_CHAIN, PICK_CHAIN_BY_NAME)) && substr($key, 1, 1) eq '*' {
+        elsif nqp::bitand_i($!mode, nqp::bitor_i(DYNAMIC_CHAIN, PICK_CHAIN_BY_NAME)) && $key.substr-eq("*",1) {
             my $found := nqp::getlexreldyn(
                 nqp::getattr(self, PseudoStash, '$!ctx'),
                 $nkey);
             nqp::isnull($found) ?? Any !! $found
         }
-        else {
+        else { # STATIC_CHAIN
             my $found := nqp::getlexrel(
                 nqp::getattr(self, PseudoStash, '$!ctx'),
                 $nkey);
             nqp::isnull($found) ?? Any !! $found
         }
     }
-    
-    method bind_key($key is copy, \value) {
-        $key = $key.Str;
-        if %pseudoers.exists_key($key) {
+
+    method BIND-KEY(Str() $key, \value) is raw {
+        if %pseudoers.EXISTS-KEY($key) {
             X::Bind.new(target => "pseudo-package $key").throw;
         }
         elsif nqp::bitand_i($!mode, PRECISE_SCOPE) {
-            my Mu $store := nqp::getattr(self, EnumMap, '$!storage');
+            my Mu $store := nqp::getattr(self, Map, '$!storage');
             nqp::bindkey($store, nqp::unbox_s($key), value)
         }
-        elsif nqp::bitand_i($!mode, nqp::bitor_i(DYNAMIC_CHAIN, PICK_CHAIN_BY_NAME)) && substr($key, 1, 1) eq '*' {
+        elsif nqp::bitand_i($!mode, nqp::bitor_i(DYNAMIC_CHAIN, PICK_CHAIN_BY_NAME)) && $key.substr-eq("*",1) {
             die "Binding to dynamic variables not yet implemented";
         }
-        else {
+        else { # STATIC_CHAIN
             die "This case of binding is not yet implemented";
         }
     }
-    method exists_key($key is copy) {
-        $key = $key.Str;
-        if %pseudoers.exists_key($key) {
+    # for some reason we get a ambiguous dispatch error by making this a multi
+    method EXISTS-KEY(PseudoStash:D: Str() $key) {
+        if %pseudoers.EXISTS-KEY($key) {
             True
         }
         elsif nqp::bitand_i($!mode, PRECISE_SCOPE) {
             nqp::p6bool(nqp::existskey(
-                nqp::getattr(self, EnumMap, '$!storage'),
+                nqp::getattr(self, Map, '$!storage'),
                 nqp::unbox_s($key)))
         }
-        elsif nqp::bitand_i($!mode, nqp::bitor_i(DYNAMIC_CHAIN, PICK_CHAIN_BY_NAME)) && substr($key, 1, 1) eq '*' {
+        elsif nqp::bitand_i($!mode, nqp::bitor_i(DYNAMIC_CHAIN, PICK_CHAIN_BY_NAME)) && $key.substr-eq("*",1) {
             nqp::isnull(
                 nqp::getlexreldyn(
                     nqp::getattr(self, PseudoStash, '$!ctx'),
                     nqp::unbox_s($key)))
                 ?? False !! True
         }
-        else {
+        else { # STATIC_CHAIN
             nqp::isnull(
                 nqp::getlexrel(
                     nqp::getattr(self, PseudoStash, '$!ctx'),
