@@ -3,9 +3,7 @@ my class Complex { ... }
 my role Real does Numeric {
     method Rat(Real:D: Real $epsilon = 1.0e-6) { self.Bridge.Rat($epsilon) }
     method abs()  { self < 0 ?? -self !! self }
-    proto method sign(|) {*}
-    multi method sign(Real:U:) { Mu }
-    multi method sign(Real:D:) { self < 0 ?? -1 !! self == 0 ?? 0 !! 1 }
+    method sign(Real:D:) { self > 0 ?? 1 !! self < 0 ?? -1 !! +self }
     method conj(Real:D:) { self }
     method sqrt() { self.Bridge.sqrt }
     method rand() { self.Bridge.rand }
@@ -38,8 +36,15 @@ my role Real does Numeric {
     method acotanh() { self.Bridge.acotanh }
     method floor() { self.Bridge.floor }
     method ceiling() { self.Bridge.ceiling }
-    # cannot use '0.5' here, because Rat isn't initialized yet
-    method round($scale as Real = 1) { (self / $scale + 1/2).floor * $scale }
+
+    proto method round(|) { * }
+    multi method round(Real:D:) {
+        (self + 1/2).floor; # Rat NYI here, so no .5
+    }
+    multi method round(Real:D: Real() $scale) {
+        (self / $scale + 1/2).floor * $scale;
+    }
+
     method unpolar(Real $angle) {
         Complex.new(self * $angle.cos, self * $angle.sin);
     }
@@ -57,23 +62,58 @@ my role Real does Numeric {
     }
     method isNaN { Bool::False }
 
-    method base(Int:D $base) {
+    method polymod(Real:D: +@mods) {
+        my $more = self;
+        my $inf = @mods.is-lazy;
+        fail X::OutOfRange.new(
+          what => 'invocant to polymod', got => $more, range => "0..*"
+        ) if $more < 0;
+        gather {
+            for @mods -> $mod {
+                last if $inf and not $more;
+                fail X::Numeric::DivideByZero.new(
+                  using => 'polymod', numerator => $more
+                ) unless $mod;
+                take my $rem = $more % $mod;
+                $more -= $rem;
+                $more /= $mod;
+            }
+            take $more unless $inf;
+        }
+    }
+
+    method base(Int:D $base, $digits?) {
+        fail X::OutOfRange.new(
+                what => 'digits argument to base', got => $digits, range => "0..*"
+            ) if $digits.defined and $digits < 0;
+        my $prec = $digits // 1e8.log($base.Num).Int;
         my Int $int_part = self.Int;
         my $frac = abs(self - $int_part);
         my @frac_digits;
-        my @conversion = qw/0 1 2 3 4 5 6 7 8 9
-                            A B C D E F G H I J
-                            K L M N O P Q R S T/;
-        # pretty arbitrary precision limit for now
-        # but better than endless loops
-        my $limit = 1e8.log($base.Num).Int;
-        for ^$limit {
-            last if $frac == 0;
+        my @conversion := <0 1 2 3 4 5 6 7 8 9
+                           A B C D E F G H I J
+                           K L M N O P Q R S T
+                           U V W X Y Z>;
+        for ^$prec {
+            last unless $digits // $frac;
             $frac = $frac * $base;
-            push @frac_digits, @conversion[$frac.Int];
+            push @frac_digits, $frac.Int;
             $frac = $frac - $frac.Int;
         }
-        my Str $r = $int_part.base($base) ~ '.' ~ @frac_digits.join('');
+        if 2 * $frac >= 1 {
+            if @frac_digits {
+                for @frac_digits-1 ... 0 -> $x {
+                    last if ++@frac_digits[$x] < $base;
+                    @frac_digits[$x] = 0;
+                    $int_part++ if $x == 0
+                }
+            }
+            else {
+                $int_part++;
+            }
+        }
+        my Str $r = $int_part.base($base);
+        $r ~= '.' ~ @conversion[@frac_digits].join if @frac_digits;
         # if $int_part is 0, $int_part.base doesn't see the sign of self
         $int_part == 0 && self < 0 ?? '-' ~ $r !! $r;
     }
@@ -88,35 +128,35 @@ my role Real does Numeric {
 proto sub cis($) {*}
 multi sub cis(Real $a) { $a.cis }
 
-multi infix:<+>(Real \a, Real \b)   { a.Bridge + b.Bridge }
+multi sub infix:<+>(Real \a, Real \b)   { a.Bridge + b.Bridge }
 
-multi infix:<->(Real \a, Real \b)   { a.Bridge - b.Bridge }
+multi sub infix:<->(Real \a, Real \b)   { a.Bridge - b.Bridge }
 
-multi infix:<*>(Real \a, Real \b)   { a.Bridge * b.Bridge }
+multi sub infix:<*>(Real \a, Real \b)   { a.Bridge * b.Bridge }
 
-multi infix:</>(Real \a, Real \b)   { a.Bridge / b.Bridge }
+multi sub infix:</>(Real \a, Real \b)   { a.Bridge / b.Bridge }
 
-multi infix:<%>(Real \a, Real \b)   { a.Bridge % b.Bridge }
+multi sub infix:<%>(Real \a, Real \b)   { a.Bridge % b.Bridge }
 
-multi infix:<**>(Real \a, Real \b)  { a.Bridge ** b.Bridge }
+multi sub infix:<**>(Real \a, Real \b)  { a.Bridge ** b.Bridge }
 
-multi infix:«<=>»(Real \a, Real \b) { a.Bridge <=> b.Bridge }
+multi sub infix:«<=>»(Real \a, Real \b) { a.Bridge <=> b.Bridge }
 
-multi infix:<==>(Real \a, Real \b)  { a.Bridge == b.Bridge }
+multi sub infix:<==>(Real \a, Real \b)  { a.Bridge == b.Bridge }
 
-multi infix:«<»(Real \a, Real \b)   { a.Bridge < b.Bridge }
+multi sub infix:«<»(Real \a, Real \b)   { a.Bridge < b.Bridge }
 
-multi infix:«<=»(Real \a, Real \b)  { a.Bridge <= b.Bridge }
+multi sub infix:«<=»(Real \a, Real \b)  { a.Bridge <= b.Bridge }
 
-multi infix:«>»(Real \a, Real \b)   { a.Bridge > b.Bridge }
+multi sub infix:«>»(Real \a, Real \b)   { a.Bridge > b.Bridge }
 
-multi infix:«>=»(Real \a, Real \b)  { a.Bridge >= b.Bridge }
+multi sub infix:«>=»(Real \a, Real \b)  { a.Bridge >= b.Bridge }
 
-multi prefix:<->(Real \a)            { -a.Bridge }
+multi sub prefix:<->(Real:D \a)            { -a.Bridge }
 
-# NOTE: According to the spec, infix:<mod> is "Not coercive, 
+# NOTE: According to the spec, infix:<mod> is "Not coercive,
 # so fails on differing types."  Thus no casts here.
-proto sub infix:<mod>($, $) {*}
+proto sub infix:<mod>($, $) is pure {*}
 multi sub infix:<mod>(Real $a, Real $b) {
     $a - ($a div $b) * $b;
 }
