@@ -2,9 +2,15 @@ my class Regex { # declared in BOOTSTRAP
     # class Regex is Method {
     #     has Mu $!caps;
     #     has Mu $!nfa;
-    #     has Mu $!alt_nfas
+    #     has Mu $!alt_nfas;
+    #     has Mu $!source;
 
-    multi method ACCEPTS(Regex:D \SELF: Mu \topic) {
+    proto method ACCEPTS(|) { * }
+    multi method ACCEPTS(Regex:D: Mu:U \a) { False }
+    multi method ACCEPTS(Regex:U: Any \topic) { # use of Any on topic to force autothreading
+        nqp::p6bool(nqp::istype(topic, self)) # so that all(@foo) ~~ Type works as expected
+    }
+    multi method ACCEPTS(Regex:D \SELF: Any \topic) {
         my $dollar_slash := nqp::getlexrelcaller(
             nqp::ctxcallerskipthunks(nqp::ctx()),
             '$/');
@@ -15,7 +21,7 @@ my class Regex { # declared in BOOTSTRAP
         my $dollar_slash := nqp::getlexrelcaller(
             nqp::ctxcallerskipthunks(nqp::ctx()),
             '$/');
-        for @a {
+        for flat @a {
             $dollar_slash = SELF.(Cursor.'!cursor_init'($_, :c(0))).MATCH_SAVE;
             return $dollar_slash if $dollar_slash;
         }
@@ -33,14 +39,21 @@ my class Regex { # declared in BOOTSTRAP
     }
 
     multi method Bool(Regex:D:) {
-        my $dollar_slash := nqp::getlexrelcaller(
-            nqp::ctxcallerskipthunks(nqp::ctx()),
-            '$/');
-        my $dollar_underscore := nqp::getlexrelcaller(
-            nqp::ctxcallerskipthunks(nqp::ctx()),
-            '$_');
-        $dollar_slash = $dollar_underscore.match(self);
-        $dollar_slash.Bool()
+        my $underscore;
+        my $ctx := nqp::ctxcallerskipthunks(nqp::ctx());
+        $ctx := nqp::ctxcallerskipthunks($ctx)
+          until ($underscore := nqp::getlexrelcaller($ctx,'$_')).DEFINITE;
+        my $slash := nqp::getlexrelcaller($ctx,'$/');
+        $slash = $underscore.match(self);
+        $slash.Bool()
+    }
+
+    multi method gist(Regex:D:) {
+        nqp::isnull($!source) ?? '' !! $!source
+    }
+
+    multi method perl(Regex:D:) {
+        nqp::isnull($!source) ?? '' !! $!source
     }
 }
 
