@@ -20,22 +20,14 @@ my class Routine { # declared in BOOTSTRAP
     #     has Mu $!dispatch_order;
     #     has Mu $!dispatch_cache;
 
-    method of() { self.signature.returns }
-    method returns() { self.signature.returns }
     method onlystar() { nqp::p6bool($!onlystar) }
-    
-    method assuming($r: *@curried_pos, *%curried_named) {
-        return sub CURRIED (*@pos, *%named) {
-            $r(|@curried_pos, |@pos, |%curried_named, |%named)
-        }
-    }
-    
+
     method candidates() {
         self.is_dispatcher ??
             nqp::hllize($!dispatchees) !!
             (self,)
     }
-    
+
     method cando(Capture $c) {
         my $disp;
         if self.is_dispatcher {
@@ -51,25 +43,25 @@ my class Routine { # declared in BOOTSTRAP
         }
         checker(|$c);
     }
-    
+
     method multi() {
         self.dispatcher.defined
     }
-    
+
     multi method perl(Routine:D:) {
         my $perl = ( self.^name ~~ m/^\w+/ ).lc;
         if self.name() -> $n {
             $perl ~= " $n";
         }
-        $perl ~= ' ' ~ self.signature().perl.substr(1); # lose colon prefix
+        $perl ~= ' ' ~ substr(self.signature().perl,1); # lose colon prefix
         $perl ~= ' { #`(' ~ self.WHICH ~ ') ... }';
         $perl
     }
-    
+
     method soft() {
         Mu
     }
-    
+
     method wrap(&wrapper) {
         my class WrapHandle {
             has $!dispatcher;
@@ -85,26 +77,26 @@ my class Routine { # declared in BOOTSTRAP
                 $!dispatcher := WrapDispatcher.new()
                     unless nqp::isconcrete($!dispatcher);
                 $!dispatcher.add(&wrapper);
-                
+
                 # Return a handle.
                 my $handle := nqp::create(WrapHandle);
                 nqp::bindattr($handle, WrapHandle, '$!dispatcher', $!dispatcher);
                 nqp::bindattr($handle, WrapHandle, '$!wrapper', &wrapper);
                 $handle
             }
-            method postcircumfix:<( )>($c) is rw {
-                $!dispatcher.enter(|$c);
+            method CALL-ME(|c) is raw {
+                $!dispatcher.enter(|c);
             }
             method soft() { True }
         }
-        
+
         # We can't wrap a hardened routine (that is, one that's been
         # marked inlinable).
         if nqp::istype(self, HardRoutine) {
             die "Cannot wrap a HardRoutine, since it may have been inlined; " ~
                 "use the 'soft' pragma to avoid marking routines as hard.";
         }
-        
+
         # If we're not wrapped already, do the initial dispatcher
         # creation.
         unless nqp::istype(self, Wrapped) {
@@ -116,17 +108,21 @@ my class Routine { # declared in BOOTSTRAP
         # Add this wrapper.
         self.UNSHIFT_WRAPPER(&wrapper);
     }
-    
+
     method unwrap($handle) {
         $handle.can('restore') && $handle.restore() ||
             X::Routine::Unwrap.new.throw
     }
-    
+
     method yada() {
         nqp::p6bool(nqp::getattr_i(self, Routine, '$!yada'))
     }
 
     method package() { $!package }
+
+    method leave(*@) {
+        X::NYI.new(:feature("{self.^name}.leave()")).throw;
+    }
 }
 
 multi sub trait_mod:<is>(Routine $r, :$cached!) {
@@ -135,9 +131,9 @@ multi sub trait_mod:<is>(Routine $r, :$cached!) {
       if $r.onlystar; # disable optimization
     $r.wrap(-> |c {
         my $key := c.gist;
-        %cache{$key}:exists
+        %cache.EXISTS-KEY($key)
           ?? %cache{$key}
-          !! (%cache{$key} = callsame);
+          !! (%cache{$key} := callsame);
     });
 }
 
