@@ -21,8 +21,8 @@ my class Signature { # declared in BOOTSTRAP
     multi method ACCEPTS(Signature:D: Signature:D $topic) {
         my $sclass = self.params.classify({.named});
         my $tclass = $topic.params.classify({.named});
-        my @spos := $sclass<False> // ();
-        my @tpos := $tclass<False> // ();
+        my @spos := $sclass{False} // ();
+        my @tpos := $tclass{False} // ();
 
         while @spos {
             my $s;
@@ -50,20 +50,20 @@ my class Signature { # declared in BOOTSTRAP
             return False unless @spos[0].optional or @spos[0].slurpy or @spos[0].capture;
         }
 
-        for ($sclass<True> // ()).grep({!.optional and !.slurpy}) -> $this {
+        for flat ($sclass{True} // ()).grep({!.optional and !.slurpy}) -> $this {
             my $other;
-            return False unless $other=($tclass<True> // ()).grep(
+            return False unless $other=($tclass{True} // ()).grep(
                 {!.optional and $_ ~~ $this });
             return False unless +$other == 1;
         }
 
-        my $here=$sclass<True>.SetHash;
-        my $hasslurpy=($sclass<True> // ()).grep({.slurpy}).Bool;
-        for @($tclass<True> // ()) -> $other {
+        my $here=$sclass{True}.SetHash;
+        my $hasslurpy=($sclass{True} // ()).grep({.slurpy}).Bool;
+        for flat @($tclass{True} // ()) -> $other {
             my $this;
 
             if $other.slurpy {
-                return False if any($here.keys) ~~ { .type !=:= Mu };
+                return False if any($here.keys) ~~ { !(.type =:= Mu) };
                 return $hasslurpy;
             }
             if $this=$here.keys.grep( -> $t { $other ~~ $t }) {
@@ -73,7 +73,7 @@ my class Signature { # declared in BOOTSTRAP
                 return False unless $hasslurpy;
             }
         }
-        return True;
+        True;
     }
 
     method arity() {
@@ -85,34 +85,51 @@ my class Signature { # declared in BOOTSTRAP
     }
 
     method params() {
-        nqp::p6list(nqp::clone($!params), List, Mu);
+        nqp::p6bindattrinvres(nqp::create(List), List, '$!reified',
+            nqp::clone($!params));
     }
 
-    # XXX TODO: Parameter separators.
-    multi method perl(Signature:D:) {
+    method !gistperl(Signature:D: $perl, Mu:U :$elide-type = Mu) {
         # Opening.
-        my $perl = ':(';
+        my $text = $perl ?? ':(' !! '(';
 
         # Parameters.
-        my $params = self.params();
-        my $sep = '';
-        my int $i = 0;
-        while $i < $params.elems {
-            my $param := $params[$i];
-            $perl = $perl ~ $sep ~ $param.perl;
-            # this works because methods always have at least one
-            # other parameter, *%_
-            $sep = ($i == 0 && $param.invocant) ?? ': ' !! ', ';
-            $i = $i + 1;
+        if self.params.Array -> @params {
+            $text ~= @params.shift.perl(:$elide-type) ~ ': '
+                if @params[0].invocant;
+            $text ~= ';; '
+                if !@params[0].multi-invocant;
+
+            my $sep = '';
+            for @params.kv -> $i, $param {
+                $text ~= $sep ~ $param.perl(:$elide-type);
+                $text .= subst(/' $'$/,'') unless $perl;
+                $sep = $param.multi-invocant && !@params[$i+1].?multi-invocant
+                  ?? ';; '
+                  !! ', '
+            }
         }
-        if !nqp::isnull($!returns) && $!returns !=:= Mu {
-            $perl ~= ' --> ' ~ $!returns.perl
+        if !nqp::isnull($!returns) && !($!returns =:= Mu) {
+            $text = $text ~ ' --> ' ~ $!returns.perl
         }
         # Closer.
-        $perl ~ ')'
+        $text ~ ')'
+    }
+
+    method !deftype(Signature:D:) {
+         !nqp::isnull($!code) && $!code ~~ Routine ?? Any !! Mu
+    }
+
+    multi method perl(Signature:D:) {
+        self!gistperl(True, :elide-type(self!deftype))
+    }
+    multi method gist(Signature:D:) {
+        self!gistperl(False, :elide-type(self!deftype))
     }
 
     method returns() { $!returns }
 }
 
+multi sub infix:<eqv>(Signature $a, Signature $b) { $a.perl eq $b.perl }
+
 # vim: ft=perl6 expandtab sw=4
