@@ -1,17 +1,17 @@
 my class Stash { # declared in BOOTSTRAP
     # class Stash is Hash {
+    #     has str $!longname;
 
-    multi method at_key(Stash:D: $key is copy, :$global_fallback) is rw {
-        my Mu $storage := nqp::defined(nqp::getattr(self, EnumMap, '$!storage')) ??
-            nqp::getattr(self, EnumMap, '$!storage') !!
-            nqp::bindattr(self, EnumMap, '$!storage', nqp::hash());
-        $key = $key.Str;
+    multi method AT-KEY(Stash:D: Str() $key, :$global_fallback) is raw {
+        my Mu $storage := nqp::defined(nqp::getattr(self, Map, '$!storage')) ??
+            nqp::getattr(self, Map, '$!storage') !!
+            nqp::bindattr(self, Map, '$!storage', nqp::hash());
         if nqp::existskey($storage, nqp::unbox_s($key)) {
             nqp::atkey($storage, nqp::unbox_s($key))
         }
         elsif $global_fallback {
             nqp::existskey(GLOBAL.WHO, $key)
-                ?? GLOBAL.WHO.at_key($key)
+                ?? GLOBAL.WHO.AT-KEY($key)
                 !! fail("Could not find symbol '$key'")
         }
         else {
@@ -19,20 +19,28 @@ my class Stash { # declared in BOOTSTRAP
                  -> { nqp::bindkey($storage, nqp::unbox_s($key), $v) } )
         }
     }
-    
+
     method package_at_key(Stash:D: str $key) {
-        my Mu $storage := nqp::defined(nqp::getattr(self, EnumMap, '$!storage')) ??
-            nqp::getattr(self, EnumMap, '$!storage') !!
-            nqp::bindattr(self, EnumMap, '$!storage', nqp::hash());
+        my Mu $storage := nqp::defined(nqp::getattr(self, Map, '$!storage')) ??
+            nqp::getattr(self, Map, '$!storage') !!
+            nqp::bindattr(self, Map, '$!storage', nqp::hash());
         if nqp::existskey($storage, nqp::unbox_s($key)) {
             nqp::atkey($storage, $key)
         }
         else {
             my $pkg := Metamodel::PackageHOW.new_type(:name($key));
-            $pkg.HOW.compose($pkg);
+            $pkg.^compose;
             nqp::bindkey($storage, $key, $pkg)
         }
     }
+
+    multi method gist(Stash:D:) {
+        self.Str
+    }
+
+    multi method Str(Stash:D:) {
+        nqp::isnull_s($!longname) ?? '<anon>' !! $!longname
+    }
 }
 
 # vim: ft=perl6 expandtab sw=4
