@@ -1,23 +1,35 @@
 my class DateTime { ... }
 my class Date     { ... }
 
-my @UNITS = <second minute hour day week month year> X~ '', 's';
-
 my role Dateish {
-    method is-leap-year($y = $.year) {
-        $y %% 4 and not $y %% 100 or $y %% 400
+    has Int $.year;
+    has Int $.month = 1;
+    has Int $.day = 1;
+
+    method IO(|c) { IO::Path.new(self) }
+
+    method !VALID-UNIT($unit) {
+        state %UNITS =  # core setting doesn't build if it is a my at role level
+          (<second minute hour day week month year> X~ "","s").map: {$_ => 1};
+        X::DateTime::InvalidDeltaUnit.new(:$unit).throw
+          unless %UNITS.EXISTS-KEY($unit);
     }
 
-    method days-in-month($year = $.year, $month = $.month) {
-           $month == 2        ?? self.is-leap-year($year) ?? 29 !! 28
-        !! $month == 4|6|9|11 ?? 30
-        !! 31
-    }
+    sub IS-LEAP-YEAR($y) { $y %% 4 and not $y %% 100 or $y %% 400 }
+    proto method is-leap-year(|) { * }
+    multi method is-leap-year(Dateish:D:) { IS-LEAP-YEAR($!year) }
+    multi method is-leap-year(Dateish: $y) { IS-LEAP-YEAR($y) }
 
-    method daycount-from-ymd($y is copy, $m is copy, $d) {
+    sub DAYS-IN-MONTH($year, $month) {
+        state @l = 0, 31, 0, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31;
+        $month == 2 ?? 28 + IS-LEAP-YEAR($year) !! @l.AT-POS($month);
+    }
+    proto method days-in-month(|) { * }
+    multi method days-in-month(Dateish:D:) { DAYS-IN-MONTH($!year,$!month) }
+    multi method days-in-month(Dateish: $y, $m) { DAYS-IN-MONTH($y,$m) }
+
+    method daycount-from-ymd(Int() $y is copy, Int() $m is copy, $d) {
         # taken from <http://www.merlyn.demon.co.uk/daycount.htm>
-        $y .= Int;
-        $m .= Int;
         if $m < 3 {
             $m += 12;
             --$y;
@@ -29,11 +41,11 @@ my role Dateish {
 
     method ymd-from-daycount($daycount) {
         # taken from <http://www.merlyn.demon.co.uk/daycount.htm>
-        my int $day = $daycount.Int + 678881;
-        my int $t = (4 * ($day + 36525)) div 146097 - 1;
-        my int $year = 100 * $t;
-        $day = $day - (36524 * $t + ($t div 4));
-        $t = (4 * ($day + 366)) div 1461 - 1;
+        my Int $dc = $daycount.Int + 678881;
+        my Int $ti = (4 * ($dc + 36525)) div 146097 - 1;
+        my Int $year = 100 * $ti;
+        my int $day = $dc - (36524 * $ti + ($ti div 4));
+        my int $t = (4 * ($day + 366)) div 1461 - 1;
         $year = $year + $t;
         $day = $day - (365 * $t + ($t div 4));
         my int $month = (5 * $day + 2) div 153;
@@ -46,32 +58,32 @@ my role Dateish {
     }
 
     method get-daycount {
-        self.daycount-from-ymd($.year, $.month, $.day)
+        self.daycount-from-ymd($!year, $!month, $!day)
     }
 
-    method day-of-month() { $.day }
-  
+    method day-of-month() { $!day }
+
     method day-of-week($daycount = self.get-daycount) {
         ($daycount + 2) % 7 + 1
     }
 
     method week() { # algorithm from Claus Tøndering
-        my $a = $.year - ($.month <= 2).floor.Int;
+        my $a = $!year - ($!month <= 2).floor.Int;
         my $b = $a div 4 - $a div 100 + $a div 400;
         my $c = ($a - 1) div 4 - ($a - 1) div 100 + ($a - 1) div 400;
         my $s = $b - $c;
-        my $e = $.month <= 2 ?? 0 !! $s + 1;
-        my $f = $.day + do $.month <= 2
-         ?? 31*($.month - 1) - 1
-         !! (153*($.month - 3) + 2) div 5 + 58 + $s;
+        my $e = $!month <= 2 ?? 0 !! $s + 1;
+        my $f = $!day + do $!month <= 2
+         ?? 31*($!month - 1) - 1
+         !! (153*($!month - 3) + 2) div 5 + 58 + $s;
 
         my $g = ($a + $b) % 7;
         my $d = ($f + $g - $e) % 7;
         my $n = $f + 3 - $d;
 
-           $n < 0           ?? ($.year - 1, 53 - ($g - $s) div 5)
-        !! $n > 364 + $s    ?? ($.year + 1, 1)
-        !!                     ($.year,     $n div 7 + 1);
+           $n < 0           ?? ($!year - 1, 53 - ($g - $s) div 5)
+        !! $n > 364 + $s    ?? ($!year + 1, 1)
+        !!                     ($!year,     $n div 7 + 1);
     }
 
     method week-year() {
@@ -83,11 +95,11 @@ my role Dateish {
     }
 
     method weekday-of-month {
-        ($.day - 1) div 7 + 1
+        ($!day - 1) div 7 + 1
     }
 
     method day-of-year() {
-        [+] $.day, map { self.days-in-month($.year, $^m) }, 1 ..^ $.month
+        [+] flat $!day, map { self.days-in-month($!year, $^m) }, 1 ..^ $!month
     }
 
     method check-value($val is copy, $name, $range, :$allow-nonint) {
@@ -99,17 +111,16 @@ my role Dateish {
                         range   => $range,
                ).throw;
     }
-  
-    method check-date { 
-        self.check-value($.month, 'month', 1 .. 12);
-        self.check-value($.day, "day of $.year/$.month",
+
+    method check-date {
+        self.check-value($!month, 'month', 1 .. 12);
+        self.check-value($!day, "day of $!year/$!month",
             1 .. self.days-in-month);
     }
 
-    method truncate-parts(Cool $unit, %parts? is copy) {
+    method truncate-parts(Cool:D $unit, %parts? is copy) {
         # Helper for DateTime.truncated-to and Date.truncated-to.
-        X::DateTime::InvalidDeltaUnit.new(:$unit).throw
-            unless $unit eq any(@UNITS);
+        self!VALID-UNIT($unit);
         if $unit eq 'week' | 'weeks' {
             my $dc = self.get-daycount;
             my $new-dc = $dc - self.day-of-week($dc) + 1;
@@ -130,13 +141,14 @@ sub default-formatter(DateTime $dt, Bool :$subseconds) {
     my $o = $dt.offset;
     $o %% 60
         or warn "Default DateTime formatter: offset $o not divisible by 60.\n";
-    sprintf '%04d-%02d-%02dT%02d:%02d:%s%s',
-        $dt.year, $dt.month, $dt.day, $dt.hour, $dt.minute,
+    my $year = sprintf((0 <= $dt.year <= 9999 ?? '%04d' !! '%+05d'), $dt.year);
+    sprintf '%s-%02d-%02dT%02d:%02d:%s%s',
+        $year, $dt.month, $dt.day, $dt.hour, $dt.minute,
         $subseconds
           ?? $dt.second.fmt('%09.6f')
           !! $dt.whole-second.fmt('%02d'),
         do $o
-         ?? sprintf '%s%02d%02d',
+         ?? sprintf '%s%02d:%02d',
                 $o < 0 ?? '-' !! '+',
                 ($o.abs / 60 / 60).floor,
                 ($o.abs / 60 % 60).floor
@@ -158,23 +170,19 @@ sub get-local-timezone-offset {
 }
 
 my class DateTime does Dateish {
-     has Int $.year;
-     has Int $.month = 1;
-     has Int $.day = 1;
- 
-     has Int $.hour      = 0;
-     has Int $.minute    = 0;
-     has     $.second    = 0.0;
-     has     $.timezone  = 0; # UTC
-     has     &.formatter = &default-formatter;
-       # Not an optimization but a necessity to ensure that
-       # $dt.utc.local.utc is equivalent to $dt.utc. Otherwise,
-       # DST-induced ambiguity could ruin our day.
-    
-     multi method new() {
-        fail "Must provide arguments to DateTime.new()";
+    has Int $.hour      = 0;
+    has Int $.minute    = 0;
+    has     $.second    = 0.0;
+    has     $.timezone  = 0; # UTC
+    has     &.formatter = &default-formatter;
+      # Not an optimization but a necessity to ensure that
+      # $dt.utc.local.utc is equivalent to $dt.utc. Otherwise,
+      # DST-induced ambiguity could ruin our day.
+
+    multi method new() {
+        fail "Required named parameter 'year' is missing for DateTime.new()";
     }
- 
+
     multi method new(Int :$year!, :&formatter=&default-formatter, *%_) {
         my $dt = self.bless(:$year, :&formatter, |%_);
         $dt.check-date;
@@ -216,19 +224,19 @@ my class DateTime does Dateish {
         }
     }
 
-    multi method new(Date :$date!, *%_) {
+    multi method new(Date:D :$date!, *%_) {
         self.new(year => $date.year, month => $date.month,
             day => $date.day, |%_)
     }
 
-    multi method new(Instant $i, :$timezone=0, :&formatter=&default-formatter) {
+    multi method new(Instant:D $i, :$timezone=0, :&formatter=&default-formatter) {
         my ($p, $leap-second) = $i.to-posix;
         my $dt = self.new: floor($p - $leap-second).Int, :&formatter;
         $dt.clone(second => ($dt.second + $p % 1 + $leap-second)
             ).in-timezone($timezone);
     }
 
-    multi method new(Int $time is copy, :$timezone=0, :&formatter=&default-formatter) {
+    multi method new(Int:D $time is copy, :$timezone=0, :&formatter=&default-formatter) {
         # Interpret $time as a POSIX time.
         my $second  = $time % 60; $time = $time div 60;
         my $minute  = $time % 60; $time = $time div 60;
@@ -251,11 +259,11 @@ my class DateTime does Dateish {
     }
 
     multi method new(Str $format, :$timezone is copy = 0, :&formatter=&default-formatter) {
-        $format ~~ /^ (\d**4) '-' (\d\d) '-' (\d\d) T (\d\d) ':' (\d\d) ':' (\d\d) (Z || (<[\-\+]>) (\d\d) (':'? (\d\d))? )? $/
+        $format ~~ /^ (\d**4) '-' (\d\d) '-' (\d\d) <[Tt]> (\d\d) ':' (\d\d) ':' (\d\d) (<[Zz]> || (<[\-\+]>) (\d\d) (':'? (\d\d))? )? $/
             or X::Temporal::InvalidFormat.new(
                     invalid-str => $format,
                     target      => 'DateTime',
-                    format      => 'an ISO 8601 timestamp (yyyy-mm-ddThh::mm::ssZ or yyyy-mm-ddThh::mm::ss+0100)',
+                    format      => 'an ISO 8601 timestamp (yyyy-mm-ddThh:mm:ssZ or yyyy-mm-ddThh:mm:ss+01:00)',
                 ).throw;
         my $year   = (+$0).Int;
         my $month  = (+$1).Int;
@@ -266,9 +274,16 @@ my class DateTime does Dateish {
         if $6 {
             $timezone
                 and X::DateTime::TimezoneClash.new.throw;
-            if $6 eq 'Z' {
-                $timezone = 0;                
+            if $6.chars == 1 {
+                $timezone = 0;
             } else {
+                if $6[2] && $6[2][0] > 59 {
+                    X::OutOfRange.new(
+                        what    => "minutes of timezone",
+                        got     => +$6[2][0],
+                        range   => 0..59,
+                   ).throw;
+                }
                 $timezone = (($6[1]*60 + ($6[2][0] // 0)) * 60).Int;
                   # RAKUDO: .Int is needed to avoid to avoid the nasty '-0'.
                 $6[0] eq '-' and $timezone = -$timezone;
@@ -278,7 +293,7 @@ my class DateTime does Dateish {
             :$second, :$timezone, :&formatter);
     }
 
-    method now(:$timezone=$*TZ, :&formatter=&default-formatter) {
+    method now(:$timezone=$*TZ, :&formatter=&default-formatter) returns DateTime:D {
         self.new(now, :$timezone, :&formatter)
     }
 
@@ -295,20 +310,20 @@ my class DateTime does Dateish {
     }
 
     method Instant() {
-        Instant.from-posix: self.posix + $.second % 1, $.second >= 60;
+        Instant.from-posix: self.posix + $!second % 1, $!second >= 60;
     }
 
     method posix($ignore-timezone?) {
         $ignore-timezone or self.offset == 0
             or return self.utc.posix;
         # algorithm from Claus Tøndering
-        my $a = (14 - $.month.Int) div 12;
-        my $y = $.year.Int + 4800 - $a;
-        my $m = $.month.Int + 12 * $a - 3;
-        my $jd = $.day + (153 * $m + 2) div 5 + 365 * $y
+        my $a = (14 - $!month.Int) div 12;
+        my $y = $!year.Int + 4800 - $a;
+        my $m = $!month.Int + 12 * $a - 3;
+        my $jd = $!day + (153 * $m + 2) div 5 + 365 * $y
             + $y div 4 - $y div 100 + $y div 400 - 32045;
         ($jd - 2440588) * 24 * 60 * 60
-            + 60*(60*$.hour + $.minute) + self.whole-second;
+            + 60*(60*$!hour + $!minute) + self.whole-second;
     }
 
     method offset {
@@ -327,17 +342,18 @@ my class DateTime does Dateish {
         die "More than one time unit supplied"
             if %unit.keys > 1;
 
-        my ($unit, $amount) = %unit.kv;
+        die "No time unit supplied"
+            unless %unit.keys;
 
-        X::DateTime::InvalidDeltaUnit.new(:$unit).throw
-            unless $unit eq any(@UNITS);
+        my ($unit, $amount) = %unit.kv;
+        self!VALID-UNIT($unit);
 
         my ($hour, $minute) = $!hour, $!minute;
         my $date;
 
         given $unit {
             when 'second' | 'seconds' {
-                return DateTime.new(self.Instant + $amount);
+                return self.new(self.Instant + $amount, :$.timezone, :&.formatter);
             }
 
             when 'minute' | 'minutes' { $minute += $amount; proceed }
@@ -358,7 +374,10 @@ my class DateTime does Dateish {
                 $month += $amount;
                 $year += floor(($month - 1) / 12);
                 $month = ($month - 1) % 12 + 1;
-                $date = Date.new(:$year, :$month, :$!day);
+                # If we overflow on days in the month, rather than throw an
+                # exception, we just clip to the last of the month
+                my $day = $!day min $.days-in-month($year, $month);
+                $date = Date.new(:$year, :$month, :$day);
                 succeed;
             }
 
@@ -386,7 +405,7 @@ my class DateTime does Dateish {
                 }
             }
         }
-        self.new(:$date, :$hour, :$minute, :$second);
+        self.new(:$date, :$hour, :$minute, :$second, :$.timezone, :&.formatter);
     }
 
     method earlier(*%unit) {
@@ -411,7 +430,7 @@ my class DateTime does Dateish {
     }
 
     method whole-second() {
-        floor($.second).Int
+        $!second.Int
     }
 
     method in-timezone($timezone) {
@@ -423,15 +442,17 @@ my class DateTime does Dateish {
         # I don't know, but it passes the tests!
         my $a = ($!second >= 60 ?? 59 !! $!second)
             + $new-offset - $old-offset;
-        %parts<second> = $!second >= 60 ?? $!second !! ($a % 60).Int;
-        my $b = $!minute + floor $a / 60;
-        %parts<minute> = ($b % 60).Int;
-        my $c = $!hour + floor $b / 60;
-        %parts<hour> = ($c % 24).Int;
+        %parts<second> = $!second >= 60 ?? $!second !! $a % 60;
+        my Int $b = $!minute + floor($a) div 60;
+        %parts<minute> = $b % 60;
+        my Int $c = $!hour + $b div 60;
+        %parts<hour> = $c % 24;
+
         # Let Dateish handle any further rollover.
-        floor $c / 24 and %parts<year month day> =
-           self.ymd-from-daycount\
-               (self.get-daycount + floor $c / 24);
+        if ($c div 24) {
+            %parts<year month day> =
+                self.ymd-from-daycount(self.get-daycount + $c div 24);
+        }
         self.clone-without-validating:
             :$timezone, |%parts;
     }
@@ -444,7 +465,7 @@ my class DateTime does Dateish {
     }
 
     method Date() {
-        Date.new(:$.year, :$.month, :$.day);
+        Date.new(:$!year, :$!month, :$!day);
     }
 
     method Str() {
@@ -452,11 +473,11 @@ my class DateTime does Dateish {
     }
 
     multi method perl(DateTime:D:) {
-        sprintf 'DateTime.new(%s)', join ', ', map { "{.key} => {.value}" }, do
-            :$.year, :$.month, :$.day, :$.hour, :$.minute,
-            second => $.second.perl,
-            (timezone => $.timezone.perl
-                unless $.timezone === 0),
+        sprintf '%s.new(%s)', self.^name, join ', ', map { "{.key} => {.value}" }, do
+            :$!year, :$!month, :$!day, :$!hour, :$!minute,
+            second => $!second.perl,
+            (timezone => $!timezone.perl
+                unless $!timezone === 0),
             (formatter => $.formatter.perl
                 unless &.formatter eqv &default-formatter)
     }
@@ -466,11 +487,33 @@ my class DateTime does Dateish {
     }
 }
 
-my class Date does Dateish {
-    has Int $.year;
-    has Int $.month = 1;
-    has Int $.day = 1;
+multi sub infix:«<»(DateTime:D \a, DateTime:D \b) {
+    a.Instant < b.Instant
+}
+multi sub infix:«>»(DateTime:D \a, DateTime:D \b) {
+    a.Instant > b.Instant
+}
+multi sub infix:«<=»(DateTime:D \a, DateTime:D \b) {
+    a.Instant <= b.Instant
+}
+multi sub infix:«>=»(DateTime:D \a, DateTime:D \b) {
+    a.Instant >= b.Instant
+}
+multi sub infix:«==»(DateTime:D \a, DateTime:D \b) {
+    a.Instant == b.Instant
+}
+multi sub infix:«!=»(DateTime:D \a, DateTime:D \b) {
+    a.Instant != b.Instant
+}
+multi sub infix:«<=>»(DateTime:D \a, DateTime:D \b) {
+    a.Instant <=> b.Instant
+}
+multi sub infix:«cmp»(DateTime:D \a, DateTime:D \b) {
+    a.Instant cmp b.Instant
+}
 
+
+my class Date does Dateish {
     has Int $.daycount;
 
     method !set-daycount($dc) { $!daycount = $dc }
@@ -500,19 +543,24 @@ my class Date does Dateish {
     multi method new() {
         my $n = self.today;
         if $n.month == 12 && $n.day >= 24 {
-            Date.new($n.year + 1, 12, 24);
+            self.new($n.year + 1, 12, 24);
         } else {
-            Date.new($n.year, 12, 24);
+            self.new($n.year, 12, 24);
         }
     }
 
-    multi method new(DateTime $dt) {
+    multi method new(Dateish $d) {
         self.bless(
-            :year($dt.year), :month($dt.month), :day($dt.day),
-            :daycount(self.daycount-from-ymd($dt.year,$dt.month,$dt.day))
+            :year($d.year), :month($d.month), :day($d.day),
+            :daycount(self.daycount-from-ymd($d.year,$d.month,$d.day))
         );
     }
 
+    multi method new(Instant $i) {
+        my $dt = DateTime.new($i);
+        self.new($dt);
+    }
+
     multi method WHICH(Date:D:) {
         nqp::box_s(
             nqp::concat(
@@ -540,10 +588,11 @@ my class Date does Dateish {
         die "More than one time unit supplied"
             if %unit.keys > 1;
 
-        my ($unit, $amount) = %unit.kv;
+        die "No time unit supplied"
+            unless %unit.keys;
 
-        X::DateTime::InvalidDeltaUnit.new(:$unit).throw
-            unless $unit eq any(@UNITS);
+        my ($unit, $amount) = %unit.kv;
+        self!VALID-UNIT($unit);
 
         my $date;
 
@@ -560,17 +609,20 @@ my class Date does Dateish {
                 $month += $amount;
                 $year += floor(($month - 1) / 12);
                 $month = ($month - 1) % 12 + 1;
-                $date = Date.new(:$year, :$month, :$!day);
+                # If we overflow on days in the month, rather than throw an
+                # exception, we just clip to the last of the month
+                my $day = $!day min $.days-in-month($year, $month);
+                $date = Date.new(:$year, :$month, :$day);
                 succeed;
             }
 
             when 'year' | 'years' {
                 my $year = $!year + $amount;
-                $date = Date.new(:$year, :$!month, :$!day);
+                $date = self.new(:$year, :$!month, :$!day);
                 succeed;
             }
 
-            $date = Date.new-from-daycount(self.daycount + $day-delta);
+            $date = self.new-from-daycount(self.daycount + $day-delta);
         }
 
         $date;
@@ -586,72 +638,78 @@ my class Date does Dateish {
     }
 
     method succ() {
-        Date.new-from-daycount($!daycount + 1);
+        self.new-from-daycount($!daycount + 1);
     }
     method pred() {
-        Date.new-from-daycount($!daycount - 1);
+        self.new-from-daycount($!daycount - 1);
     }
 
     multi method gist(Date:D:) {
-        sprintf '%04d-%02d-%02d', $.year, $.month, $.day;
+        self.Str
     }
 
     multi method Str(Date:D:) {
-        sprintf '%04d-%02d-%02d', $.year, $.month, $.day;
+        my str $format = (0 <= $!year <= 9999 ?? '%04d-%02d-%02d' !! '%+05d-%02d-%02d');
+        sprintf $format, $!year, $!month, $!day;
     }
 
     multi method perl(Date:D:) {
-        "Date.new($.year.perl(), $.month.perl(), $.day.perl())";
+        self.^name ~ ".new($!year.perl(), $!month.perl(), $!day.perl())";
+    }
+
+    multi method ACCEPTS(Date:D: DateTime:D $dt) {
+        $dt.year == $!year && $dt.month == $!month && $dt.day == $!day;
     }
 }
- 
-multi infix:<+>(Date:D $d, Int:D $x) {
+
+multi sub infix:<+>(Date:D $d, Int:D $x) {
     Date.new-from-daycount($d.daycount + $x)
 }
-multi infix:<+>(Int:D $x, Date:D $d) {
+multi sub infix:<+>(Int:D $x, Date:D $d) {
     Date.new-from-daycount($d.daycount + $x)
 }
-multi infix:<->(Date:D $d, Int:D $x) {
+multi sub infix:<->(Date:D $d, Int:D $x) {
     Date.new-from-daycount($d.daycount - $x)
 }
-multi infix:<->(Date:D $a, Date:D $b) {
+multi sub infix:<->(Date:D $a, Date:D $b) {
     $a.daycount - $b.daycount;
 }
-multi infix:<cmp>(Date:D $a, Date:D $b) {
+multi sub infix:<cmp>(Date:D $a, Date:D $b) {
     $a.daycount cmp $b.daycount
 }
-multi infix:«<=>»(Date:D $a, Date:D $b) {
+multi sub infix:«<=>»(Date:D $a, Date:D $b) {
     $a.daycount <=> $b.daycount
 }
-multi infix:<==>(Date:D $a, Date:D $b) {
+multi sub infix:<==>(Date:D $a, Date:D $b) {
     $a.daycount == $b.daycount
 }
-multi infix:«<=»(Date:D $a, Date:D $b) {
+multi sub infix:«<=»(Date:D $a, Date:D $b) {
     $a.daycount <= $b.daycount
 }
-multi infix:«<»(Date:D $a, Date:D $b) {
+multi sub infix:«<»(Date:D $a, Date:D $b) {
     $a.daycount < $b.daycount
 }
-multi infix:«>=»(Date:D $a, Date:D $b) {
+multi sub infix:«>=»(Date:D $a, Date:D $b) {
     $a.daycount >= $b.daycount
 }
-multi infix:«>»(Date:D $a, Date:D $b) {
+multi sub infix:«>»(Date:D $a, Date:D $b) {
     $a.daycount > $b.daycount
 }
 
-$PROCESS::TZ = get-local-timezone-offset();
+multi sub INITIALIZE_DYNAMIC('$*TZ') {
+    PROCESS::<$TZ> = get-local-timezone-offset();
+}
 
 sub sleep($seconds = Inf --> Nil) {
-    if $seconds ~~ (Inf|Whatever) {
+    if $seconds == Inf || nqp::istype($seconds,Whatever) {
         nqp::sleep(1e16) while True;
     }
     elsif $seconds > 0 {
         nqp::sleep($seconds.Num);
     }
-    Nil;
 }
 
-sub sleep-timer (Real $seconds = Inf --> Duration) {
+sub sleep-timer(Real() $seconds = Inf --> Duration) {
     if $seconds <= 0 {
         Duration.new(0);
     }
@@ -662,8 +720,8 @@ sub sleep-timer (Real $seconds = Inf --> Duration) {
     }
 }
 
-sub sleep-till (Instant $till --> Bool) {
-    my $seconds = $till - now;
+sub sleep-until(Instant() $until --> Bool) {
+    my $seconds = $until - now;
     return False if $seconds < 0;
 
     1 while $seconds = sleep-timer($seconds);
@@ -671,23 +729,23 @@ sub sleep-till (Instant $till --> Bool) {
 }
 
 # =begin pod
-# 
+#
 # =head1 SEE ALSO
-# Perl 6 spec <S32-Temporal|http://perlcabal.org/syn/S32/Temporal.html>.
+# Perl 6 spec <S32-Temporal|http://design.perl6.org/S32/Temporal.html>.
 # The Perl 5 DateTime Project home page L<http://datetime.perl.org>.
 # Perl 5 perldoc L<doc:DateTime> and L<doc:Time::Local>.
-#  
+#
 # The best yet seen explanation of calendars, by Claus Tøndering
 # L<Calendar FAQ|http://www.tondering.dk/claus/calendar.html>.
 # Similar algorithms at L<http://www.hermetic.ch/cal_stud/jdn.htm>
 # and L<http://www.merlyn.demon.co.uk/daycount.htm>.
-#  
+#
 # <ISO 8601|http://en.wikipedia.org/wiki/ISO_8601>
 # <Time zones|http://en.wikipedia.org/wiki/List_of_time_zones>
-# 
+#
 # As per the recommendation, the strftime() method has bee moved into a
 # loadable module called DateTime::strftime.
-# 
+#
 # =end pod
 
 # vim: ft=perl6 expandtab sw=4
