@@ -2,17 +2,26 @@ class Version {
     has @.parts;
     has Bool $.plus = False;
 
-    multi method new(Str:D $s) {
+    multi method new(Whatever) { self.bless(:parts(*)) }
+    multi method new(Str() $s) {
         my @parts = $s.comb(/:r '*' || \d+ || <.alpha>+/);
         for @parts {
-            $_ .= Numeric if .Numeric.defined ;
-            $_ = * if $_ eq '*';
+            if $_ eq '*' {
+                $_ = *;
+            }
+            else {
+                my $numeric = .Numeric;
+                $_ = $numeric if $numeric.defined;
+            }
         }
-        self.bless(:parts(@parts), :plus($s.substr(*-1) eq '+'));
+        self.bless(
+          :parts(@parts),
+          :plus($s.ends-with("+")),
+        );
     };
 
     multi method Str(Version:D:) {
-        @!parts.map({ $_ ~~ Whatever ?? '*' !! $_}).join('.') 
+        @!parts.map({ nqp::istype($_,Whatever) ?? '*' !! $_}).join('.')
           ~ ($!plus ?? '+' !! '');
     }
     multi method gist(Version:D:) { 'v' ~ self.Str }
@@ -22,10 +31,10 @@ class Version {
     }
     multi method ACCEPTS(Version:D: Version:D $other) {
         for @!parts.kv -> $i, $v {
-            next if $v ~~ Whatever;
+            next if nqp::istype($v,Whatever);
             my $o = $other.parts[$i];
             return True unless defined $o;
-            next if $o ~~ Whatever;
+            next if nqp::istype($o,Whatever);
             return $.plus if $o after  $v;
             return False  if $o before $v;
         }
