@@ -26,12 +26,11 @@ public final class Binder {
     private static final int SIG_ELEM_SLURPY_POS          = 8;
     private static final int SIG_ELEM_SLURPY_NAMED        = 16;
     private static final int SIG_ELEM_SLURPY_LOL          = 32;
-    private static final int SIG_ELEM_SLURPY              = (SIG_ELEM_SLURPY_POS | SIG_ELEM_SLURPY_NAMED | SIG_ELEM_SLURPY_LOL);
     private static final int SIG_ELEM_INVOCANT            = 64;
     private static final int SIG_ELEM_MULTI_INVOCANT      = 128;
     private static final int SIG_ELEM_IS_RW               = 256;
     private static final int SIG_ELEM_IS_COPY             = 512;
-    private static final int SIG_ELEM_IS_PARCEL           = 1024;
+    private static final int SIG_ELEM_IS_RAW              = 1024;
     private static final int SIG_ELEM_IS_OPTIONAL         = 2048;
     private static final int SIG_ELEM_ARRAY_SIGIL         = 4096;
     private static final int SIG_ELEM_HASH_SIGIL          = 8192;
@@ -46,6 +45,8 @@ public final class Binder {
     private static final int SIG_ELEM_NATIVE_NUM_VALUE    = 4194304;
     private static final int SIG_ELEM_NATIVE_STR_VALUE    = 8388608;
     private static final int SIG_ELEM_NATIVE_VALUE        = (SIG_ELEM_NATIVE_INT_VALUE | SIG_ELEM_NATIVE_NUM_VALUE | SIG_ELEM_NATIVE_STR_VALUE);
+    private static final int SIG_ELEM_SLURPY_ONEARG       = 16777216;
+    private static final int SIG_ELEM_SLURPY              = (SIG_ELEM_SLURPY_POS | SIG_ELEM_SLURPY_NAMED | SIG_ELEM_SLURPY_LOL | SIG_ELEM_SLURPY_ONEARG);
     
     /* Hints for Parameter attributes. */
     private static final int HINT_variable_name = 0;
@@ -65,6 +66,7 @@ public final class Binder {
     private static final int HINT_ENUMMAP_storage = 0;
     private static final int HINT_CAPTURE_list = 0;
     private static final int HINT_CAPTURE_hash = 1;
+    private static final int HINT_LIST_reified = 0;
     private static final int HINT_SIG_params = 0;
 
     private static SixModelObject createBox(ThreadContext tc, RakOps.GlobalExt gcx, Object arg, int flag) {
@@ -84,7 +86,7 @@ public final class Binder {
             int numParams, int numPosArgs, boolean tooMany) {
         int arity = 0;
         int count = 0;
-        String fail = tooMany ? "Too many" : "Not enough";
+        String fail = tooMany ? "Too many" : "Too few";
 
         /* Work out how many we could have been passed. */
         for (int i = 0; i < numParams; i++) {
@@ -98,8 +100,8 @@ public final class Binder {
                 continue;
             if ((flags & SIG_ELEM_SLURPY_NAMED) != 0)
                 continue;
-            if ((flags & SIG_ELEM_SLURPY_POS) != 0) {
-                count = -1;
+            if ((flags & SIG_ELEM_SLURPY) != 0) {
+                count = -1000; // cargo-culted from BOOTSTRAP.nqp: "in case a pos can sneak past a slurpy somehow"
             }
             else if ((flags & SIG_ELEM_IS_OPTIONAL) != 0) {
                 count++;
@@ -113,24 +115,16 @@ public final class Binder {
         /* Now generate decent error. */
         if (arity == count)
             return String.format(
-                "%s positional parameters passed; got %d but expected %d",
-                fail, numPosArgs, arity);
-        else if (count == -1)
+                "%s positionals passed; expected %d arguments but got %d",
+                fail, arity, numPosArgs);
+        else if (count <= -1)
             return String.format(
-                "%s positional parameters passed; got %d but expected at least %d",
-                fail, numPosArgs, arity);
+                "%s positionals passed; expected at least %d arguments but got only %d",
+                fail, arity, numPosArgs);
         else
             return String.format(
-                "%s positional parameters passed; got %d but expected between %d and %d",
-                fail, numPosArgs, arity, count);
-    }
-    
-    /* Returns an appropriate failure mode (junction fail or normal fail). */
-    private static int junc_or_fail(RakOps.GlobalExt gcx, SixModelObject value) {
-        if (value.st.WHAT == gcx.Junction)
-            return BIND_RESULT_JUNCTION;
-        else
-            return BIND_RESULT_FAIL;
+                "%s positionals passed; expected %d %s %d arguments but got %d",
+                fail, arity, arity + 1 == count ? "or" : "to" , count, numPosArgs);
     }
     
     /* Binds any type captures. */
@@ -145,7 +139,7 @@ public final class Binder {
     
     /* Assigns an attributive parameter to the desired attribute. */
     private static int assignAttributive(ThreadContext tc, CallFrame cf, String varName,
-            int paramFlags, SixModelObject attrPackage, SixModelObject value, String[] error) {
+            int paramFlags, SixModelObject attrPackage, SixModelObject value, Object[] error) {
         /* Find self. */
         StaticCodeInfo sci = cf.codeRef.staticInfo;
         Integer selfIdx = sci.oTryGetLexicalIdx("self");
@@ -175,7 +169,7 @@ public final class Binder {
     
     /* Returns an appropriate failure mode (junction fail or normal fail). */
     private static int juncOrFail(ThreadContext tc, RakOps.GlobalExt gcx, SixModelObject value) {
-        if (value.st.WHAT == gcx.Junction)
+        if (value.st.WHAT == gcx.Junction && Ops.isconcrete(value, tc) != 0)
             return BIND_RESULT_JUNCTION;
         else
             return BIND_RESULT_FAIL;
@@ -197,7 +191,7 @@ public final class Binder {
     private static final CallSiteDescriptor bindThrower = new CallSiteDescriptor(
         new byte[] { CallSiteDescriptor.ARG_OBJ, CallSiteDescriptor.ARG_OBJ, CallSiteDescriptor.ARG_STR }, null);
     private static int bindOneParam(ThreadContext tc, RakOps.GlobalExt gcx, CallFrame cf, SixModelObject param,
-            Object origArg, byte origFlag, boolean noNomTypeCheck, String[] error) {
+            Object origArg, byte origFlag, boolean noNomTypeCheck, Object[] error) {
         /* Get parameter flags and variable name. */
         param.get_attribute_native(tc, gcx.Parameter, "$!flags", HINT_flags);
         int paramFlags = (int)tc.native_i;
@@ -216,8 +210,42 @@ public final class Binder {
         
         /* Check if boxed/unboxed expections are met. */
         int desiredNative = paramFlags & SIG_ELEM_NATIVE_VALUE;
-        int gotNative = origFlag & 7;
-        if (desiredNative == 0 && gotNative == CallSiteDescriptor.ARG_OBJ) {
+        boolean is_rw = (paramFlags & SIG_ELEM_IS_RW) != 0;
+        int gotNative = origFlag & (CallSiteDescriptor.ARG_INT | CallSiteDescriptor.ARG_NUM | CallSiteDescriptor.ARG_STR);
+        if (is_rw && desiredNative != 0) {
+            switch (desiredNative) {
+            case SIG_ELEM_NATIVE_INT_VALUE:
+                if (gotNative != 0 || Ops.iscont_i((SixModelObject)origArg) == 0) {
+                    if (error != null)
+                        error[0] = String.format(
+                            "Expected a native int argument for '%s'",
+                            varName);
+                    return BIND_RESULT_FAIL;
+                }
+                break;
+            case SIG_ELEM_NATIVE_NUM_VALUE:
+                if (gotNative != 0 || Ops.iscont_n((SixModelObject)origArg) == 0) {
+                    if (error != null)
+                        error[0] = String.format(
+                            "Expected a native num argument for '%s'",
+                            varName);
+                    return BIND_RESULT_FAIL;
+                }
+                break;
+            case SIG_ELEM_NATIVE_STR_VALUE:
+                if (gotNative != 0 || Ops.iscont_s((SixModelObject)origArg) == 0) {
+                    if (error != null)
+                        error[0] = String.format(
+                            "Expected a native str argument for '%s'",
+                            varName);
+                    return BIND_RESULT_FAIL;
+                }
+                break;
+            }
+            flag = CallSiteDescriptor.ARG_OBJ;
+            arg_o = (SixModelObject)origArg;
+        }
+        else if (desiredNative == 0 && gotNative == CallSiteDescriptor.ARG_OBJ) {
             flag = gotNative;
             arg_o = (SixModelObject)origArg;
         }
@@ -295,12 +323,16 @@ public final class Binder {
          * bind if it passes the type check, or a native value that needs no
          * further checking. */
         SixModelObject decontValue = null;
-        if (flag == CallSiteDescriptor.ARG_OBJ) {
+        boolean didHLLTransform = false;
+        if (flag == CallSiteDescriptor.ARG_OBJ && !(is_rw && desiredNative != 0)) {
             /* We need to work on the decontainerized value. */
             decontValue = Ops.decont(arg_o, tc);
             
             /* HLL map it as needed. */
+            SixModelObject beforeHLLize = decontValue;
             decontValue = Ops.hllize(decontValue, tc);
+            if (decontValue != beforeHLLize)
+                didHLLTransform = true;
             
             /* Skip nominal type check if not needed. */
             if (!noNomTypeCheck) {
@@ -321,6 +353,15 @@ public final class Binder {
                     nomType = Ops.result_o(tc.curFrame);
                 }
 
+                /* If the expected type is Positional, see if we need to do the
+                 * positional bind failover. */
+                if (nomType == gcx.Positional && Ops.istype_nodecont(arg_o, gcx.PositionalBindFailover, tc) != 0) {
+                    SixModelObject ig = Ops.findmethod(tc, arg_o, "cache");
+                    Ops.invokeDirect(tc, ig, Ops.invocantCallSite, new Object[] { arg_o });
+                    arg_o = Ops.result_o(tc.curFrame);
+                    decontValue = Ops.decont(arg_o, tc);
+                }
+
                 /* If not, do the check. If the wanted nominal type is Mu, then
                  * anything goes. */
                 if (nomType != gcx.Mu && Ops.istype_nodecont(decontValue, nomType, tc) == 0) {
@@ -328,20 +369,54 @@ public final class Binder {
                     if (error != null) {
 
                         SixModelObject thrower = RakOps.getThrower(tc, "X::TypeCheck::Binding");
-                        if (thrower != null && decontValue.st.WHAT != gcx.Junction) {
-                            Ops.invokeDirect(tc, thrower,
-                                bindThrower, new Object[] { decontValue.st.WHAT, nomType.st.WHAT, varName });
-                            return BIND_RESULT_FAIL;
+                        if (thrower != null) {
+                            error[0] = thrower;
+                            error[1] = bindThrower;
+                            error[2] = new Object[] { decontValue, nomType.st.WHAT, (varName != null ? varName : "<anon>") };
                         }
-                        else
+                        else {
                             error[0] = String.format(
                                 "Nominal type check failed for parameter '%s'",
                                 varName);
+                        }
                     }
                 
                     /* Report junction failure mode if it's a junction. */
                     return juncOrFail(tc, gcx, decontValue);
                 }
+
+                /* Also enforce definedness check */
+                if ( (paramFlags & SIG_ELEM_DEFINEDNES_CHECK) != 0) {
+
+                    /* Don't check decontValue for concreteness though, but arg_o,
+                       seeing as we don't have a isconcrete_nodecont */
+                    if ((paramFlags & SIG_ELEM_UNDEFINED_ONLY) != 0 && Ops.isconcrete(arg_o, tc) == 1) {
+                        if (error != null) {
+                            if ((paramFlags & SIG_ELEM_INVOCANT) != 0) {
+                                error[0] = "Invocant requires a type object, but an object instance was passed";
+                            }
+                            else {
+                                error[0] = String.format(
+                                    "Parameter '%s' requires a type object, but an object instance was passed",
+                                    varName);
+                            }
+                        }
+                        return juncOrFail(tc, gcx, decontValue);
+                    }
+                    if ((paramFlags & SIG_ELEM_DEFINED_ONLY) != 0 && Ops.isconcrete(arg_o, tc) != 1) {
+                        if (error != null) {
+                            if ((paramFlags & SIG_ELEM_INVOCANT) != 0) {
+                                error[0] = "Invocant requires an instance, but a type object was passed";
+                            }
+                            else {
+                                error[0] = String.format(
+                                    "Parameter '%s' requires an instance, but a type object was passed",
+                                    varName);
+                            }
+                        }
+                        return juncOrFail(tc, gcx, decontValue);
+                    }
+                }
             }
         }
         
@@ -374,7 +449,8 @@ public final class Binder {
                     Ops.invokeDirect(tc, coerceMeth,
                         Ops.invocantCallSite,
                         new Object[] { decontValue });
-                    decontValue = Ops.decont(Ops.result_o(tc.curFrame), tc);
+                    arg_o = Ops.result_o(tc.curFrame);
+                    decontValue = Ops.decont(arg_o, tc);
                 }
                 else {
                     if (error != null)
@@ -406,14 +482,14 @@ public final class Binder {
             }
             
             /* Otherwise it's some objecty case. */
-            else if ((paramFlags & SIG_ELEM_IS_RW) != 0) {
+            else if (is_rw) {
                 /* XXX TODO Check if rw flag is set; also need to have a
                  * wrapper container that carries extra constraints. */
                 cf.oLex[sci.oTryGetLexicalIdx(varName)] = arg_o;
             }
-            else if ((paramFlags & SIG_ELEM_IS_PARCEL) != 0) {
+            else if ((paramFlags & SIG_ELEM_IS_RAW) != 0) {
                 /* Just bind the thing as is into the lexpad. */
-                cf.oLex[sci.oTryGetLexicalIdx(varName)] = arg_o;
+                cf.oLex[sci.oTryGetLexicalIdx(varName)] = didHLLTransform ? decontValue : arg_o;
             }
             else {
                 /* If it's an array, copy means make a new one and store,
@@ -422,7 +498,10 @@ public final class Binder {
                 if ((paramFlags & SIG_ELEM_ARRAY_SIGIL) != 0) {
                     SixModelObject bindee = decontValue;
                     if ((paramFlags & SIG_ELEM_IS_COPY) != 0) {
-                        bindee = RakOps.p6list(gcx.EMPTYARR.clone(tc), gcx.Array, gcx.True, tc);
+                        SixModelObject BOOTArray = tc.gc.BOOTArray;
+                        bindee = gcx.Array.st.REPR.allocate(tc, gcx.Array.st);
+                        bindee.bind_attribute_boxed(tc, gcx.List, "$!reified",
+                            HINT_LIST_reified, BOOTArray.st.REPR.allocate(tc, BOOTArray.st));
                         RakOps.p6store(bindee, decontValue, tc);
                     }
                     cf.oLex[sci.oTryGetLexicalIdx(varName)] = bindee;
@@ -434,7 +513,7 @@ public final class Binder {
                     if ((paramFlags & SIG_ELEM_IS_COPY) != 0) {
                         SixModelObject BOOTHash = tc.gc.BOOTHash;
                         bindee = gcx.Hash.st.REPR.allocate(tc, gcx.Hash.st);
-                        bindee.bind_attribute_boxed(tc, gcx.EnumMap, "$!storage",
+                        bindee.bind_attribute_boxed(tc, gcx.Map, "$!storage",
                             HINT_ENUMMAP_storage, BOOTHash.st.REPR.allocate(tc, BOOTHash.st));
                         RakOps.p6store(bindee, decontValue, tc);
                     }
@@ -615,7 +694,8 @@ public final class Binder {
         /* Otherwise, go by sigil to pick the correct default type of value. */
         else {
             if ((flags & SIG_ELEM_ARRAY_SIGIL) != 0) {
-                return RakOps.p6list(null, gcx.Array, gcx.True, tc);
+                SixModelObject res = gcx.Array.st.REPR.allocate(tc, gcx.Array.st);
+                return res;
             }
             else if ((flags & SIG_ELEM_HASH_SIGIL) != 0) {
                 SixModelObject res = gcx.Hash.st.REPR.allocate(tc, gcx.Hash.st);
@@ -631,9 +711,11 @@ public final class Binder {
      * into the provided callframe. Returns BIND_RESULT_OK if binding works out,
      * BIND_RESULT_FAIL if there is a failure and BIND_RESULT_JUNCTION if the
      * failure was because of a Junction being passed (meaning we need to auto-thread). */
+     private static final CallSiteDescriptor slurpyFromArgs = new CallSiteDescriptor(
+        new byte[] { CallSiteDescriptor.ARG_OBJ, CallSiteDescriptor.ARG_OBJ }, null);
     public static int bind(ThreadContext tc, RakOps.GlobalExt gcx, CallFrame cf, SixModelObject params,
             CallSiteDescriptor csd, Object[] args,
-            boolean noNomTypeCheck, String[] error) {
+            boolean noNomTypeCheck, Object[] error) {
         int bindFail = BIND_RESULT_OK;
         int curPosArg = 0;
         
@@ -718,7 +800,7 @@ public final class Binder {
             else if ((flags & SIG_ELEM_SLURPY_NAMED) != 0) {
                 SixModelObject slurpy = vmHashOfRemainingNameds(tc, gcx, namedArgsCopy, args);
                 SixModelObject bindee = gcx.Hash.st.REPR.allocate(tc, gcx.Hash.st);
-                bindee.bind_attribute_boxed(tc, gcx.EnumMap, "$!storage",
+                bindee.bind_attribute_boxed(tc, gcx.Map, "$!storage",
                     HINT_ENUMMAP_storage, slurpy);
                 bindFail = bindOneParam(tc, gcx, cf, param, bindee, CallSiteDescriptor.ARG_OBJ,
                     noNomTypeCheck, error);
@@ -733,7 +815,7 @@ public final class Binder {
             /* Otherwise, maybe it's a positional of some kind. */
             else if (namedNames == null) {
                 /* Slurpy or LoL-slurpy? */
-                if ((flags & (SIG_ELEM_SLURPY_POS | SIG_ELEM_SLURPY_LOL)) != 0) {
+                if ((flags & (SIG_ELEM_SLURPY_POS | SIG_ELEM_SLURPY_LOL | SIG_ELEM_SLURPY_ONEARG)) != 0) {
                     /* Create Perl 6 array, create VM array of all remaining things,
                      * then store it. */
                     SixModelObject slurpy = gcx.EMPTYARR.clone(tc);
@@ -754,18 +836,14 @@ public final class Binder {
                         }
                         curPosArg++;
                     }
-                    
-                    SixModelObject bindee;
-                    if ((flags & SIG_ELEM_SLURPY_POS) != 0) {
-                        if ((flags & SIG_ELEM_IS_RW) != 0)
-                            bindee = RakOps.p6list(slurpy, gcx.List, gcx.True, tc);
-                        else
-                            bindee = RakOps.p6list(slurpy, gcx.Array, gcx.True, tc);
-                    }
-                    else {
-                        bindee = RakOps.p6list(slurpy, gcx.LoL, gcx.False, tc);
-                    }
-                    
+
+                    SixModelObject slurpyType = (flags & SIG_ELEM_IS_RAW) != 0 ? gcx.List : gcx.Array;
+                    SixModelObject sm = Ops.findmethod(tc, slurpyType,
+                        (flags & SIG_ELEM_SLURPY_ONEARG) != 0 ? "from-slurpy-onearg" :
+                        (flags & SIG_ELEM_SLURPY_POS) != 0 ? "from-slurpy-flat" : "from-slurpy");
+                    Ops.invokeDirect(tc, sm, slurpyFromArgs, new Object[] { slurpyType, slurpy });
+                    SixModelObject bindee = Ops.result_o(tc.curFrame);
+
                     bindFail = bindOneParam(tc, gcx, cf, param, bindee, CallSiteDescriptor.ARG_OBJ,
                         noNomTypeCheck, error);
                     if (bindFail != 0)
@@ -827,7 +905,7 @@ public final class Binder {
                     }
                     else if (!suppressArityFail) {
                         if (error != null)
-                            error[0] = "Required named parameter '" +
+                            error[0] = "Required named argument '" +
                                 namedNames.at_pos_boxed(tc, 0).get_str(tc) +
                                 "' not passed";
                         return BIND_RESULT_FAIL;
@@ -857,11 +935,11 @@ public final class Binder {
                 int numExtra = namedArgsCopy.size();
                 if (numExtra == 1) {
                     for (String name : namedArgsCopy.keySet())
-                        error[0] = "Unexpected named parameter '" + name + "' passed";
+                        error[0] = "Unexpected named argument '" + name + "' passed";
                 }
                 else {
                     boolean first = true;
-                    error[0] = numExtra + " unexpected named parameters passed (";
+                    error[0] = numExtra + " unexpected named arguments passed (";
                     for (String name : namedArgsCopy.keySet()) {
                         if (!first)
                             error[0] += ", ";
@@ -905,128 +983,4 @@ public final class Binder {
         }
         return slurpy;
     }
-    
-    /* Compile time trial binding; tries to determine at compile time whether
-     * certain binds will/won't work. */
-    public static int trialBind(ThreadContext tc, RakOps.GlobalExt gcx, SixModelObject params,
-            CallSiteDescriptor csd, Object[] args) {
-        /* If there's a single capture parameter, then we're OK. (Worth
-         * handling especially as it's the common case for protos). */
-        int numParams = (int)params.elems(tc);
-        if (numParams == 1) {
-            SixModelObject param = params.at_pos_boxed(tc, 0);
-            param.get_attribute_native(tc, gcx.Parameter, "$!flags", HINT_flags);
-            int flags = (int)tc.native_i;
-            if ((flags & SIG_ELEM_IS_CAPTURE) != 0)
-                return TRIAL_BIND_OK;
-        }
-            
-        /* Walk through the signature and consider the parameters. */
-        int numPosArgs = csd.numPositionals;
-        int curPosArg = 0;
-        for (int i = 0; i < numParams; i++) {
-            /* If the parameter is anything other than a boring old
-             * positional parameter, we won't analyze it. */
-            SixModelObject param = params.at_pos_boxed(tc, i);
-            param.get_attribute_native(tc, gcx.Parameter, "$!flags", HINT_flags);
-            int flags = (int)tc.native_i;
-            if ((flags & ~(
-                    SIG_ELEM_MULTI_INVOCANT | SIG_ELEM_IS_PARCEL |
-                    SIG_ELEM_IS_COPY | SIG_ELEM_ARRAY_SIGIL |
-                    SIG_ELEM_HASH_SIGIL | SIG_ELEM_NATIVE_VALUE |
-                    SIG_ELEM_IS_OPTIONAL)) != 0)
-                return TRIAL_BIND_NOT_SURE;
-            SixModelObject namedNames = param.get_attribute_boxed(tc,
-                gcx.Parameter, "$!named_names", HINT_named_names);
-            if (namedNames != null)
-                return TRIAL_BIND_NOT_SURE;
-            SixModelObject postConstraints = param.get_attribute_boxed(tc,
-                gcx.Parameter, "$!post_constraints", HINT_post_constraints);
-            if (postConstraints != null)
-                return TRIAL_BIND_NOT_SURE;
-            SixModelObject typeCaptures = param.get_attribute_boxed(tc,
-                gcx.Parameter, "$!type_captures", HINT_type_captures);
-            if (typeCaptures != null)
-                return TRIAL_BIND_NOT_SURE;
-
-            /* Do we have an argument for this parameter? */
-            if (curPosArg >= numPosArgs) {
-                /* No; if it's not optional, fail.*/
-                if ((flags & SIG_ELEM_IS_OPTIONAL) == 0)
-                    return TRIAL_BIND_NO_WAY;
-            }
-            else {
-                /* Yes, need to consider type. */
-                int gotPrim = csd.argFlags[curPosArg];
-                if ((flags & SIG_ELEM_NATIVE_VALUE) != 0) {
-                    if (gotPrim == CallSiteDescriptor.ARG_OBJ) {
-                        /* We got an object; if we aren't sure we can unbox, we can't
-                         * be sure about the dispatch. */
-                        SixModelObject arg = (SixModelObject)args[i];
-                        StorageSpec spec = arg.st.REPR.get_storage_spec(tc, arg.st);
-                        switch (flags & SIG_ELEM_NATIVE_VALUE) {
-                            case SIG_ELEM_NATIVE_INT_VALUE:
-                                if ((spec.can_box & StorageSpec.CAN_BOX_INT) == 0)
-                                    return TRIAL_BIND_NOT_SURE;
-                                break;
-                            case SIG_ELEM_NATIVE_NUM_VALUE:
-                                if ((spec.can_box & StorageSpec.CAN_BOX_NUM) == 0)
-                                    return TRIAL_BIND_NOT_SURE;
-                                break;
-                            case SIG_ELEM_NATIVE_STR_VALUE:
-                                if ((spec.can_box & StorageSpec.CAN_BOX_STR) == 0)
-                                    return TRIAL_BIND_NOT_SURE;
-                                break;
-                            default:
-                                /* WTF... */
-                                return TRIAL_BIND_NOT_SURE;
-                        }
-                    }
-                    else {
-                        /* If it's the wrong type of native, there's no way it
-                        * can ever bind. */
-                        if (((flags & SIG_ELEM_NATIVE_INT_VALUE) != 0 && gotPrim != CallSiteDescriptor.ARG_INT) ||
-                            ((flags & SIG_ELEM_NATIVE_NUM_VALUE) != 0 && gotPrim != CallSiteDescriptor.ARG_NUM) ||
-                            ((flags & SIG_ELEM_NATIVE_STR_VALUE) != 0 && gotPrim != CallSiteDescriptor.ARG_STR))
-                            return TRIAL_BIND_NO_WAY;
-                    }
-                }
-                else {
-                    /* Work out a parameter type to consider, and see if it matches. */
-                    SixModelObject arg =
-                        gotPrim == CallSiteDescriptor.ARG_OBJ ? (SixModelObject)args[curPosArg] :
-                        gotPrim == CallSiteDescriptor.ARG_INT ? gcx.Int :
-                        gotPrim == CallSiteDescriptor.ARG_NUM ? gcx.Num :
-                                                                gcx.Str;
-                    SixModelObject nominalType = param.get_attribute_boxed(tc,
-                        gcx.Parameter, "$!nominal_type", HINT_nominal_type);
-                    if (nominalType != gcx.Mu && Ops.istype(arg, nominalType, tc) == 0) {
-                        /* If it failed because we got a junction, may auto-thread;
-                         * hand back "not sure" for now. */
-                        if (arg.st.WHAT == gcx.Junction)
-                            return TRIAL_BIND_NOT_SURE;
-                        
-                        /* It failed to, but that doesn't mean it can't work at runtime;
-                         * we perhaps want an Int, and the most we know is we have an Any,
-                         * which would include Int. However, the Int ~~ Str case can be
-                         * rejected now, as there's no way it'd ever match. Basically, we
-                         * just flip the type check around. */
-                        return Ops.istype(nominalType, arg, tc) != 0
-                            ? TRIAL_BIND_NOT_SURE
-                            : TRIAL_BIND_NO_WAY;
-                    }
-                }
-            }
-
-            /* Continue to next argument. */
-            curPosArg++;
-        }
-
-        /* If we have any left over arguments, it's a binding fail. */
-        if (curPosArg < numPosArgs)
-            return TRIAL_BIND_NO_WAY;
-
-        /* Otherwise, if we get there, all is well. */
-        return TRIAL_BIND_OK;
-    }
 }
