@@ -9,6 +9,7 @@ import org.perl6.nqp.sixmodel.*;
 import org.perl6.nqp.sixmodel.reprs.CallCaptureInstance;
 import org.perl6.nqp.sixmodel.reprs.ContextRefInstance;
 import org.perl6.nqp.sixmodel.reprs.LexoticInstance;
+import org.perl6.nqp.sixmodel.reprs.NativeRefInstance;
 import org.perl6.nqp.sixmodel.reprs.VMArrayInstance;
 
 /**
@@ -27,7 +28,6 @@ public final class RakOps {
     public static class GlobalExt {
         public SixModelObject Mu;
         public SixModelObject Any;
-        public SixModelObject Parcel;
         public SixModelObject Code;
         public SixModelObject Routine;
         public SixModelObject Signature;
@@ -38,9 +38,8 @@ public final class RakOps {
         public SixModelObject List;
         public SixModelObject ListIter;
         public SixModelObject Array;
-        public SixModelObject LoL;
         public SixModelObject Nil;
-        public SixModelObject EnumMap;
+        public SixModelObject Map;
         public SixModelObject Hash;
         public SixModelObject Junction;
         public SixModelObject Scalar;
@@ -49,6 +48,8 @@ public final class RakOps {
         public SixModelObject False;
         public SixModelObject True;
         public SixModelObject AutoThreader;
+        public SixModelObject Positional;
+        public SixModelObject PositionalBindFailover;
         public SixModelObject EMPTYARR;
         public SixModelObject EMPTYHASH;
         public RakudoJavaInterop rakudoInterop;
@@ -62,10 +63,9 @@ public final class RakOps {
     public static ContextKey<ThreadExt, GlobalExt> key = new ContextKey< >(ThreadExt.class, GlobalExt.class);
 
     /* Parameter hints for fast lookups. */
-    private static final int HINT_PARCEL_STORAGE = 0;
     private static final int HINT_CODE_DO = 0;
     private static final int HINT_CODE_SIG = 1;
-    private static final int HINT_ROUTINE_RW = 7;
+    private static final int HINT_ROUTINE_RW = 8;
     private static final int HINT_SIG_PARAMS = 0;
     private static final int HINT_SIG_RETURNS = 1;
     private static final int HINT_SIG_CODE = 4;
@@ -73,13 +73,6 @@ public final class RakOps {
     public static final int HINT_CD_RW = 1;
     public static final int HINT_CD_NAME = 2;
     public static final int HINT_CD_DEFAULT = 3;
-    private static final int HINT_LIST_items = 0;
-    private static final int HINT_LIST_flattens = 1;
-    private static final int HINT_LIST_nextiter = 2;
-    private static final int HINT_LISTITER_reified = 0;
-    private static final int HINT_LISTITER_nextiter = 1;
-    private static final int HINT_LISTITER_rest = 2;
-    private static final int HINT_LISTITER_list = 3;
     
     public static SixModelObject p6init(ThreadContext tc) {
         GlobalExt gcx = key.getGC(tc);
@@ -99,7 +92,6 @@ public final class RakOps {
         GlobalExt gcx = key.getGC(tc);
         gcx.Mu = conf.at_key_boxed(tc, "Mu");
         gcx.Any = conf.at_key_boxed(tc, "Any");
-        gcx.Parcel = conf.at_key_boxed(tc, "Parcel");
         gcx.Code = conf.at_key_boxed(tc, "Code");
         gcx.Routine = conf.at_key_boxed(tc, "Routine");
         gcx.Signature = conf.at_key_boxed(tc, "Signature");
@@ -110,9 +102,8 @@ public final class RakOps {
         gcx.List = conf.at_key_boxed(tc, "List");
         gcx.ListIter = conf.at_key_boxed(tc, "ListIter");
         gcx.Array = conf.at_key_boxed(tc, "Array");
-        gcx.LoL = conf.at_key_boxed(tc, "LoL");
         gcx.Nil = conf.at_key_boxed(tc, "Nil");
-        gcx.EnumMap = conf.at_key_boxed(tc, "EnumMap");
+        gcx.Map = conf.at_key_boxed(tc, "Map");
         gcx.Hash = conf.at_key_boxed(tc, "Hash");
         gcx.Junction = conf.at_key_boxed(tc, "Junction");
         gcx.Scalar = conf.at_key_boxed(tc, "Scalar");
@@ -144,7 +135,14 @@ public final class RakOps {
         gcx.AutoThreader = autoThreader;
         return autoThreader;
     }
-    
+
+    public static SixModelObject p6configposbindfailover(SixModelObject p, SixModelObject pbf, ThreadContext tc) {
+        GlobalExt gcx = key.getGC(tc);
+        gcx.Positional = p;
+        gcx.PositionalBindFailover = pbf;
+        return p;
+    }
+
     public static SixModelObject booleanize(int x, ThreadContext tc) {
         GlobalExt gcx = key.getGC(tc);
         return x == 0 ? gcx.False : gcx.True;
@@ -176,89 +174,7 @@ public final class RakOps {
         res.set_str(tc, value);
         return res;
     }
-    
-    public static SixModelObject p6list(SixModelObject arr, SixModelObject type, SixModelObject flattens, ThreadContext tc) {
-        GlobalExt gcx = key.getGC(tc);
-        SixModelObject list = type.st.REPR.allocate(tc, type.st);
-        if (arr != null) 
-            list.bind_attribute_boxed(tc, gcx.List, "$!nextiter", HINT_LIST_nextiter,
-                p6listiter(arr, list, tc));
-        list.bind_attribute_boxed(tc, gcx.List, "$!flattens", HINT_LIST_flattens, flattens);
-        return list;
-    }
-    
-    public static SixModelObject p6listitems(SixModelObject list, ThreadContext tc) {
-        GlobalExt gcx = key.getGC(tc);
-        SixModelObject items = list.get_attribute_boxed(tc, gcx.List, "$!items", HINT_LIST_items);
-        if (!(items instanceof VMArrayInstance)) {
-            items = gcx.EMPTYARR.clone(tc);
-            list.bind_attribute_boxed(tc, gcx.List, "$!items", HINT_LIST_items, items);
-        }
-        return items;
-    }
-    
-    public static long p6arrfindtypes(SixModelObject arr, SixModelObject types, long start, long last, ThreadContext tc) {
-        int ntypes = (int)types.elems(tc);
-        SixModelObject[] typeArr = new SixModelObject[ntypes];
-        for (int i = 0; i < ntypes; i++)
-            typeArr[i] = types.at_pos_boxed(tc, i);
 
-        long elems = arr.elems(tc);
-        if (elems < last)
-            last = elems;
-
-        long index;
-        for (index = start; index < last; index++) {
-            SixModelObject val = arr.at_pos_boxed(tc, index);
-            if (val.st.ContainerSpec == null) {
-                boolean found = false;
-                for (int typeIndex = 0; typeIndex < ntypes; typeIndex++) {
-                    if (Ops.istype(val, typeArr[typeIndex], tc) != 0) {
-                        found = true;
-                        break;
-                    }
-                }
-                if (found)
-                    break;
-            }
-        }
-
-        return index;
-    }
-    
-    public static SixModelObject p6shiftpush(SixModelObject a, SixModelObject b, long total, ThreadContext tc) {
-        long count = total;
-        long elems = b.elems(tc);
-        if (count > elems)
-            count = elems;
-
-        if (a != null && total > 0) {
-            long getPos = 0;
-            long setPos = a.elems(tc);
-            a.set_elems(tc, setPos + count);
-            while (count > 0) {
-                a.bind_pos_boxed(tc, setPos, b.at_pos_boxed(tc, getPos));
-                count--;
-                getPos++;
-                setPos++;
-            }
-        }
-        if (total > 0) {
-            GlobalExt gcx = key.getGC(tc);
-            b.splice(tc, gcx.EMPTYARR, 0, total);
-        }
-        
-        return a;
-    }
-    
-    public static SixModelObject p6listiter(SixModelObject arr, SixModelObject list, ThreadContext tc) {
-        GlobalExt gcx = key.getGC(tc);
-        SixModelObject iter = gcx.ListIter.st.REPR.allocate(tc, gcx.ListIter.st);
-        iter.bind_attribute_boxed(tc, gcx.ListIter, "$!rest", HINT_LISTITER_rest, arr);
-        iter.bind_attribute_boxed(tc, gcx.ListIter, "$!list", HINT_LISTITER_list, list);
-        return iter;
-    }
-    
     public static SixModelObject p6argvmarray(ThreadContext tc, CallSiteDescriptor csd, Object[] args) {
         SixModelObject BOOTArray = tc.gc.BOOTArray;
         SixModelObject res = BOOTArray.st.REPR.allocate(tc, BOOTArray.st);
@@ -305,10 +221,16 @@ public final class RakOps {
             .get_attribute_boxed(tc, gcx.Signature, "$!params", HINT_SIG_PARAMS);
         
         /* Run binder, and handle any errors. */
-        String[] error = new String[1];
+        Object[] error = new Object[3];
         switch (Binder.bind(tc, gcx, cf, params, csd, args, false, error)) {
             case Binder.BIND_RESULT_FAIL:
-                throw ExceptionHandling.dieInternal(tc, error[0]);
+                if (error[0] instanceof String) {
+                    throw ExceptionHandling.dieInternal(tc, (String) error[0]);
+                }
+                else {
+                    Ops.invokeDirect(tc, (SixModelObject) error[0],
+                        (CallSiteDescriptor) error[1], (Object[]) error[2]);
+                }
             case Binder.BIND_RESULT_JUNCTION:
                 /* Invoke the auto-threader. */
                 csd = csd.injectInvokee(tc, args, cf.codeRef.codeObject);
@@ -334,11 +256,17 @@ public final class RakOps {
         SixModelObject params = sig.get_attribute_boxed(tc, gcx.Signature,
             "$!params", HINT_SIG_PARAMS);
         
-        String[] error = new String[1];
+        Object[] error = new Object[3];
         switch (Binder.bind(tc, gcx, cf, params, csd, tc.flatArgs, false, error)) {
             case Binder.BIND_RESULT_FAIL:
             case Binder.BIND_RESULT_JUNCTION:
-                throw ExceptionHandling.dieInternal(tc, error[0]);
+                if (error[0] instanceof String) {
+                    throw ExceptionHandling.dieInternal(tc, (String) error[0]);
+                }
+                else {
+                    Ops.invokeDirect(tc, (SixModelObject) error[0],
+                        (CallSiteDescriptor) error[1], (Object[]) error[2]);
+                }
             default:
                 return sig;
         }        
@@ -379,56 +307,6 @@ public final class RakOps {
         }
     }
     
-    public static long p6trialbind(SixModelObject sig, SixModelObject values, SixModelObject flags, ThreadContext tc) {
-        /* Get signature and parameters. */
-        GlobalExt gcx = key.getGC(tc);
-        SixModelObject params = sig.get_attribute_boxed(tc, gcx.Signature, "$!params", HINT_SIG_PARAMS);
-
-        /* Form argument array and call site descriptor. */
-        int numArgs = (int)values.elems(tc);
-        Object[] args = new Object[numArgs];
-        byte[] argFlags = new byte[numArgs];
-        for (int i = 0; i < numArgs; i++) {
-            switch ((int)flags.at_pos_boxed(tc, i).get_int(tc)) {
-                case CallSiteDescriptor.ARG_INT:
-                    args[i] = 0;
-                    argFlags[i] = CallSiteDescriptor.ARG_INT;
-                    break;
-                case CallSiteDescriptor.ARG_NUM:
-                    args[i] = 0.0;
-                    argFlags[i] = CallSiteDescriptor.ARG_NUM;
-                    break;
-                case CallSiteDescriptor.ARG_STR:
-                    args[i] = "";
-                    argFlags[i] = CallSiteDescriptor.ARG_STR;
-                    break;
-                default:
-                    args[i] = values.at_pos_boxed(tc, i);
-                    argFlags[i] = CallSiteDescriptor.ARG_OBJ;
-                    break;
-            }
-        }
-
-        /* Do trial bind. */
-        return Binder.trialBind(tc, gcx, params, new CallSiteDescriptor(argFlags, null), args);
-    }
-    
-    public static SixModelObject p6parcel(SixModelObject array, SixModelObject fill, ThreadContext tc) {
-        GlobalExt gcx = key.getGC(tc);
-        SixModelObject parcel = gcx.Parcel.st.REPR.allocate(tc, gcx.Parcel.st);
-        parcel.bind_attribute_boxed(tc, gcx.Parcel, "$!storage", HINT_PARCEL_STORAGE, array);
-
-        if (fill != null) {
-            long elems = array.elems(tc);
-            for (long i = 0; i < elems; i++) {
-                if (array.at_pos_boxed(tc, i) == null)
-                    array.bind_pos_boxed(tc, i, fill);
-            }
-        }
-
-        return parcel;
-    }
-    
     private static final CallSiteDescriptor STORE = new CallSiteDescriptor(
         new byte[] { CallSiteDescriptor.ARG_OBJ, CallSiteDescriptor.ARG_OBJ }, null);
     private static final CallSiteDescriptor storeThrower = new CallSiteDescriptor(
@@ -458,15 +336,22 @@ public final class RakOps {
     
     public static SixModelObject p6decontrv(SixModelObject routine, SixModelObject cont, ThreadContext tc) {
         GlobalExt gcx = key.getGC(tc);
-        if (cont != null && isRWScalar(tc, gcx, cont)) {
-            routine.get_attribute_native(tc, gcx.Routine, "$!rw", HINT_ROUTINE_RW);
-            if (tc.native_i == 0) {
-                /* Recontainerize to RO. */
-                SixModelObject roCont = gcx.Scalar.st.REPR.allocate(tc, gcx.Scalar.st);
-                roCont.bind_attribute_boxed(tc, gcx.Scalar, "$!value",
-                    RakudoContainerSpec.HINT_value,
-                    cont.st.ContainerSpec.fetch(tc, cont));
-                return roCont;
+        if (cont != null) {
+            if (isRWScalar(tc, gcx, cont)) {
+                routine.get_attribute_native(tc, gcx.Routine, "$!rw", HINT_ROUTINE_RW);
+                if (tc.native_i == 0) {
+                    /* Recontainerize to RO. */
+                    SixModelObject roCont = gcx.Scalar.st.REPR.allocate(tc, gcx.Scalar.st);
+                    roCont.bind_attribute_boxed(tc, gcx.Scalar, "$!value",
+                        RakudoContainerSpec.HINT_value,
+                        cont.st.ContainerSpec.fetch(tc, cont));
+                    return roCont;
+                }
+            }
+            else if (cont instanceof NativeRefInstance) {
+                routine.get_attribute_native(tc, gcx.Routine, "$!rw", HINT_ROUTINE_RW);
+                if (tc.native_i == 0)
+                    return cont.st.ContainerSpec.fetch(tc, cont);
             }
         }
         return cont;
@@ -537,7 +422,7 @@ public final class RakOps {
     
     private static final CallSiteDescriptor rvThrower = new CallSiteDescriptor(
         new byte[] { CallSiteDescriptor.ARG_OBJ, CallSiteDescriptor.ARG_OBJ }, null);
-    public static SixModelObject p6typecheckrv(SixModelObject rv, SixModelObject routine, ThreadContext tc) {
+    public static SixModelObject p6typecheckrv(SixModelObject rv, SixModelObject routine, SixModelObject failure, ThreadContext tc) {
         GlobalExt gcx = key.getGC(tc);
         SixModelObject sig = routine.get_attribute_boxed(tc, gcx.Code, "$!signature", HINT_CODE_SIG);
         SixModelObject rtype = sig.get_attribute_boxed(tc, gcx.Signature, "$!returns", HINT_SIG_RETURNS);
@@ -548,13 +433,16 @@ public final class RakOps {
                  * an Int that can unbox into an int or similar. */
                 StorageSpec spec = rtype.st.REPR.get_storage_spec(tc, rtype.st);
                 if (spec.inlineable == 0 || Ops.istype(rtype, decontValue.st.WHAT, tc) == 0) {
-                    SixModelObject thrower = getThrower(tc, "X::TypeCheck::Return");
-                    if (thrower == null)
-                        throw ExceptionHandling.dieInternal(tc,
-                            "Type check failed for return value");
-                    else
-                        Ops.invokeDirect(tc, thrower,
-                            rvThrower, new Object[] { decontValue, rtype });
+                    if (Ops.istype(failure, decontValue.st.WHAT, tc) == 0
+                    &&  Ops.istype(failure, gcx.Nil, tc) == 0) {
+                        SixModelObject thrower = getThrower(tc, "X::TypeCheck::Return");
+                        if (thrower == null)
+                            throw ExceptionHandling.dieInternal(tc,
+                                "Type check failed for return value");
+                        else
+                            Ops.invokeDirect(tc, thrower,
+                                rvThrower, new Object[] { decontValue, rtype });
+                    }
                 }
             }
         }
