@@ -9,12 +9,24 @@ static MVMCallsite no_arg_callsite = { NULL, 0, 0, 0 };
 static MVMCallsiteEntry tc_flags[] = { MVM_CALLSITE_ARG_OBJ,
                                        MVM_CALLSITE_ARG_OBJ,
                                        MVM_CALLSITE_ARG_OBJ };
-static MVMCallsite     tc_callsite = { tc_flags, 3, 3, 0 };
+static MVMCallsite     tc_callsite = { tc_flags, 3, 3, 3, 0 };
 
 static void rakudo_scalar_fetch(MVMThreadContext *tc, MVMObject *cont, MVMRegister *res) {
     res->o = ((Rakudo_Scalar *)cont)->value;
 }
 
+static void rakudo_scalar_fetch_i(MVMThreadContext *tc, MVMObject *cont, MVMRegister *res) {
+    res->i64 = MVM_repr_get_int(tc, ((Rakudo_Scalar *)cont)->value);
+}
+
+static void rakudo_scalar_fetch_n(MVMThreadContext *tc, MVMObject *cont, MVMRegister *res) {
+    res->n64 = MVM_repr_get_num(tc, ((Rakudo_Scalar *)cont)->value);
+}
+
+static void rakudo_scalar_fetch_s(MVMThreadContext *tc, MVMObject *cont, MVMRegister *res) {
+    res->s = MVM_repr_get_str(tc, ((Rakudo_Scalar *)cont)->value);
+}
+
 MVMObject * get_nil();
 MVMObject * get_mu();
 
@@ -92,7 +104,7 @@ static void rakudo_scalar_store(MVMThreadContext *tc, MVMObject *cont, MVMObject
             /* If we get here, need to call .^type_check on the value we're
              * checking, unless it's an accepts check. */
             if (!STABLE(obj)->type_check_cache || (mode & MVM_TYPE_CHECK_CACHE_THEN_METHOD)) {
-                MVMObject *HOW = STABLE(rcd->of)->HOW;
+                MVMObject *HOW = MVM_6model_get_how_obj(tc, rcd->of);
                 MVMObject *meth = MVM_6model_find_method_cache_only(tc, HOW,
                     tc->instance->str_consts.type_check);
                 if (meth) {
@@ -117,7 +129,7 @@ static void rakudo_scalar_store(MVMThreadContext *tc, MVMObject *cont, MVMObject
 
             /* If the flag to call .accepts_type on the target value is set, do so. */
             if (mode & MVM_TYPE_CHECK_NEEDS_ACCEPTS) {
-                MVMObject *HOW = STABLE(rcd->of)->HOW;
+                MVMObject *HOW = MVM_6model_get_how_obj(tc, rcd->of);
                 MVMObject *meth = MVM_6model_find_method_cache_only(tc, HOW,
                     tc->instance->str_consts.accepts_type);
                 if (meth) {
@@ -149,6 +161,30 @@ static void rakudo_scalar_store(MVMThreadContext *tc, MVMObject *cont, MVMObject
     finish_store(tc, cont, obj);
 }
 
+static void rakudo_scalar_store_i(MVMThreadContext *tc, MVMObject *cont, MVMint64 value) {
+    MVMObject *boxed;
+    MVMROOT(tc, cont, {
+        boxed = MVM_repr_box_int(tc, MVM_hll_current(tc)->int_box_type, value);
+    });
+    rakudo_scalar_store(tc, cont, boxed);
+}
+
+static void rakudo_scalar_store_n(MVMThreadContext *tc, MVMObject *cont, MVMnum64 value) {
+    MVMObject *boxed;
+    MVMROOT(tc, cont, {
+        boxed = MVM_repr_box_num(tc, MVM_hll_current(tc)->num_box_type, value);
+    });
+    rakudo_scalar_store(tc, cont, boxed);
+}
+
+static void rakudo_scalar_store_s(MVMThreadContext *tc, MVMObject *cont, MVMString *value) {
+    MVMObject *boxed;
+    MVMROOT(tc, cont, {
+        boxed = MVM_repr_box_str(tc, MVM_hll_current(tc)->str_box_type, value);
+    });
+    rakudo_scalar_store(tc, cont, boxed);
+}
+
 static void rakudo_scalar_store_unchecked(MVMThreadContext *tc, MVMObject *cont, MVMObject *obj) {
     finish_store(tc, cont, obj);
 }
@@ -176,17 +212,29 @@ static void rakudo_scalar_spesh(MVMThreadContext *tc, MVMSTable *st, MVMSpeshGra
     }
 }
 
+static MVMint32 rakudo_scalar_can_store(MVMThreadContext *tc, MVMObject *cont) {
+    Rakudo_Scalar *rs = (Rakudo_Scalar *)cont;
+    Rakudo_ContainerDescriptor *rcd = (Rakudo_ContainerDescriptor *)rs->descriptor;
+    return rcd && IS_CONCRETE(rcd) && rcd->rw;
+}
 
 static const MVMContainerSpec rakudo_scalar_spec = {
     "rakudo_scalar",
     rakudo_scalar_fetch,
+    rakudo_scalar_fetch_i,
+    rakudo_scalar_fetch_n,
+    rakudo_scalar_fetch_s,
     rakudo_scalar_store,
+    rakudo_scalar_store_i,
+    rakudo_scalar_store_n,
+    rakudo_scalar_store_s,
     rakudo_scalar_store_unchecked,
     rakudo_scalar_spesh,
     NULL,
     NULL,
     rakudo_scalar_serialize,
     rakudo_scalar_deserialize,
+    rakudo_scalar_can_store,
     1
 };
 
