@@ -10,25 +10,29 @@
 #include <sys/time.h>
 #endif
 
-#define GET_REG(tc, idx)    (*tc->interp_reg_base)[*((MVMuint16 *)(*tc->interp_cur_op + idx))]
+#ifndef MVM_spesh_get_and_use_facts
+#define MVM_spesh_get_and_use_facts MVM_spesh_get_facts
+#endif
+
+#define GET_REG(tc, idx)    (*tc->interp_reg_base)[*((MVMuint16 *)(cur_op + idx))]
 #define REAL_BODY(tc, obj)  MVM_p6opaque_real_data(tc, OBJECT_BODY(obj))
 
 /* Dummy zero and one-arg callsite. */
-static MVMCallsite      no_arg_callsite = { NULL, 0, 0, 0 };
+static MVMCallsite      no_arg_callsite = { NULL, 0, 0, 0, 0 };
 static MVMCallsiteEntry one_arg_flags[] = { MVM_CALLSITE_ARG_OBJ };
-static MVMCallsite     one_arg_callsite = { one_arg_flags, 1, 1, 0 };
+static MVMCallsite     one_arg_callsite = { one_arg_flags, 1, 1, 1, 0 };
 static MVMCallsiteEntry one_str_flags[] = { MVM_CALLSITE_ARG_STR };
-static MVMCallsite     one_str_callsite = { one_str_flags, 1, 1, 0 };
+static MVMCallsite     one_str_callsite = { one_str_flags, 1, 1, 1, 0 };
 
 /* Assignment type check failed callsite. */
 static MVMCallsiteEntry atcf_flags[] = { MVM_CALLSITE_ARG_STR, MVM_CALLSITE_ARG_OBJ, 
                                          MVM_CALLSITE_ARG_OBJ };
-static MVMCallsite     atcf_callsite = { atcf_flags, 3, 3, 0 };
+static MVMCallsite     atcf_callsite = { atcf_flags, 3, 3, 3, 0 };
 
 /* Dispatcher vivify_for callsite. */
 static MVMCallsiteEntry disp_flags[] = { MVM_CALLSITE_ARG_OBJ, MVM_CALLSITE_ARG_OBJ, 
                                          MVM_CALLSITE_ARG_OBJ, MVM_CALLSITE_ARG_OBJ };
-static MVMCallsite     disp_callsite = { disp_flags, 4, 4, 0 };
+static MVMCallsite     disp_callsite = { disp_flags, 4, 4, 4, 0 };
 
 /* Are we initialized yet? */
 static int initialized = 0;
@@ -40,10 +44,6 @@ static MVMObject *Int                 = NULL;
 static MVMObject *Num                 = NULL;
 static MVMObject *Str                 = NULL;
 static MVMObject *Scalar              = NULL;
-static MVMObject *Parcel              = NULL;
-static MVMObject *Iterable            = NULL;
-static MVMObject *List                = NULL;
-static MVMObject *ListIter            = NULL;
 static MVMObject *True                = NULL;
 static MVMObject *False               = NULL;
 static MVMObject *ContainerDescriptor = NULL;
@@ -62,29 +62,6 @@ static MVMString *str_xnodisp    = NULL;
 static MVMString *str_xatcf      = NULL;
 static MVMString *str_cfr        = NULL;
 
-/* Parcel, as laid out as a P6opaque. */
-typedef struct {
-    MVMP6opaque  p6o;
-    MVMObject   *storage;
-} Rakudo_Parcel;
-
-/* ListIter, as laid out as a P6opaque. */
-typedef struct {
-    MVMP6opaque  p6o;
-    MVMObject   *reified;
-    MVMObject   *nextiter;
-    MVMObject   *rest;
-    MVMObject   *list;
-} Rakudo_ListIter;
-
-/* List, as laid out as a P6opaque. */
-typedef struct {
-    MVMP6opaqueBody  p6o;
-    MVMObject       *items;
-    MVMObject       *flattens;
-    MVMObject       *nextiter;
-} Rakudo_List;
-
 /* Expose Nil and Mu for containers. */
 MVMObject * get_nil() { return Nil; }
 MVMObject * get_mu() { return Mu; }
@@ -114,7 +91,7 @@ void Rakudo_assign_typecheck_failed(MVMThreadContext *tc, MVMObject *cont, MVMOb
 }
 
 /* Initializes the Perl 6 extension ops. */
-static void p6init(MVMThreadContext *tc) {
+static void p6init(MVMThreadContext *tc, MVMuint8 *cur_op) {
     if (!initialized) {
         Rakudo_containers_setup(tc);
     }
@@ -129,7 +106,7 @@ static void p6init(MVMThreadContext *tc) {
 static MVMuint8 s_p6settypes[] = {
     MVM_operand_obj | MVM_operand_read_reg
 };
-static void p6settypes(MVMThreadContext *tc) {
+static void p6settypes(MVMThreadContext *tc, MVMuint8 *cur_op) {
     MVMObject *conf = GET_REG(tc, 0).o;
     MVMROOT(tc, conf, {
         get_type(tc, conf, "Mu", Mu);
@@ -138,10 +115,6 @@ static void p6settypes(MVMThreadContext *tc) {
         get_type(tc, conf, "Num", Num);
         get_type(tc, conf, "Str", Str);
         get_type(tc, conf, "Scalar", Scalar);
-        get_type(tc, conf, "Parcel", Parcel);
-        get_type(tc, conf, "Iterable", Iterable);
-        get_type(tc, conf, "List", List);
-        get_type(tc, conf, "ListIter", ListIter);
         get_type(tc, conf, "True", True);
         get_type(tc, conf, "False", False);
         get_type(tc, conf, "ContainerDescriptor", ContainerDescriptor);
@@ -183,138 +156,77 @@ static void p6settypes(MVMThreadContext *tc) {
 }
 
 /* Boxing to Perl 6 types. */
+static void discover_create(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshIns *ins, MVMObject *type) {
+    MVMSpeshFacts *tfacts = MVM_spesh_get_facts(tc, g, ins->operands[0]);
+    tfacts->flags |= MVM_SPESH_FACT_CONCRETE | MVM_SPESH_FACT_KNOWN_TYPE | MVM_SPESH_FACT_DECONTED;
+    tfacts->type   = type;
+}
+
+static void p6box_i_discover(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshIns *ins) {
+    discover_create(tc, g, ins, Int);
+#ifdef MVM_SPESH_FACT_KNOWN_BOX_SRC
+    MVM_spesh_get_facts(tc, g, ins->operands[0])->flags |= MVM_SPESH_FACT_KNOWN_BOX_SRC;
+#endif
+}
+static void p6box_n_discover(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshIns *ins) {
+    discover_create(tc, g, ins, Num);
+#ifdef MVM_SPESH_FACT_KNOWN_BOX_SRC
+    MVM_spesh_get_facts(tc, g, ins->operands[0])->flags |= MVM_SPESH_FACT_KNOWN_BOX_SRC;
+#endif
+}
+static void p6box_s_discover(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshIns *ins) {
+    discover_create(tc, g, ins, Str);
+#ifdef MVM_SPESH_FACT_KNOWN_BOX_SRC
+    MVM_spesh_get_facts(tc, g, ins->operands[0])->flags |= MVM_SPESH_FACT_KNOWN_BOX_SRC;
+#endif
+}
+
 static MVMuint8 s_p6box_i[] = {
     MVM_operand_obj | MVM_operand_write_reg,
     MVM_operand_int64 | MVM_operand_read_reg,
 };
-static void p6box_i(MVMThreadContext *tc) {
+static void p6box_i(MVMThreadContext *tc, MVMuint8 *cur_op) {
      GET_REG(tc, 0).o = MVM_repr_box_int(tc, Int, GET_REG(tc, 2).i64);
 }
 static MVMuint8 s_p6box_n[] = {
     MVM_operand_obj | MVM_operand_write_reg,
     MVM_operand_num64 | MVM_operand_read_reg,
 };
-static void p6box_n(MVMThreadContext *tc) {
+static void p6box_n(MVMThreadContext *tc, MVMuint8 *cur_op) {
      GET_REG(tc, 0).o = MVM_repr_box_num(tc, Num, GET_REG(tc, 2).n64);
 }
 static MVMuint8 s_p6box_s[] = {
     MVM_operand_obj | MVM_operand_write_reg,
     MVM_operand_str | MVM_operand_read_reg,
 };
-static void p6box_s(MVMThreadContext *tc) {
+static void p6box_s(MVMThreadContext *tc, MVMuint8 *cur_op) {
      GET_REG(tc, 0).o = MVM_repr_box_str(tc, Str, GET_REG(tc, 2).s);
 }
 
-static MVMuint8 s_p6parcel[] = {
-    MVM_operand_obj | MVM_operand_write_reg,
-    MVM_operand_obj | MVM_operand_read_reg,
-    MVM_operand_obj | MVM_operand_read_reg
-};
-static void p6parcel(MVMThreadContext *tc) {
-    MVMObject *parcel = MVM_repr_alloc_init(tc, Parcel);
-    MVMObject *vmarr  = GET_REG(tc, 2).o;
-    MVMObject *fill   = GET_REG(tc, 4).o;
-    MVM_ASSIGN_REF(tc, &(parcel->header), ((Rakudo_Parcel *)parcel)->storage, vmarr);
-
-    if (!MVM_is_null(tc, fill)) {
-        MVMint64 elems = MVM_repr_elems(tc, vmarr);
-        MVMint64 i;
-        for (i = 0; i < elems; i++)
-            if (MVM_is_null(tc, MVM_repr_at_pos_o(tc, vmarr, i)))
-                MVM_repr_bind_pos_o(tc, vmarr, i, fill);
-    }
-
-    GET_REG(tc, 0).o = parcel;
-}
-
-/* Produces a lazy Perl 6 list of the specified type with the given items. */
-static MVMObject * make_listiter(MVMThreadContext *tc, MVMObject *items, MVMObject *list) {
-    MVMObject *result;
-    MVMROOT(tc, items, {
-    MVMROOT(tc, list, {
-        result = MVM_repr_alloc_init(tc, ListIter);
-        MVM_ASSIGN_REF(tc, &(result->header), ((Rakudo_ListIter *)result)->rest, items);
-        MVM_ASSIGN_REF(tc, &(result->header), ((Rakudo_ListIter *)result)->list, list);
-    });
-    });
-    return result;
-}
-static MVMuint8 s_p6list[] = {
-    MVM_operand_obj | MVM_operand_write_reg,
-    MVM_operand_obj | MVM_operand_read_reg,
-    MVM_operand_obj | MVM_operand_read_reg,
-    MVM_operand_obj | MVM_operand_read_reg
-};
-static void p6list(MVMThreadContext *tc) {
-     MVMObject *list = MVM_repr_alloc_init(tc, GET_REG(tc, 4).o);
-     if (MVM_6model_istype_cache_only(tc, list, List)) {
-        MVMROOT(tc, list, {
-            MVMObject *items = GET_REG(tc, 2).o;
-            if (!MVM_is_null(tc, items)) {
-                MVMObject *iter = make_listiter(tc, items, list);
-                MVM_ASSIGN_REF(tc, &(list->header), ((Rakudo_List *)REAL_BODY(tc, list))->nextiter, iter);
-            }
-            MVM_ASSIGN_REF(tc, &(list->header), ((Rakudo_List *)REAL_BODY(tc, list))->flattens, GET_REG(tc, 6).o);
-        });
-        GET_REG(tc, 0).o = list;
-    }
-    else {
-        MVM_exception_throw_adhoc(tc, "p6list may only be used on a List");
-    }
-}
-
-static MVMuint8 s_p6listiter[] = {
-    MVM_operand_obj | MVM_operand_write_reg,
-    MVM_operand_obj | MVM_operand_read_reg,
-    MVM_operand_obj | MVM_operand_read_reg
-};
-static void p6listiter(MVMThreadContext *tc) {
-    MVMObject  *arr = GET_REG(tc, 2).o;
-    MVMObject *list = GET_REG(tc, 4).o;
-    GET_REG(tc, 0).o = make_listiter(tc, arr, list);
-}
-
-/* Returns the $!items attribute of a List, vivifying it to a
- * low-level array if it isn't one already. */
-static MVMuint8 s_p6listitems[] = {
-    MVM_operand_obj | MVM_operand_write_reg,
-    MVM_operand_obj | MVM_operand_read_reg
-};
-static void p6listitems(MVMThreadContext *tc) {
-     MVMObject *list = GET_REG(tc, 2).o;
-     if (MVM_6model_istype_cache_only(tc, list, List)) {
-        MVMObject *items = ((Rakudo_List *)REAL_BODY(tc, list))->items;
-        if (MVM_is_null(tc, items) || !IS_CONCRETE(items) || REPR(items)->ID != MVM_REPR_ID_MVMArray) {
-            MVMROOT(tc, list, {
-                items = MVM_repr_alloc_init(tc, tc->instance->boot_types.BOOTArray);
-                MVM_ASSIGN_REF(tc, &(list->header), ((Rakudo_List *)REAL_BODY(tc, list))->items, items);
-            });
-        }
-        GET_REG(tc, 0).o = items;
-     }
-     else {
-        MVM_exception_throw_adhoc(tc, "p6listitems may only be used on a List");
-     }
-}
-
 /* Turns zero to False and non-zero to True. */
 static MVMuint8 s_p6bool[] = {
     MVM_operand_obj | MVM_operand_write_reg,
     MVM_operand_int64 | MVM_operand_read_reg,
 };
-static void p6bool(MVMThreadContext *tc) {
+static void p6bool(MVMThreadContext *tc, MVMuint8 *cur_op) {
      GET_REG(tc, 0).o = GET_REG(tc, 2).i64 ? True : False;
 }
+static void p6bool_discover(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshIns *ins) {
+    discover_create(tc, g, ins, STABLE(True)->WHAT);
+#ifdef MVM_SPESH_FACT_KNOWN_BOX_SRC
+    MVM_spesh_get_facts(tc, g, ins->operands[0])->flags |= MVM_SPESH_FACT_KNOWN_BOX_SRC;
+#endif
+}
 
 /* Creates a Scalar from the specified descriptor. */
 static MVMuint8 s_p6scalarfromdesc[] = {
     MVM_operand_obj | MVM_operand_write_reg,
     MVM_operand_obj | MVM_operand_read_reg,
 };
-static void p6scalarfromdesc(MVMThreadContext *tc) {
+static void p6scalarfromdesc(MVMThreadContext *tc, MVMuint8 *cur_op) {
     MVMObject *new_scalar = MVM_repr_alloc_init(tc, Scalar);
     MVMObject *descriptor = GET_REG(tc, 2).o;
-    if (MVM_is_null(tc, descriptor)) {
+    if (MVM_is_null(tc, descriptor) || !IS_CONCRETE(descriptor)) {
         descriptor = default_cont_desc;
     }
     MVM_ASSIGN_REF(tc, &(new_scalar->header), ((Rakudo_Scalar *)new_scalar)->descriptor, descriptor);
@@ -322,12 +234,17 @@ static void p6scalarfromdesc(MVMThreadContext *tc) {
         ((Rakudo_ContainerDescriptor *)descriptor)->the_default);
     GET_REG(tc, 0).o = new_scalar;
 }
+static void p6scalarfromdesc_discover(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshIns *ins) {
+    MVMSpeshFacts *tfacts = MVM_spesh_get_and_use_facts(tc, g, ins->operands[0]);
+    tfacts->flags |= MVM_SPESH_FACT_CONCRETE | MVM_SPESH_FACT_KNOWN_TYPE;
+    tfacts->type   = Scalar;
+}
 
 static MVMuint8 s_p6recont_ro[] = {
     MVM_operand_obj | MVM_operand_write_reg,
     MVM_operand_obj | MVM_operand_read_reg,
 };
-static void p6recont_ro(MVMThreadContext *tc) {
+static void p6recont_ro(MVMThreadContext *tc, MVMuint8 *cur_op) {
     MVMObject *check = GET_REG(tc, 2).o;
     if (STABLE(check)->container_spec == Rakudo_containers_get_scalar()) {
         MVMObject *desc = ((Rakudo_Scalar *)check)->descriptor;
@@ -351,7 +268,7 @@ static MVMuint8 s_p6var[] = {
     MVM_operand_obj | MVM_operand_write_reg,
     MVM_operand_obj | MVM_operand_read_reg,
 };
-static void p6var(MVMThreadContext *tc) {
+static void p6var(MVMThreadContext *tc, MVMuint8 *cur_op) {
      MVMObject *wrappee = GET_REG(tc, 2).o;
      if (STABLE(wrappee)->container_spec) {
         MVMROOT(tc, wrappee, {
@@ -369,7 +286,7 @@ static MVMuint8 s_p6reprname[] = {
     MVM_operand_obj | MVM_operand_write_reg,
     MVM_operand_obj | MVM_operand_read_reg,
 };
-static void p6reprname(MVMThreadContext *tc) {
+static void p6reprname(MVMThreadContext *tc, MVMuint8 *cur_op) {
     MVMObject *obj = GET_REG(tc, 2).o;
     MVMROOT(tc, obj, {
         MVMObject *name = MVM_repr_alloc_init(tc, tc->instance->boot_types.BOOTStr);
@@ -381,51 +298,73 @@ static void p6reprname(MVMThreadContext *tc) {
         });
     });
 }
+static void p6reprname_discover(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshIns *ins) {
+    discover_create(tc, g, ins, tc->instance->boot_types.BOOTStr);
+}
 
 /* Decontainerizes the return value of a routine as needed. */
 static MVMuint8 s_p6decontrv[] = {
     MVM_operand_obj | MVM_operand_write_reg,
     MVM_operand_obj | MVM_operand_read_reg,
 };
-static void p6decontrv(MVMThreadContext *tc) {
-    MVMObject *retval = GET_REG(tc, 2).o;
+static MVMObject *Iterable = NULL;
+static void p6decontrv(MVMThreadContext *tc, MVMuint8 *cur_op) {
+    MVMObject *retval;
+    if (!Iterable)
+        Iterable = MVM_frame_find_lexical_by_name(tc,
+            MVM_string_ascii_decode_nt(tc, tc->instance->VMString, "Iterable"),
+            MVM_reg_obj)->o;
+    retval = GET_REG(tc, 2).o;
     if (MVM_is_null(tc, retval)) {
        retval = Mu;
     }
-    else if (IS_CONCRETE(retval) && STABLE(retval)->container_spec == Rakudo_containers_get_scalar()) {
-        Rakudo_ContainerDescriptor *cd = (Rakudo_ContainerDescriptor *)
-            ((Rakudo_Scalar *)retval)->descriptor;
-        if (!MVM_is_null(tc, (MVMObject *)cd) && cd->rw) {
-            MVMObject *value = ((Rakudo_Scalar *)retval)->value;
-            if (MVM_6model_istype_cache_only(tc, value, Iterable)
-                || MVM_6model_istype_cache_only(tc, value, Parcel)) {
-                MVMROOT(tc, value, {
-                    MVMObject *cont = MVM_repr_alloc_init(tc, Scalar);
-                    MVM_ASSIGN_REF(tc, &(cont->header), ((Rakudo_Scalar *)cont)->value,
-                        value);
-                    retval = cont;
-                });
-            }
-            else {
-                retval = value;
+    else if (IS_CONCRETE(retval)) {
+        const MVMContainerSpec *spec = STABLE(retval)->container_spec;
+        if (spec == Rakudo_containers_get_scalar()) {
+            Rakudo_ContainerDescriptor *cd = (Rakudo_ContainerDescriptor *)
+                ((Rakudo_Scalar *)retval)->descriptor;
+            if (!MVM_is_null(tc, (MVMObject *)cd) && cd->rw) {
+                MVMObject *value = ((Rakudo_Scalar *)retval)->value;
+                if (MVM_6model_istype_cache_only(tc, value, Iterable)) {
+                    MVMROOT(tc, value, {
+                        MVMObject *cont = MVM_repr_alloc_init(tc, Scalar);
+                        MVM_ASSIGN_REF(tc, &(cont->header), ((Rakudo_Scalar *)cont)->value,
+                            value);
+                        retval = cont;
+                    });
+                }
+                else {
+                    retval = value;
+                }
             }
         }
+        else if (spec && spec->fetch_never_invokes) {
+            MVMRegister res;
+            spec->fetch(tc, retval, &res);
+            retval = res.o;
+        }
     }
     GET_REG(tc, 0).o = retval;
 }
+static void p6decontrv_spesh(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshBB *bb, MVMSpeshIns *ins) {
+    /* If it's already deconted, can just become a set. */
+    MVMSpeshFacts *obj_facts = MVM_spesh_get_and_use_facts(tc, g, ins->operands[1]);
+    if (obj_facts->flags & (MVM_SPESH_FACT_DECONTED | MVM_SPESH_FACT_TYPEOBJ))
+        ins->info = MVM_op_get_op(MVM_OP_set);
+}
 
 static MVMuint8 s_p6routinereturn[] = {
     MVM_operand_obj | MVM_operand_write_reg,
     MVM_operand_obj | MVM_operand_read_reg,
 };
-static void p6routinereturn(MVMThreadContext *tc) {
-    MVMObject *ret = MVM_frame_find_lexical_by_name_rel_caller(tc, str_return,
-        tc->cur_frame)->o;
+static void p6routinereturn(MVMThreadContext *tc, MVMuint8 *cur_op) {
+    MVMRegister *reg = MVM_frame_find_lexical_by_name_rel_caller(tc, str_return,
+                                                                 tc->cur_frame);
+    MVMObject   *ret = (reg ? reg->o : NULL);
     if (!MVM_is_null(tc, ret) && IS_CONCRETE(ret) && REPR(ret)->ID == MVM_REPR_ID_Lexotic) {
         MVM_args_setup_thunk(tc, NULL, MVM_RETURN_VOID, &one_arg_callsite);
         tc->cur_frame->args[0].o = GET_REG(tc, 2).o;
         STABLE(ret)->invoke(tc, ret, &one_arg_callsite, tc->cur_frame->args);
-        *(tc->interp_cur_op) -= 4; /* Oh my, what a hack... */
     }
     else {
         MVMObject *thrower = get_thrower(tc, str_cfr);
@@ -443,7 +382,7 @@ static MVMuint8 s_p6capturelex[] = {
     MVM_operand_obj | MVM_operand_write_reg,
     MVM_operand_obj | MVM_operand_read_reg,
 };
-static void p6capturelex(MVMThreadContext *tc) {
+static void p6capturelex(MVMThreadContext *tc, MVMuint8 *cur_op) {
     MVMObject *p6_code_obj = GET_REG(tc, 2).o;
     MVMInvocationSpec *is = STABLE(p6_code_obj)->invocation_spec;
     MVMObject *vm_code_obj;
@@ -464,7 +403,7 @@ static MVMuint8 s_p6capturelexwhere[] = {
     MVM_operand_obj | MVM_operand_write_reg,
     MVM_operand_obj | MVM_operand_read_reg,
 };
-static void p6capturelexwhere(MVMThreadContext *tc) {
+static void p6capturelexwhere(MVMThreadContext *tc, MVMuint8 *cur_op) {
     MVMObject *p6_code_obj = GET_REG(tc, 2).o;
     MVMObject *vm_code_obj = MVM_frame_find_invokee(tc, p6_code_obj, NULL);
     if (REPR(vm_code_obj)->ID == MVM_REPR_ID_MVMCode) {
@@ -490,7 +429,7 @@ static MVMuint8 s_p6getouterctx[] = {
     MVM_operand_obj | MVM_operand_write_reg,
     MVM_operand_obj | MVM_operand_read_reg
 };
-static void p6getouterctx(MVMThreadContext *tc) {
+static void p6getouterctx(MVMThreadContext *tc, MVMuint8 *cur_op) {
     MVMObject *p6_code_obj = GET_REG(tc, 2).o;
     MVMObject *vm_code_obj = MVM_frame_find_invokee(tc, p6_code_obj, NULL);
     MVMFrame  *outer       = ((MVMCode *)vm_code_obj)->body.outer;
@@ -505,7 +444,7 @@ static MVMuint8 s_p6captureouters[] = {
     MVM_operand_obj | MVM_operand_read_reg,
     MVM_operand_obj | MVM_operand_read_reg
 };
-static void p6captureouters(MVMThreadContext *tc) {
+static void p6captureouters(MVMThreadContext *tc, MVMuint8 *cur_op) {
     MVMObject *todo  = GET_REG(tc, 0).o;
     MVMObject *tgt   = GET_REG(tc, 2).o;
     MVMint64   elems = MVM_repr_elems(tc, todo);
@@ -534,7 +473,7 @@ static void p6captureouters(MVMThreadContext *tc) {
 static MVMuint8 s_p6stateinit[] = {
     MVM_operand_int64 | MVM_operand_write_reg
 };
-static void p6stateinit(MVMThreadContext *tc) {
+static void p6stateinit(MVMThreadContext *tc, MVMuint8 *cur_op) {
     GET_REG(tc, 0).i64 = tc->cur_frame->flags & MVM_FRAME_FLAG_STATE_INIT ? 1 : 0;
 }
 
@@ -545,7 +484,7 @@ static MVMuint8 s_p6setfirstflag[] = {
     MVM_operand_obj | MVM_operand_write_reg,
     MVM_operand_obj | MVM_operand_read_reg
 };
-static void p6setfirstflag(MVMThreadContext *tc) {
+static void p6setfirstflag(MVMThreadContext *tc, MVMuint8 *cur_op) {
     MVMObject *code_obj = GET_REG(tc, 2).o;
     MVMObject *vm_code  = MVM_frame_find_invokee(tc, code_obj, NULL);
     vm_code->header.flags |= RAKUDO_FIRST_FLAG;
@@ -554,7 +493,7 @@ static void p6setfirstflag(MVMThreadContext *tc) {
 static MVMuint8 s_p6takefirstflag[] = {
     MVM_operand_int64 | MVM_operand_write_reg
 };
-static void p6takefirstflag(MVMThreadContext *tc) {
+static void p6takefirstflag(MVMThreadContext *tc, MVMuint8 *cur_op) {
     MVMObject *vm_code = tc->cur_frame->code_ref;
     if (vm_code->header.flags & RAKUDO_FIRST_FLAG) {
         vm_code->header.flags ^= RAKUDO_FIRST_FLAG;
@@ -570,7 +509,7 @@ static void p6takefirstflag(MVMThreadContext *tc) {
 static MVMuint8 s_p6setpre[] = {
     MVM_operand_obj | MVM_operand_write_reg
 };
-static void p6setpre(MVMThreadContext *tc) {
+static void p6setpre(MVMThreadContext *tc, MVMuint8 *cur_op) {
     tc->cur_frame->flags |= RAKUDO_FRAME_PRE_FLAG;
     GET_REG(tc, 0).o = NULL;
 }
@@ -578,7 +517,7 @@ static void p6setpre(MVMThreadContext *tc) {
 static MVMuint8 s_p6clearpre[] = {
     MVM_operand_obj | MVM_operand_write_reg
 };
-static void p6clearpre(MVMThreadContext *tc) {
+static void p6clearpre(MVMThreadContext *tc, MVMuint8 *cur_op) {
     if (tc->cur_frame->flags & RAKUDO_FRAME_PRE_FLAG)
         tc->cur_frame->flags ^= RAKUDO_FRAME_PRE_FLAG;
     GET_REG(tc, 0).o = NULL;
@@ -587,9 +526,10 @@ static void p6clearpre(MVMThreadContext *tc) {
 static MVMuint8 s_p6inpre[] = {
     MVM_operand_int64 | MVM_operand_write_reg
 };
-static void p6inpre(MVMThreadContext *tc) {
-    if (tc->cur_frame->flags & RAKUDO_FRAME_PRE_FLAG) {
-        tc->cur_frame->flags ^= RAKUDO_FRAME_PRE_FLAG;
+static void p6inpre(MVMThreadContext *tc, MVMuint8 *cur_op) {
+    MVMFrame *test_frame = tc->cur_frame->caller;
+    if (test_frame && test_frame->flags & RAKUDO_FRAME_PRE_FLAG) {
+        test_frame->flags ^= RAKUDO_FRAME_PRE_FLAG;
         GET_REG(tc, 0).i64 = 1;
     }
     else {
@@ -606,7 +546,7 @@ void store_dispatcher(MVMThreadContext *tc, void *sr_data) {
     srd[0]->o = srd[1]->o;
     free(srd);
 }
-static void p6finddispatcher(MVMThreadContext *tc) {
+static void p6finddispatcher(MVMThreadContext *tc, MVMuint8 *cur_op) {
     MVMFrame  *ctx = tc->cur_frame;
     while (ctx) {
         /* Do we have a dispatcher here? */
@@ -625,7 +565,7 @@ static void p6finddispatcher(MVMThreadContext *tc) {
                         ((MVMContext *)ctx_ref)->body.context = MVM_frame_inc_ref(tc, ctx);
                     });
                     capture = MVM_args_use_capture(tc, ctx);
-                    p6sub = ((MVMCode *)ctx->code_ref)->body.code_object;
+                    p6sub = MVM_frame_get_code_object(tc, (MVMCode *)ctx->code_ref);
 
                     /* Lookup method, invoke it, and set up callback to ensure it
                      * is also stored in the lexical. */
@@ -645,7 +585,6 @@ static void p6finddispatcher(MVMThreadContext *tc) {
                     tc->cur_frame->args[2].o = ctx_ref;
                     tc->cur_frame->args[3].o = capture;
                     STABLE(meth)->invoke(tc, meth, &disp_callsite, tc->cur_frame->args);
-                    *(tc->interp_cur_op) -= 4; /* Coutneract addition of ext-op size. */
                     return;
                 }
                 else {
@@ -668,7 +607,6 @@ static void p6finddispatcher(MVMThreadContext *tc) {
             MVM_args_setup_thunk(tc, NULL, MVM_RETURN_VOID, &one_str_callsite);
             tc->cur_frame->args[0].s = usage;
             STABLE(thrower)->invoke(tc, thrower, &one_str_callsite, tc->cur_frame->args);
-            *(tc->interp_cur_op) -= 4; /* Coutneract addition of ext-op size. */
         }
         else {
             MVM_exception_throw_adhoc(tc,
@@ -682,7 +620,7 @@ static MVMuint8 s_p6argsfordispatcher[] = {
     MVM_operand_obj | MVM_operand_write_reg,
     MVM_operand_obj | MVM_operand_read_reg
 };
-static void p6argsfordispatcher(MVMThreadContext *tc) {
+static void p6argsfordispatcher(MVMThreadContext *tc, MVMuint8 *cur_op) {
     MVMFrame  *ctx = tc->cur_frame;
     while (ctx) {
         /* Do we have the dispatcher we're looking for? */
@@ -703,88 +641,11 @@ static void p6argsfordispatcher(MVMThreadContext *tc) {
     MVM_exception_throw_adhoc(tc, "Could not find arguments for dispatcher");
 }
 
-static MVMuint8 s_p6shiftpush[] = {
-    MVM_operand_obj   | MVM_operand_write_reg,
-    MVM_operand_obj   | MVM_operand_read_reg,
-    MVM_operand_obj   | MVM_operand_read_reg,
-    MVM_operand_int64 | MVM_operand_read_reg
-};
-static void p6shiftpush(MVMThreadContext *tc) {
-    MVMObject   *a = GET_REG(tc, 2).o;
-    MVMObject   *b = GET_REG(tc, 4).o;
-    MVMint64 count = GET_REG(tc, 6).i64;
-    MVMint64 total = count;
-    MVMint64 elems = MVM_repr_elems(tc, b);
-    if (count > elems)
-        count = elems;
-
-    if (!MVM_is_null(tc, a) && total > 0) {
-        MVMint64 getPos = 0;
-        MVMint64 setPos = MVM_repr_elems(tc, a);
-        REPR(a)->pos_funcs.set_elems(tc, STABLE(a), a, OBJECT_BODY(a), setPos + count);
-        while (count > 0) {
-            MVM_repr_bind_pos_o(tc, a, setPos, MVM_repr_at_pos_o(tc, b, getPos));
-            count--;
-            getPos++;
-            setPos++;
-        }
-    }
-    if (total > 0) {
-        MVMROOT(tc, a, {
-        MVMROOT(tc, b, {
-            MVMObject *copy = MVM_repr_alloc_init(tc, tc->instance->boot_types.BOOTArray);
-            REPR(b)->pos_funcs.splice(tc, STABLE(b), b, OBJECT_BODY(b),
-                copy, 0, total);
-        });
-        });
-    }
-
-    GET_REG(tc, 0).o = a;
-}
-
-static MVMuint8 s_p6arrfindtypes[] = {
-    MVM_operand_int64 | MVM_operand_write_reg,
-    MVM_operand_obj   | MVM_operand_read_reg,
-    MVM_operand_obj   | MVM_operand_read_reg,
-    MVM_operand_int64 | MVM_operand_read_reg,
-    MVM_operand_int64 | MVM_operand_read_reg
-};
-static void p6arrfindtypes(MVMThreadContext *tc) {
-    MVMObject *arr    = GET_REG(tc, 2).o;
-    MVMObject *types  = GET_REG(tc, 4).o;
-    MVMint64   start  = GET_REG(tc, 6).i64;
-    MVMint64   last   = GET_REG(tc, 8).i64;
-    MVMint64   elems  = MVM_repr_elems(tc, arr);
-    MVMint64   ntypes = MVM_repr_elems(tc, types);
-    MVMint64   index, type_index;
-
-    if (elems < last)
-        last = elems;
-
-    for (index = start; index < last; index++) {
-        MVMObject *val = MVM_repr_at_pos_o(tc, arr, index);
-        if (!MVM_is_null(tc, val) && !STABLE(val)->container_spec) {
-            MVMint64 found = 0;
-            for (type_index = 0; type_index < ntypes; type_index++) {
-                MVMObject *type = MVM_repr_at_pos_o(tc, types, type_index);
-                if (MVM_6model_istype_cache_only(tc, val, type)) {
-                    found = 1;
-                    break;
-                }
-            }
-            if (found)
-                break;
-        }
-    }
-
-    GET_REG(tc, 0).i64 = index;
-}
-
 static MVMuint8 s_p6decodelocaltime[] = {
     MVM_operand_obj   | MVM_operand_write_reg,
     MVM_operand_int64 | MVM_operand_read_reg
 };
-static void p6decodelocaltime(MVMThreadContext *tc) {
+static void p6decodelocaltime(MVMThreadContext *tc, MVMuint8 *cur_op) {
     MVMObject *result = MVM_repr_alloc_init(tc, tc->instance->boot_types.BOOTIntArray);
     const time_t t = (time_t)GET_REG(tc, 2).i64;
     struct tm tm;
@@ -814,7 +675,7 @@ static MVMuint8 s_p6staticouter[] = {
     MVM_operand_obj | MVM_operand_write_reg,
     MVM_operand_obj | MVM_operand_read_reg
 };
-static void p6staticouter(MVMThreadContext *tc) {
+static void p6staticouter(MVMThreadContext *tc, MVMuint8 *cur_op) {
     MVMObject *code = GET_REG(tc, 2).o;
     if (!MVM_is_null(tc, code) && IS_CONCRETE(code) && REPR(code)->ID == MVM_REPR_ID_MVMCode) {
         MVMStaticFrame *sf = ((MVMCode *)code)->body.sf;
@@ -833,9 +694,9 @@ static MVMuint8 s_p6invokeunder[] = {
     MVM_operand_obj | MVM_operand_read_reg
 };
 static void return_from_fake(MVMThreadContext *tc, void *sr_data) {
-    MVM_frame_try_return(tc);
+    MVM_frame_try_return_no_exit_handlers(tc);
 }
-static void p6invokeunder(MVMThreadContext *tc) {
+static void p6invokeunder(MVMThreadContext *tc, MVMuint8 *cur_op) {
     MVMRegister *res  = &GET_REG(tc, 0);
     MVMObject   *fake = GET_REG(tc, 2).o;
     MVMObject   *code = GET_REG(tc, 4).o;
@@ -856,42 +717,35 @@ static void p6invokeunder(MVMThreadContext *tc) {
     MVM_args_setup_thunk(tc, res, MVM_RETURN_OBJ, &no_arg_callsite);
     tc->cur_frame->special_return = return_from_fake;
     STABLE(code)->invoke(tc, code, &no_arg_callsite, tc->cur_frame->args);
-    *(tc->interp_cur_op) -= 6;
 }
 
 /* Registers the extops with MoarVM. */
 MVM_DLL_EXPORT void Rakudo_ops_init(MVMThreadContext *tc) {
-    MVM_ext_register_extop(tc, "p6init",  p6init, 0, NULL);
-    MVM_ext_register_extop(tc, "p6box_i",  p6box_i, 2, s_p6box_i);
-    MVM_ext_register_extop(tc, "p6box_n",  p6box_n, 2, s_p6box_n);
-    MVM_ext_register_extop(tc, "p6box_s",  p6box_s, 2, s_p6box_s);
-    MVM_ext_register_extop(tc, "p6parcel",  p6parcel, 3, s_p6parcel);
-    MVM_ext_register_extop(tc, "p6listiter",  p6listiter, 3, s_p6listiter);
-    MVM_ext_register_extop(tc, "p6list",  p6list, 4, s_p6list);
-    MVM_ext_register_extop(tc, "p6listitems",  p6listitems, 2, s_p6listitems);
-    MVM_ext_register_extop(tc, "p6settypes",  p6settypes, 1, s_p6settypes);
-    MVM_ext_register_extop(tc, "p6bool",  p6bool, 2, s_p6bool);
-    MVM_ext_register_extop(tc, "p6scalarfromdesc",  p6scalarfromdesc, 2, s_p6scalarfromdesc);
-    MVM_ext_register_extop(tc, "p6recont_ro",  p6recont_ro, 2, s_p6recont_ro);
-    MVM_ext_register_extop(tc, "p6var",  p6var, 2, s_p6var);
-    MVM_ext_register_extop(tc, "p6reprname",  p6reprname, 2, s_p6reprname);
-    MVM_ext_register_extop(tc, "p6decontrv",  p6decontrv, 2, s_p6decontrv);
-    MVM_ext_register_extop(tc, "p6routinereturn",  p6routinereturn, 2, s_p6routinereturn);
-    MVM_ext_register_extop(tc, "p6capturelex",  p6capturelex, 2, s_p6capturelex);
-    MVM_ext_register_extop(tc, "p6capturelexwhere",  p6capturelexwhere, 2, s_p6capturelexwhere);
-    MVM_ext_register_extop(tc, "p6getouterctx", p6getouterctx, 2, s_p6getouterctx);
-    MVM_ext_register_extop(tc, "p6captureouters", p6captureouters, 2, s_p6captureouters);
-    MVM_ext_register_extop(tc, "p6stateinit", p6stateinit, 1, s_p6stateinit);
-    MVM_ext_register_extop(tc, "p6setfirstflag", p6setfirstflag, 2, s_p6setfirstflag);
-    MVM_ext_register_extop(tc, "p6takefirstflag", p6takefirstflag, 1, s_p6takefirstflag);
-    MVM_ext_register_extop(tc, "p6setpre", p6setpre, 1, s_p6setpre);
-    MVM_ext_register_extop(tc, "p6clearpre", p6clearpre, 1, s_p6clearpre);
-    MVM_ext_register_extop(tc, "p6inpre", p6inpre, 1, s_p6inpre);
-    MVM_ext_register_extop(tc, "p6finddispatcher", p6finddispatcher, 2, s_p6finddispatcher);
-    MVM_ext_register_extop(tc, "p6argsfordispatcher", p6argsfordispatcher, 2, s_p6argsfordispatcher);
-    MVM_ext_register_extop(tc, "p6shiftpush", p6shiftpush, 4, s_p6shiftpush);
-    MVM_ext_register_extop(tc, "p6arrfindtypes", p6arrfindtypes, 5, s_p6arrfindtypes);
-    MVM_ext_register_extop(tc, "p6decodelocaltime", p6decodelocaltime, 2, s_p6decodelocaltime);
-    MVM_ext_register_extop(tc, "p6staticouter", p6staticouter, 2, s_p6staticouter);
-    MVM_ext_register_extop(tc, "p6invokeunder", p6invokeunder, 3, s_p6invokeunder);
+    MVM_ext_register_extop(tc, "p6init",  p6init, 0, NULL, NULL, NULL, 0);
+    MVM_ext_register_extop(tc, "p6box_i",  p6box_i, 2, s_p6box_i, NULL, p6box_i_discover, MVM_EXTOP_PURE | MVM_EXTOP_ALLOCATING);
+    MVM_ext_register_extop(tc, "p6box_n",  p6box_n, 2, s_p6box_n, NULL, p6box_n_discover, MVM_EXTOP_PURE | MVM_EXTOP_ALLOCATING);
+    MVM_ext_register_extop(tc, "p6box_s",  p6box_s, 2, s_p6box_s, NULL, p6box_s_discover, MVM_EXTOP_PURE | MVM_EXTOP_ALLOCATING);
+    MVM_ext_register_extop(tc, "p6settypes",  p6settypes, 1, s_p6settypes, NULL, NULL, 0);
+    MVM_ext_register_extop(tc, "p6bool",  p6bool, 2, s_p6bool, NULL, p6bool_discover, MVM_EXTOP_PURE);
+    MVM_ext_register_extop(tc, "p6scalarfromdesc",  p6scalarfromdesc, 2, s_p6scalarfromdesc, NULL, p6scalarfromdesc_discover, MVM_EXTOP_PURE | MVM_EXTOP_ALLOCATING);
+    MVM_ext_register_extop(tc, "p6recont_ro",  p6recont_ro, 2, s_p6recont_ro, NULL, NULL, MVM_EXTOP_PURE);
+    MVM_ext_register_extop(tc, "p6var",  p6var, 2, s_p6var, NULL, NULL, MVM_EXTOP_PURE | MVM_EXTOP_ALLOCATING);
+    MVM_ext_register_extop(tc, "p6reprname",  p6reprname, 2, s_p6reprname, NULL, p6reprname_discover, MVM_EXTOP_PURE | MVM_EXTOP_ALLOCATING);
+    MVM_ext_register_extop(tc, "p6decontrv",  p6decontrv, 2, s_p6decontrv, p6decontrv_spesh, NULL, MVM_EXTOP_PURE);
+    MVM_ext_register_extop(tc, "p6routinereturn",  p6routinereturn, 2, s_p6routinereturn, NULL, NULL, MVM_EXTOP_INVOKISH);
+    MVM_ext_register_extop(tc, "p6capturelex",  p6capturelex, 2, s_p6capturelex, NULL, NULL, 0);
+    MVM_ext_register_extop(tc, "p6capturelexwhere",  p6capturelexwhere, 2, s_p6capturelexwhere, NULL, NULL, 0);
+    MVM_ext_register_extop(tc, "p6getouterctx", p6getouterctx, 2, s_p6getouterctx, NULL, NULL, MVM_EXTOP_PURE | MVM_EXTOP_ALLOCATING);
+    MVM_ext_register_extop(tc, "p6captureouters", p6captureouters, 2, s_p6captureouters, NULL, NULL, 0);
+    MVM_ext_register_extop(tc, "p6stateinit", p6stateinit, 1, s_p6stateinit, NULL, NULL, 0);
+    MVM_ext_register_extop(tc, "p6setfirstflag", p6setfirstflag, 2, s_p6setfirstflag, NULL, NULL, 0);
+    MVM_ext_register_extop(tc, "p6takefirstflag", p6takefirstflag, 1, s_p6takefirstflag, NULL, NULL, 0);
+    MVM_ext_register_extop(tc, "p6setpre", p6setpre, 1, s_p6setpre, NULL, NULL, 0);
+    MVM_ext_register_extop(tc, "p6clearpre", p6clearpre, 1, s_p6clearpre, NULL, NULL, 0);
+    MVM_ext_register_extop(tc, "p6inpre", p6inpre, 1, s_p6inpre, NULL, NULL, 0);
+    MVM_ext_register_extop(tc, "p6finddispatcher", p6finddispatcher, 2, s_p6finddispatcher, NULL, NULL, MVM_EXTOP_NO_JIT);
+    MVM_ext_register_extop(tc, "p6argsfordispatcher", p6argsfordispatcher, 2, s_p6argsfordispatcher, NULL, NULL, 0);
+    MVM_ext_register_extop(tc, "p6decodelocaltime", p6decodelocaltime, 2, s_p6decodelocaltime, NULL, NULL, MVM_EXTOP_ALLOCATING);
+    MVM_ext_register_extop(tc, "p6staticouter", p6staticouter, 2, s_p6staticouter, NULL, NULL, 0);
+    MVM_ext_register_extop(tc, "p6invokeunder", p6invokeunder, 3, s_p6invokeunder, NULL, NULL, MVM_EXTOP_INVOKISH);
 }
