@@ -1,4 +1,4 @@
-module OuterModule;
+unit module OuterModule;
 use v6;
 
 use InnerModule :ALL :EXPORT;
