@@ -1,4 +1,4 @@
-module Test::Util;
+unit module Test::Util;
 
 use Test;
 
@@ -99,7 +99,7 @@ sub get_out( Str $code, Str $input?, :@args, :@compiler-args) is export {
         $clobber( "$fnbase.in", $input );
         $clobber( "$fnbase.code", $code ) if defined $code;
 
-        my $perl6 = $*EXECUTABLE_NAME;
+        my $perl6 = ~$*EXECUTABLE;
         my $cmd = $perl6 ~~ m:i/niecza/ ?? "mono $perl6 " !! "$perl6 ";
         $perl6 ~~ s{^perl6} = './perl6';
         $cmd = $perl6 ~ ' ';
