@@ -27,11 +27,11 @@ sub replace {
     my ($find, $fmt, $f) = @_;
     $tu =~ s
         {^( *)#BEGIN $find\n.+?^ *#END $find\n}
-        { sprintf "$1#BEGIN $find\n$1<\n%s\n$1>\n$1#END $find\n", join "\n",
-              map { sprintf "%s$fmt", $1, $f->(@$_) } @dates }ems
+        { sprintf "$1#BEGIN $find\n$1(\n%s\n$1)\n$1#END $find\n", join "\n",
+              map { sprintf "%s$fmt,", $1, $f->(@$_) } @dates }ems
       or die "Couldn't replace $find";
 }
-replace 'leap-second-dates', '%d-%02d-%02d', sub { @_ };
+replace 'leap-second-dates', "'%d-%02d-%02d'", sub { @_ };
 replace 'leap-second-posix', '%10d', sub {
     my ($y, $m, $d) = @_;
     1 + timegm 59, 59, 23, $d, $m - 1, $y - 1900;
